<?php
/***********************************************************************

  Copyright (C) 2002-2007  Rickard Andersson (rickard@punbb.org)

  This file is part of PunBB.

  PunBB is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 2 of the License,
  or (at your option) any later version.

  PunBB is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
  MA  02111-1307  USA

************************************************************************/


// This script updates the forum database to the latest version.
// Copy this file to the forum root directory and run it. Then remove it from
// the root directory.

define('UPDATE_TO', '1.3 SVN');

// The number of items to process per pageview (lower this if the update script times out during UTF-8 conversion)
define('PER_PAGE', 300);


// Make sure we are running at least PHP 4.3.0
if (!function_exists('version_compare') || version_compare(PHP_VERSION, '4.3.0', '<'))
	exit('You are running PHP version '.PHP_VERSION.'. '.UPDATE_TO.' requires at least PHP 4.3.0 to run properly. You must upgrade your PHP installation before you can continue.');


define('PUN_ROOT', './');
@include PUN_ROOT.'config.php';

// If PUN isn't defined, config.php is missing or corrupt or we are outside the root directory
if (!defined('PUN'))
	exit('This file must be run from the forum root directory.');

// Enable debug mode
define('PUN_DEBUG', 1);

// Turn on full PHP error reporting
error_reporting(E_ALL);

// Turn off magic_quotes_runtime
set_magic_quotes_runtime(0);

// Turn off PHP time limit
@set_time_limit(0);

// If the cache directory is not specified, we use the default setting
if (!defined('PUN_CACHE_DIR'))
	define('PUN_CACHE_DIR', PUN_ROOT.'cache/');

// Load the functions script
require PUN_ROOT.'include/functions.php';


// Load DB abstraction layer and try to connect
require PUN_ROOT.'include/dblayer/common_db.php';


// Check current version
$result = $db->query('SELECT conf_value FROM '.$db->prefix.'config WHERE conf_name=\'o_cur_version\'');
if (version_compare($db->result($result), '1.2', '<'))
	error('Version mismatch. The database \''.$db_name.'\' doesn\'t seem to be running a PunBB database schema supported by this update script.', __FILE__, __LINE__);


// Get the forum config
$result = $db->query('SELECT * FROM '.$db->prefix.'config');
while ($cur_config_item = $db->fetch_row($result))
	$pun_config[$cur_config_item[0]] = $cur_config_item[1];

// If $base_url isn't set, use o_base_url from config
if (!isset($base_url))
	$base_url = $pun_config['o_base_url'];

// There's no $pun_user, but we need the style element
$pun_user['style'] = $pun_config['o_default_style'];


//
// Determines whether $str is UTF-8 encoded or not
//
function seems_utf8($str)
{
	$str_len = strlen($str);
	for ($i = 0; $i < $str_len; ++$i)
	{
		if (ord($str[$i]) < 0x80) continue; # 0bbbbbbb
		else if ((ord($str[$i]) & 0xE0) == 0xC0) $n=1; # 110bbbbb
		else if ((ord($str[$i]) & 0xF0) == 0xE0) $n=2; # 1110bbbb
		else if ((ord($str[$i]) & 0xF8) == 0xF0) $n=3; # 11110bbb
		else if ((ord($str[$i]) & 0xFC) == 0xF8) $n=4; # 111110bb
		else if ((ord($str[$i]) & 0xFE) == 0xFC) $n=5; # 1111110b
		else return false; # Does not match any model

		for ($j = 0; $j < $n; ++$j) # n bytes matching 10bbbbbb follow ?
		{
			if ((++$i == strlen($str)) || ((ord($str[$i]) & 0xC0) != 0x80))
				return false;
		}
	}

	return true;
}


//
// Translates the number from an HTML numeric entity into an UTF-8 character
//
function dcr2utf8($src)
{
	$dest = '';
	if ($src < 0)
		return false;
	else if ($src <= 0x007f)
		$dest .= chr($src);
	else if ($src <= 0x07ff)
	{
		$dest .= chr(0xc0 | ($src >> 6));
		$dest .= chr(0x80 | ($src & 0x003f));
	}
	else if ($src == 0xFEFF)
	{
		// nop -- zap the BOM
	}
	else if ($src >= 0xD800 && $src <= 0xDFFF)
	{
		// found a surrogate
		return false;
	}
	else if ($src <= 0xffff)
	{
		$dest .= chr(0xe0 | ($src >> 12));
		$dest .= chr(0x80 | (($src >> 6) & 0x003f));
		$dest .= chr(0x80 | ($src & 0x003f));
	}
	else if ($src <= 0x10ffff)
	{
		$dest .= chr(0xf0 | ($src >> 18));
		$dest .= chr(0x80 | (($src >> 12) & 0x3f));
		$dest .= chr(0x80 | (($src >> 6) & 0x3f));
		$dest .= chr(0x80 | ($src & 0x3f));
	}
	else
	{
		// out of range
		return false;
	}

	return $dest;
}


//
// Attemts to convert $str from $old_charset to UTF-8. Also converts HTML entities (including numeric entities) to UTF-8 characters.
//
function convert_to_utf8(&$str, $old_charset)
{
	if ($str == '')
		return false;

	$save = $str;

	// Replace literal entities (for non-UTF-8 compliant html_entity_encode)
	if (version_compare(PHP_VERSION, '5.0.0', '<') && $old_charset == 'ISO-8859-1' || $old_charset == 'ISO-8859-15')
		$str = html_entity_decode($str, ENT_QUOTES, $old_charset);

	if (!seems_utf8($str))
	{
		if ($old_charset == 'ISO-8859-1')
			$str = utf8_encode($str);
		else if (function_exists('iconv'))
			$str = iconv($old_charset, 'UTF-8', $str);
		else if (function_exists('mb_convert_encoding'))
			$str = mb_convert_encoding($str, 'UTF-8', $old_charset);
	}

	// Replace literal entities (for UTF-8 compliant html_entity_encode)
	if (version_compare(PHP_VERSION, '5.0.0', '>='))
		$str = html_entity_decode($str, ENT_QUOTES, 'UTF-8');

	// Replace numeric entities
	$str = preg_replace_callback('/&#([0-9]+);/', create_function('$s', 'return dcr2utf8($s[1]);'), $str);
	$str = preg_replace_callback('/&#x([a-f0-9]+);/i', create_function('$s', 'return dcr2utf8(hexdec($s[1]));'), $str);

	return ($save != $str);
}


//
// Tries to determine whether post data in the database is UTF-8 encoded or not
//
function db_seems_utf8()
{
	global $db_type, $db;

	$seems_utf8 = true;

	$result = $db->query('SELECT MIN(id), MAX(id) FROM '.$db->prefix.'posts') or error(__FILE__, __LINE__);
	list($min_id, $max_id) = $db->fetch_row($result);

	// Get a random soup of data and check if it appears to be UTF-8
	for ($i = 0; $i < 50; ++$i)
	{
		$id = ($i == 0) ? $min_id : (($i == 1) ? $max_id : rand($min_id, $max_id));

		$result = $db->query('SELECT p.message, p.poster, t.subject, f.forum_name FROM '.$db->prefix.'posts AS p INNER JOIN '.$db->prefix.'topics AS t ON t.id = p.topic_id INNER JOIN '.$db->prefix.'forums AS f ON f.id = t.forum_id WHERE p.id>='.$id.' LIMIT 1') or error(__FILE__, __LINE__);
		$temp = $db->fetch_row($result);

		if (!seems_utf8($temp[0].$temp[1].$temp[2].$temp[3]))
		{
			$seems_utf8 = false;
			break;
		}
	}

	return $seems_utf8;
}


header('Content-type: text/html; charset=utf-8');


$stage = isset($_GET['stage']) ? $_GET['stage'] : '';
$old_charset = isset($_GET['req_old_charset']) ? str_replace('ISO8859', 'ISO-8859', strtoupper($_GET['req_old_charset'])) : 'ISO-8859-1';
$start_at = isset($_GET['start_at']) ? intval($_GET['start_at']) : 0;
$query_str = '';

switch ($stage)
{
	// Show form
	case '':
		$db_seems_utf8 = db_seems_utf8();

?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">

<html dir="ltr">
<head>
<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
<title>PunBB Database Update</title>
<?php

		// Include the stylesheets
		require PUN_ROOT.'style/'.$pun_user['style'].'/'.$pun_user['style'].'.php';

?>
<script type="text/javascript" src="<?php echo $base_url ?>/include/js/common.js"></script>
</head>
<body>

<div id="pun-update" class="pun pun-user" >

<div id="pun-main1" class="a-section a-main">

	<h1 class="main-title"><span>PunBB Database Update</span></h1>

	<div class="b-section b-dataform">
		<div class="c-section c-form">
			<form method="get" accept-charset="utf-8" action="<?php echo $_SERVER['PHP_SELF'] ?>">
				<div class="panel">
					<input type="hidden" name="stage" value="start" />
					<div class="form-intro">
						<h2 class="prefix"><span>Database update: </span></h2>
						<p>This script will update your PunBB forum database. The update procedure might take anything from a few seconds to a few minutes (or in extreme cases, hours) depending on the speed of the server, the size of the forum database and the number of changes required. <strong>Don't forget to make a backup of the database before continuing.</strong></p>
						<p>Did you read the update instructions in the documentation? If not, start there.</p>
					</div>
<?php

		if (!$db_seems_utf8 || isset($_GET['force']))
		{

?>
					<p id="req-msg" class="req-warn">All fields marked <em class="req-text">*<span> (Required)</span></em> must be completed before submission of this form can be accepted.</p>
					<fieldset class="checkset" id="fs1">
						<legend><span>Charset conversion</span></legend>
						<p class="field">
							<span class="input"><input type="checkbox" id="fs1f1" name="convert_charset" value="1" checked="checked" /> <label for="fs1f1">Perform database charset conversion.</label><br /> <em class="field-info">When enabled, this update script will, after it has made the required structural changes to the database, convert all text in the database from the current character set to UTF-8. This conversion is required if you're upgrading from PunBB 1.2 and you are not currently using an UTF-8 language pack.</em></span><br />
						</p>
						<p class="field required">
							<label for="fs1f2">Current character set <em class="req-text">* <span>(Required)</span></em></label><br />
							<span class="input"><input type="text" id="fs1f2" name="req_old_charset" size="12" maxlength="20" value="ISO-8859-1" /><br /> <em class="field-info">The current character set. If the primary language in your forum is English, you can leave this at the default value. However, if your forum is non-English, you should enter the character set of the primary language pack used in the forum.</em></span><br />
						</p>
					</fieldset>
<?php

			if (!function_exists('iconv') && !function_exists('mb_convert_encoding'))
			{

?>
					<div class="form-warn">
						<p><strong>Important!</strong> PunBB has detected that this PHP environment does not have support for the encoding mechanisms required to do UTF-8 conversion from character sets other than ISO-8859-1. What this means is that if the current character set is not ISO-8859-1, PunBB won't be able to convert your forum database to UTF-8 and you will have to do it manually. Instructions for doing manual charset conversion can be found in the <a href="http://docs.punbb.org/install.html#charset_conversion" onclick="window.open(this.href); return false;" title="Page opens in a new window if javascript is enabled.">update instructions</a>.</p>
					</div>
<?php

			}
		}

		if ($db_seems_utf8 && !isset($_GET['force']))
		{

?>
					<div class="form-warn">
						<p><strong>Important!</strong> Based on a random selection of 50 posts, topic subjects, usernames and forum names from the database, it appears as if text in the database is currently UTF-8 encoded. This is a good thing. Based on this, the update process will not attempt to do charset conversion. If you have reason to believe that the charset conversion is required nonetheless, you can <a href="<?php echo $_SERVER['PHP_SELF'] ?>?force=1">force the conversion to run</a>.</p>
					</div>
<?php

		}

?>
					<p class="formsubmit"><input type="submit" class="button" name="start" value="Start update" /></p>
			</div>
		</form>
		</div>
	</div>

</div>
</div>

</body>
</html>
<?php

		break;


	// Start by updating the database structure
	case 'start':
		// Add the extensions table if it doesn't already exist
		if (!$db->table_exists($db->prefix.'extensions'))
		{
			switch ($db_type)
			{
				case 'mysql':
				case 'mysqli':
					$sql = 'CREATE TABLE '.$db->prefix."extensions (
							id VARCHAR(50) NOT NULL DEFAULT '',
							title VARCHAR(255) NOT NULL DEFAULT '',
							version VARCHAR(25) NOT NULL DEFAULT '',
							description TEXT,
							author VARCHAR(50) NOT NULL DEFAULT '',
							uninstall TEXT,
							uninstall_notes TEXT,
							PRIMARY KEY(id)
							) TYPE=MyISAM;";
					break;

				case 'pgsql':
					$sql = 'CREATE TABLE '.$db->prefix."extensions (
							id VARCHAR(50) NOT NULL DEFAULT '',
							title VARCHAR(255) NOT NULL DEFAULT '',
							version VARCHAR(25) NOT NULL DEFAULT '',
							description TEXT,
							author VARCHAR(50) NOT NULL DEFAULT '',
							uninstall TEXT,
							uninstall_notes TEXT,
							PRIMARY KEY(id)
							)";
					break;

				case 'sqlite':
					$sql = 'CREATE TABLE '.$db->prefix."extensions (
							id VARCHAR(50) NOT NULL DEFAULT '',
							title VARCHAR(255) NOT NULL DEFAULT '',
							version VARCHAR(25) NOT NULL DEFAULT '',
							description TEXT,
							author VARCHAR(50) NOT NULL DEFAULT '',
							uninstall TEXT,
							uninstall_notes TEXT,
							PRIMARY KEY(id)
							)";
					break;
			}

			$db->query($sql) or error(__FILE__, __LINE__);
		}

		// Add uninstall notes field to extensions to update SVN boards
		$db->add_field($db->prefix.'extensions', 'uninstall_notes', 'TEXT', true, null, 'uninstall');

		// Add the extension_hooks table if it doesn't already exist
		if (!$db->table_exists($db->prefix.'extension_hooks'))
		{
			switch ($db_type)
			{
				case 'mysql':
				case 'mysqli':
					$sql = 'CREATE TABLE '.$db->prefix."extension_hooks (
							id VARCHAR(50) NOT NULL DEFAULT '',
							extension_id VARCHAR(50) NOT NULL DEFAULT '',
							code TEXT,
							installed INT(10) UNSIGNED NOT NULL DEFAULT 0,
							PRIMARY KEY(id, extension_id)
							) TYPE=MyISAM;";
					break;

				case 'pgsql':
					$sql = 'CREATE TABLE '.$db->prefix."extension_hooks (
							id VARCHAR(50) NOT NULL DEFAULT '',
							extension_id VARCHAR(50) NOT NULL DEFAULT '',
							code TEXT,
							installed INT NOT NULL DEFAULT 0,
							PRIMARY KEY(id, extension_id)
							)";
					break;

				case 'sqlite':
					$sql = 'CREATE TABLE '.$db->prefix."extension_hooks (
							id VARCHAR(50) NOT NULL DEFAULT '',
							extension_id VARCHAR(50) NOT NULL DEFAULT '',
							code TEXT,
							installed INTEGER NOT NULL DEFAULT 0,
							PRIMARY KEY(id, extension_id)
							)";
					break;
			}

			$db->query($sql) or error(__FILE__, __LINE__);
		}

		if ($db_type == 'mysql' || $db_type == 'mysqli')
		{
			// Make all e-mail fields VARCHAR(80)
			$db->query('ALTER TABLE '.$db->prefix.'bans CHANGE email email VARCHAR(80)') or error(__FILE__, __LINE__);
			$db->query('ALTER TABLE '.$db->prefix.'posts CHANGE poster_email poster_email VARCHAR(80)') or error(__FILE__, __LINE__);
			$db->query('ALTER TABLE '.$db->prefix.'users CHANGE email email VARCHAR(80)') or error(__FILE__, __LINE__);
			$db->query('ALTER TABLE '.$db->prefix.'users CHANGE jabber jabber VARCHAR(80)') or error(__FILE__, __LINE__);
			$db->query('ALTER TABLE '.$db->prefix.'users CHANGE msn msn VARCHAR(80)') or error(__FILE__, __LINE__);
			$db->query('ALTER TABLE '.$db->prefix.'users CHANGE activate_string activate_string VARCHAR(80)') or error(__FILE__, __LINE__);

			// Remove NOT NULL from TEXT fields for consistency. See http://dev.punbb.org/changeset/596
			$db->query('ALTER TABLE '.$db->prefix.'posts CHANGE message message TEXT') or error(__FILE__, __LINE__);
			$db->query('ALTER TABLE '.$db->prefix.'reports CHANGE message message TEXT') or error(__FILE__, __LINE__);
		}

		// Add the DST option to the users table
		$db->add_field($db->prefix.'users', 'dst', 'TINYINT(1)', false, 0, 'timezone');

		// Add the salt field to the users table
		$db->add_field($db->prefix.'users', 'salt', 'VARCHAR(12)', true, null, 'password');

		// Add the CSRF token field to the online table
		$db->add_field($db->prefix.'online', 'csrf_token', 'VARCHAR(40)', false, '', null);

		// Drop use_avatar column from users table
		$db->drop_field($db->prefix.'users', 'use_avatar');

		// Make sure we have o_additional_navlinks (was added in 1.2.1)
		if (!array_key_exists('o_additional_navlinks', $pun_config))
			$db->query('INSERT INTO '.$db->prefix.'config (conf_name, conf_value) VALUES(\'o_additional_navlinks\', \'\')') or error(__FILE__, __LINE__);

		// Insert new config options o_sef
		if (!array_key_exists('o_sef', $pun_config))
			$db->query('INSERT INTO '.$db->prefix.'config (conf_name, conf_value) VALUES(\'o_sef\', \'Default\')') or error(__FILE__, __LINE__);

		// Insert new config option o_topic_views
		if (!array_key_exists('o_topic_views', $pun_config))
			$db->query('INSERT INTO '.$db->prefix.'config (conf_name, conf_value) VALUES(\'o_topic_views\', \'1\')') or error(__FILE__, __LINE__);

		// Insert new config option o_signatures
		if (!array_key_exists('o_signatures', $pun_config))
			$db->query('INSERT INTO '.$db->prefix.'config (conf_name, conf_value) VALUES(\'o_signatures\', \'1\')') or error(__FILE__, __LINE__);

		// Insert new config option o_smtp_ssl
		if (!array_key_exists('o_smtp_ssl', $pun_config))
			$db->query('INSERT INTO '.$db->prefix.'config (conf_name, conf_value) VALUES(\'o_smtp_ssl\', \'0\')') or error(__FILE__, __LINE__);

		// Insert new config option o_check_for_updates
		if (!array_key_exists('o_check_for_updates', $pun_config))
		{
			$check_for_updates = (function_exists('curl_init') || function_exists('fsockopen') || in_array(strtolower(@ini_get('allow_url_fopen')), array('on', 'true', '1'))) ? 1 : 0;
			$db->query('INSERT INTO '.$db->prefix.'config (conf_name, conf_value) VALUES(\'o_check_for_updates\', \''.$check_for_updates.'\')') or error(__FILE__, __LINE__);
		}

		// Server timezone is now simply the default timezone
		if (!array_key_exists('o_default_timezone', $pun_config))
			$db->query('UPDATE '.$db->prefix.'config SET conf_name=\'o_default_timezone\' WHERE conf_name=\'o_server_timezone\'') or error(__FILE__, __LINE__);

		// Increase visit timeout to 30 minutes (only if it hasn't been changed from the default)
		if ($pun_config['o_timeout_visit'] == '600')
			$db->query('UPDATE '.$db->prefix.'config SET conf_value=\'1800\' WHERE conf_name=\'o_timeout_visit\'') or error(__FILE__, __LINE__);

		// Remove obsolete g_post_polls permission from groups table
		if ($db->field_exists($db->prefix.'groups', 'g_post_polls') && $db_type != 'sqlite')	// No DROP column in SQLite
			$db->query('ALTER TABLE '.$db->prefix.'groups DROP g_post_polls') or error(__FILE__, __LINE__);

		// Make room for multiple moderator groups
		if (!$db->field_exists($db->prefix.'groups', 'g_moderator'))
		{
			// Add g_moderator column to groups table
			$db->add_field($db->prefix.'groups', 'g_moderator', 'TINYINT(1)', false, 0, 'g_user_title');

			// Give the moderator group moderator privileges
			$db->query('UPDATE '.$db->prefix.'groups SET g_moderator=1 WHERE g_id=2') or error(__FILE__, __LINE__);

			// Shuffle the group IDs around a bit
			$result = $db->query('SELECT MAX(g_id)+1 FROM '.$db->prefix.'groups') or error(__FILE__, __LINE__);
			$temp_id = $db->result($result);

			$db->query('UPDATE '.$db->prefix.'groups SET g_id='.$temp_id.' WHERE g_id=2') or error(__FILE__, __LINE__);
			$db->query('UPDATE '.$db->prefix.'groups SET g_id=2 WHERE g_id=3') or error(__FILE__, __LINE__);
			$db->query('UPDATE '.$db->prefix.'groups SET g_id=3 WHERE g_id=4') or error(__FILE__, __LINE__);
			$db->query('UPDATE '.$db->prefix.'groups SET g_id=4 WHERE g_id='.$temp_id) or error(__FILE__, __LINE__);

			$db->query('UPDATE '.$db->prefix.'users SET group_id='.$temp_id.' WHERE group_id=2') or error(__FILE__, __LINE__);
			$db->query('UPDATE '.$db->prefix.'users SET group_id=2 WHERE group_id=3') or error(__FILE__, __LINE__);
			$db->query('UPDATE '.$db->prefix.'users SET group_id=3 WHERE group_id=4') or error(__FILE__, __LINE__);
			$db->query('UPDATE '.$db->prefix.'users SET group_id=4 WHERE group_id='.$temp_id) or error(__FILE__, __LINE__);

			$db->query('UPDATE '.$db->prefix.'forum_perms SET group_id='.$temp_id.' WHERE group_id=2') or error(__FILE__, __LINE__);
			$db->query('UPDATE '.$db->prefix.'forum_perms SET group_id=2 WHERE group_id=3') or error(__FILE__, __LINE__);
			$db->query('UPDATE '.$db->prefix.'forum_perms SET group_id=3 WHERE group_id=4') or error(__FILE__, __LINE__);
			$db->query('UPDATE '.$db->prefix.'forum_perms SET group_id=4 WHERE group_id='.$temp_id) or error(__FILE__, __LINE__);

			// Update the default usergroup if it uses the old ID for the members group
			$db->query('UPDATE '.$db->prefix.'config SET conf_value="3" WHERE conf_name="o_default_user_group" and conf_value="4"') or error(__FILE__, __LINE__);
		}

		// We need to add a unique index to avoid users having multiple rows in the online table
		if (!$db->index_exists($db->prefix.'online', $db->prefix.'online_user_id_ident_idx'))
		{
			$db->query('DELETE FROM '.$db->prefix.'online') or error(__FILE__, __LINE__);

			switch ($db_type)
			{
				case 'mysql':
				case 'mysqli':
					$db->query('ALTER TABLE '.$db->prefix.'online ADD UNIQUE INDEX '.$db->prefix.'online_user_id_ident_idx (user_id, ident)') or error(__FILE__, __LINE__);
					break;

				default:
					$db->query('CREATE UNIQUE INDEX '.$db->prefix.'online_user_id_ident_idx ON '.$db->prefix.'online(user_id, ident)') or error(__FILE__, __LINE__);
					break;
			}
		}

		// Get rid of the old search tables for mysql/mysqli
		if (($db_type == 'mysql' || $db_type == 'mysqli') && $db->table_exists($db->prefix.'search_cache') && $db->table_exists($db->prefix.'search_matches') && $db->table_exists($db->prefix.'search_words'))
			$db->query('DROP TABLE '.$db->prefix.'search_cache, '.$db->prefix.'search_matches, '.$db->prefix.'search_words') or error(__FILE__, __LINE__);

		// Add fulltext indexes for mysql/mysqli
		if (($db_type == 'mysql' || $db_type == 'mysqli') && !$db->index_exists($db->prefix.'posts', $db->prefix.'posts_message_idx'))
			$db->query('ALTER TABLE '.$db->prefix.'posts ADD FULLTEXT '.$db->prefix.'posts_message_idx(message)') or error(__FILE__, __LINE__);
		if (($db_type == 'mysql' || $db_type == 'mysqli') && !$db->index_exists($db->prefix.'topics', $db->prefix.'topics_subject_idx'))
			$db->query('ALTER TABLE '.$db->prefix.'topics ADD FULLTEXT '.$db->prefix.'topics_subject_idx(subject)') or error(__FILE__, __LINE__);

		// Add an index on last_post in the topics table
		if (!$db->index_exists($db->prefix.'topics', $db->prefix.'topics_last_post_idx'))
		{
			switch ($db_type)
			{
				case 'mysql':
				case 'mysqli':
					$db->query('ALTER TABLE '.$db->prefix.'topics ADD INDEX '.$db->prefix.'topics_last_post_idx(last_post)') or error(__FILE__, __LINE__);
					break;

				default:
					$db->query('CREATE INDEX '.$db->prefix.'topics_last_post_idx ON '.$db->prefix.'topics(last_post)') or error(__FILE__, __LINE__);
					break;
			}
		}

		// Add post approval fields
		$db->add_field($db->prefix.'forums', 'approval', 'TINYINT(1)', false, 0, null);
		$db->add_field($db->prefix.'groups', 'g_posts_approved', 'TINYINT(1)', false, 0, null);
		$db->add_field($db->prefix.'posts', 'approved', 'TINYINT(1)', false, 1, null);

		// Add g_view_users field to groups table
		$db->add_field($db->prefix.'groups', 'g_view_users', 'TINYINT(1)', false, 1, 'g_read_board');

		// Add the time/date format settings to the user table
		$db->add_field($db->prefix.'users', 'time_format', 'INT(10)', false, 0, 'dst');
		$db->add_field($db->prefix.'users', 'date_format', 'INT(10)', false, 0, 'dst');

		// Add the auto notify/subscription option to the users table
		$db->add_field($db->prefix.'users', 'auto_notify', 'TINYINT(1)', false, 0, 'notify_with_post');

		// Add the first_post_id column to the topics table
		if (!$db->field_exists($db->prefix.'topics', 'first_post_id'))
		{
			switch ($db_type)
			{
				case 'mysql':
				case 'mysqli':
					$db->query('ALTER TABLE '.$db->prefix.'topics ADD first_post_id INT(10) UNSIGNED NOT NULL DEFAULT 0 AFTER posted') or error(__FILE__, __LINE__);
					$db->query('ALTER TABLE '.$db->prefix.'topics ADD INDEX '.$db->prefix.'topics_first_post_id_idx(first_post_id)') or error(__FILE__, __LINE__);
					break;

				default:
					$db->add_field($db->prefix.'topics', 'first_post_id', 'INT', false, 0, null);
					$db->query('CREATE INDEX '.$db->prefix.'topics_first_post_id_idx ON '.$db->prefix.'topics(first_post_id)') or error(__FILE__, __LINE__);
					break;
			}

			// Now that we've added the column and indexed it, we need to give it correct data
			$result = $db->query('SELECT min(id) AS first_post, topic_id FROM '.$db->prefix.'posts GROUP BY topic_id') or error(__FILE__, __LINE__);

			while ($cur_post = $db->fetch_assoc($result))
			{
				$db->query('UPDATE '.$db->prefix.'topics SET first_post_id = '.$cur_post['first_post'].' WHERE id = '.$cur_post['topic_id']) or error(__FILE__, __LINE__);
			}
		}

		// Move any users with the old unverified status to their new group
		$db->query('UPDATE '.$db->prefix.'users SET group_id=0 WHERE group_id=32000') or error(__FILE__, __LINE__);

		// Add the ban_creator column to the bans table
		if (!$db->field_exists($db->prefix.'bans', 'ban_creator'))
		{
			switch ($db_type)
			{
				case 'mysql':
				case 'mysqli':
					$db->query('ALTER TABLE '.$db->prefix.'bans ADD ban_creator INT(10) UNSIGNED NOT NULL DEFAULT 0') or error(__FILE__, __LINE__);
					break;

				default:
					$db->add_field($db->prefix.'bans', 'ban_creator', 'INT', false, 0, null);
					break;
			}
		}

		// Finally, we update the version number
		$db->query('UPDATE '.$db->prefix.'config SET conf_value=\''.UPDATE_TO.'\' WHERE conf_name=\'o_cur_version\'') or error(__FILE__, __LINE__);

		// This feels like a good time to synchronize the forums
		$result = $db->query('SELECT id FROM '.$db->prefix.'forums') or error(__FILE__, __LINE__);
		while ($row = $db->fetch_row($result))
			sync_forum($row[0]);

		// We'll empty the search cache table as well (using DELETE FROM since SQLite does not support TRUNCATE TABLE)
		if ($db_type != 'mysql' && $db_type != 'mysqli')
			$db->query('DELETE FROM '.$db->prefix.'search_cache') or error(__FILE__, __LINE__);

		// Empty the PHP cache
		$d = dir(PUN_CACHE_DIR);
		while (($entry = $d->read()) !== false)
		{
			if (substr($entry, strlen($entry)-4) == '.php')
				@unlink(PUN_CACHE_DIR.$entry);
		}
		$d->close();

		// Should we do charset conversion or not?
		if (isset($_GET['convert_charset']))
			$query_str = '?stage=conv_misc&req_old_charset='.$old_charset.'&req_per_page='.PER_PAGE;
		else
			$query_str = '?stage=finish';
		break;


	// Convert config, categories, forums, groups, ranks and censor words
	case 'conv_misc':
		// Convert config
		echo 'Converting configuration …'."<br />\n";
		foreach ($pun_config as $conf_name => $conf_value)
		{
			if (convert_to_utf8($conf_value, $old_charset))
				$db->query('UPDATE '.$db->prefix.'config SET conf_value=\''.$db->escape($conf_value).'\' WHERE conf_name=\''.$conf_name.'\'') or error(__FILE__, __LINE__);
		}

		// Convert categories
		echo 'Converting categories …'."<br />\n";
		$result = $db->query('SELECT id, cat_name FROM '.$db->prefix.'categories ORDER BY id') or error(__FILE__, __LINE__);
		while ($cur_item = $db->fetch_assoc($result))
		{
			if (convert_to_utf8($cur_item['cat_name'], $old_charset))
				$db->query('UPDATE '.$db->prefix.'categories SET cat_name=\''.$db->escape($cur_item['cat_name']).'\' WHERE id='.$cur_item['id']) or error(__FILE__, __LINE__);
		}

		// Convert forums
		echo 'Converting forums …'."<br />\n";
		$result = $db->query('SELECT id, forum_name, forum_desc FROM '.$db->prefix.'forums ORDER BY id') or error(__FILE__, __LINE__);
		while ($cur_item = $db->fetch_assoc($result))
		{
			if (convert_to_utf8($cur_item['forum_name'], $old_charset) | convert_to_utf8($cur_item['forum_desc'], $old_charset))
			{
				$cur_item['forum_desc'] = $cur_item['forum_desc'] != '' ? '\''.$db->escape($cur_item['forum_desc']).'\'' : 'NULL';

				$db->query('UPDATE '.$db->prefix.'forums SET forum_name=\''.$db->escape($cur_item['forum_name']).'\', forum_desc='.$cur_item['forum_desc'].' WHERE id='.$cur_item['id']) or error(__FILE__, __LINE__);
			}
		}

		// Convert groups
		echo 'Converting groups …'."<br />\n";
		$result = $db->query('SELECT g_id, g_title, g_user_title FROM '.$db->prefix.'groups ORDER BY g_id') or error(__FILE__, __LINE__);
		while ($cur_item = $db->fetch_assoc($result))
		{
			if (convert_to_utf8($cur_item['g_title'], $old_charset) | convert_to_utf8($cur_item['g_user_title'], $old_charset))
			{
				$cur_item['g_user_title'] = $cur_item['g_user_title'] != '' ? '\''.$db->escape($cur_item['g_user_title']).'\'' : 'NULL';

				$db->query('UPDATE '.$db->prefix.'groups SET g_title=\''.$db->escape($cur_item['g_title']).'\', g_user_title='.$cur_item['g_user_title'].' WHERE g_id='.$cur_item['g_id']) or error(__FILE__, __LINE__);
			}
		}

		// Convert ranks
		echo 'Converting ranks …'."<br />\n";
		$result = $db->query('SELECT id, rank FROM '.$db->prefix.'ranks ORDER BY id') or error(__FILE__, __LINE__);
		while ($cur_item = $db->fetch_assoc($result))
		{
			if (convert_to_utf8($cur_item['rank'], $old_charset))
				$db->query('UPDATE '.$db->prefix.'ranks SET rank=\''.$db->escape($cur_item['rank']).'\' WHERE id='.$cur_item['id']) or error(__FILE__, __LINE__);
		}

		// Convert censor words
		echo 'Converting censor words …'."<br />\n";
		$result = $db->query('SELECT id, search_for, replace_with FROM '.$db->prefix.'censoring ORDER BY id') or error(__FILE__, __LINE__);
		while ($cur_item = $db->fetch_assoc($result))
		{
			if (convert_to_utf8($cur_item['search_for'], $old_charset) | convert_to_utf8($cur_item['replace_with'], $old_charset))
				$db->query('UPDATE '.$db->prefix.'censoring SET search_for=\''.$db->escape($cur_item['search_for']).'\', replace_with=\''.$db->escape($cur_item['replace_with']).'\' WHERE id='.$cur_item['id']) or error(__FILE__, __LINE__);
		}

		$query_str = '?stage=conv_reports&req_old_charset='.$old_charset.'&req_per_page='.PER_PAGE;
		break;


	// Convert reports
	case 'conv_reports':
		// Determine where to start
		if ($start_at == 0)
		{
			// Get the first report ID from the db
			$result = $db->query('SELECT id FROM '.$db->prefix.'reports ORDER BY id LIMIT 1') or error(__FILE__, __LINE__);
			if ($db->num_rows($result))
				$start_at = $db->result($result);
		}
		$end_at = $start_at + PER_PAGE;

		// Fetch reports to process this cycle
		$result = $db->query('SELECT id, message FROM '.$db->prefix.'reports WHERE id>='.$start_at.' AND id<'.$end_at.' ORDER BY id') or error(__FILE__, __LINE__);
		while ($cur_item = $db->fetch_assoc($result))
		{
			echo 'Converting report '.$cur_item['id'].' …<br />'."\n";
			if (convert_to_utf8($cur_item['message'], $old_charset))
				$db->query('UPDATE '.$db->prefix.'reports SET message=\''.$db->escape($cur_item['message']).'\' WHERE id='.$cur_item['id']) or error(__FILE__, __LINE__);
		}

		// Check if there is more work to do
		$result = $db->query('SELECT id FROM '.$db->prefix.'reports WHERE id>='.$end_at) or error(__FILE__, __LINE__);
		if ($db->num_rows($result))
			$query_str = '?stage=conv_reports&req_old_charset='.$old_charset.'&req_per_page='.PER_PAGE.'&start_at='.$end_at;
		else
			$query_str = '?stage=conv_users&req_old_charset='.$old_charset.'&req_per_page='.PER_PAGE;
		break;


	// Convert users
	case 'conv_users':
		// Determine where to start
		if ($start_at == 0)
			$start_at = 2;

		$end_at = $start_at + PER_PAGE;

		// Fetch users to process this cycle
		$result = $db->query('SELECT id, username, title, realname, location, signature, admin_note FROM '.$db->prefix.'users WHERE id>='.$start_at.' AND id<'.$end_at.' ORDER BY id') or error(__FILE__, __LINE__);
		while ($cur_item = $db->fetch_assoc($result))
		{
			echo 'Converting user '.$cur_item['id'].' …<br />'."\n";
			if (convert_to_utf8($cur_item['username'], $old_charset) | convert_to_utf8($cur_item['title'], $old_charset) | convert_to_utf8($cur_item['realname'], $old_charset) | convert_to_utf8($cur_item['location'], $old_charset) | convert_to_utf8($cur_item['signature'], $old_charset) | convert_to_utf8($cur_item['admin_note'], $old_charset))
			{
				$cur_item['title'] = $cur_item['title'] != '' ? '\''.$db->escape($cur_item['title']).'\'' : 'NULL';
				$cur_item['realname'] = $cur_item['realname'] != '' ? '\''.$db->escape($cur_item['realname']).'\'' : 'NULL';
				$cur_item['location'] = $cur_item['location'] != '' ? '\''.$db->escape($cur_item['location']).'\'' : 'NULL';
				$cur_item['signature'] = $cur_item['signature'] != '' ? '\''.$db->escape($cur_item['signature']).'\'' : 'NULL';
				$cur_item['admin_note'] = $cur_item['admin_note'] != '' ? '\''.$db->escape($cur_item['admin_note']).'\'' : 'NULL';

				$db->query('UPDATE '.$db->prefix.'users SET username=\''.$db->escape($cur_item['username']).'\', title='.$cur_item['title'].', realname='.$cur_item['realname'].', location='.$cur_item['location'].', signature='.$cur_item['signature'].', admin_note='.$cur_item['admin_note'].' WHERE id='.$cur_item['id']) or error(__FILE__, __LINE__);
			}
		}

		// Check if there is more work to do
		$result = $db->query('SELECT id FROM '.$db->prefix.'users WHERE id>='.$end_at) or error(__FILE__, __LINE__);
		if ($db->num_rows($result))
			$query_str = '?stage=conv_users&req_old_charset='.$old_charset.'&req_per_page='.PER_PAGE.'&start_at='.$end_at;
		else
			$query_str = '?stage=conv_topics&req_old_charset='.$old_charset.'&req_per_page='.PER_PAGE;
		break;


	// Convert topics
	case 'conv_topics':
		// Determine where to start
		if ($start_at == 0)
		{
			// Get the first topic ID from the db
			$result = $db->query('SELECT id FROM '.$db->prefix.'topics ORDER BY id LIMIT 1') or error(__FILE__, __LINE__);
			if ($db->num_rows($result))
				$start_at = $db->result($result);
		}
		$end_at = $start_at + PER_PAGE;

		// Fetch topics to process this cycle
		$result = $db->query('SELECT id, poster, subject, last_poster FROM '.$db->prefix.'topics WHERE id>='.$start_at.' AND id<'.$end_at.' ORDER BY id') or error(__FILE__, __LINE__);
		while ($cur_item = $db->fetch_assoc($result))
		{
			echo 'Converting topic '.$cur_item['id'].' …<br />'."\n";
			if (convert_to_utf8($cur_item['poster'], $old_charset) | convert_to_utf8($cur_item['subject'], $old_charset) | convert_to_utf8($cur_item['last_poster'], $old_charset))
				$db->query('UPDATE '.$db->prefix.'topics SET poster=\''.$db->escape($cur_item['poster']).'\', subject=\''.$db->escape($cur_item['subject']).'\', last_poster=\''.$db->escape($cur_item['last_poster']).'\' WHERE id='.$cur_item['id']) or error(__FILE__, __LINE__);
		}

		// Check if there is more work to do
		$result = $db->query('SELECT id FROM '.$db->prefix.'topics WHERE id>='.$end_at) or error(__FILE__, __LINE__);
		if ($db->num_rows($result))
			$query_str = '?stage=conv_topics&req_old_charset='.$old_charset.'&req_per_page='.PER_PAGE.'&start_at='.$end_at;
		else
			$query_str = '?stage=conv_posts&req_old_charset='.$old_charset.'&req_per_page='.PER_PAGE;
		break;


	// Convert posts
	case 'conv_posts':
		// Determine where to start
		if ($start_at == 0)
		{
			// Get the first post ID from the db
			$result = $db->query('SELECT id FROM '.$db->prefix.'posts ORDER BY id LIMIT 1') or error(__FILE__, __LINE__);
			if ($db->num_rows($result))
				$start_at = $db->result($result);
		}
		$end_at = $start_at + PER_PAGE;

		// Fetch posts to process this cycle
		$result = $db->query('SELECT id, poster, message, edited_by FROM '.$db->prefix.'posts WHERE id>='.$start_at.' AND id<'.$end_at.' ORDER BY id') or error(__FILE__, __LINE__);
		while ($cur_item = $db->fetch_assoc($result))
		{
			echo 'Converting post '.$cur_item['id'].' …<br />'."\n";
			if (convert_to_utf8($cur_item['poster'], $old_charset) | convert_to_utf8($cur_item['message'], $old_charset) | convert_to_utf8($cur_item['edited_by'], $old_charset))
			{
				$cur_item['edited_by'] = $cur_item['edited_by'] != '' ? '\''.$db->escape($cur_item['edited_by']).'\'' : 'NULL';

				$db->query('UPDATE '.$db->prefix.'posts SET poster=\''.$db->escape($cur_item['poster']).'\', message=\''.$db->escape($cur_item['message']).'\', edited_by='.$cur_item['edited_by'].' WHERE id='.$cur_item['id']) or error(__FILE__, __LINE__);
			}
		}

		// Check if there is more work to do
		$result = $db->query('SELECT id FROM '.$db->prefix.'posts WHERE id>='.$end_at) or error(__FILE__, __LINE__);
		if ($db->num_rows($result))
			$query_str = '?stage=conv_posts&req_old_charset='.$old_charset.'&req_per_page='.PER_PAGE.'&start_at='.$end_at;
		else
			$query_str = '?stage=finish';
		break;


	// Show results page
	case 'finish':
		// Drop Base URL row from database config
		if (array_key_exists('o_base_url', $pun_config))
		{
			// Generate new config file
			$new_config = "<?php\n\n\$db_type = '$db_type';\n\$db_host = '$db_host';\n\$db_name = '".addslashes($db_name)."';\n\$db_username = '".addslashes($db_username)."';\n\$db_password = '".addslashes($db_password)."';\n\$db_prefix = '".addslashes($db_prefix)."';\n\$p_connect = ".($p_connect ? 'true' : 'false').";\n\n\$base_url = '$base_url';\n\n\$cookie_name = '$cookie_name';\n\$cookie_domain = '$cookie_domain';\n\$cookie_path = '$cookie_path';\n\$cookie_secure = $cookie_secure;\n\ndefine('PUN', 1);";

			$db->query('DELETE FROM '.$db->prefix.'config WHERE conf_name=\'o_base_url\'') or error(__FILE__, __LINE__);
		}

?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">

<html dir="ltr">
<head>
<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
<title>PunBB Database Update</title>
<?php

// Include the stylesheets
require PUN_ROOT.'style/'.$pun_user['style'].'/'.$pun_user['style'].'.php';

?>
<script type="text/javascript" src="<?php echo $base_url ?>/include/js/common.js"></script>
</head>
<body>

<div id="pun-update" class="pun pun-user" >

<div id="pun-main1" class="a-section a-main">

	<h1 class="main-title"><span>PunBB Database Update completed!</span></h1>

	<div class="b-section b-dataform">
		<div class="c-section c-form">
			<div class="panel">
				<div class="form-intro">
					<p>Your forum database was updated successfully.</p>
<?php if (isset($new_config)): ?>					<p>In order to complete the process, you must now update your config.php script. <strong>Copy and paste the text in the text box below into the file called config.php in the root directory of your PunBB installation</strong>. The file already exists, so you must edit/overwrite the contents of the old file.</p>
<?php endif; ?>				</div>
<?php if (isset($new_config)): ?>				<fieldset class="checkset">
					<legend><span>New config.php contents</span></legend>
					<p class="areafield">
						<span class="input"><textarea cols="80" rows="20"><?php echo htmlspecialchars($new_config) ?></textarea></span>
					</p>
				</fieldset>
<?php endif; ?>				<div class="form-warn">
					<p><strong>Important!</strong> Once the update is completed you should remove this script from the forum root directory and follow the rest of the instructions in the update documentation.</p>
				</div>
			</div>
		</div>
	</div>

</div>
</div>

</body>
</html>
<?php

		break;
}

$db->end_transaction();
$db->close();

if ($query_str != '')
	exit('<script type="text/javascript">window.location="db_update.php'.$query_str.'"</script><br />JavaScript seems to be disabled. <a href="db_update.php'.$query_str.'">Click here to continue</a>.');
