<?php
/***********************************************************************

  Copyright (C) 2002-2007  Rickard Andersson (rickard@punbb.org)

  This file is part of PunBB.

  PunBB is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 2 of the License,
  or (at your option) any later version.

  PunBB is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
  MA  02111-1307  USA

************************************************************************/


define('PUN_ROOT', '../');
require PUN_ROOT.'include/common.php';
require PUN_ROOT.'include/common_admin.php';
require_once PUN_ROOT.'include/xml.php';


if ($pun_user['g_id'] != PUN_ADMIN)
	message($lang_common['No permission']);

// Load the admin.php language file
require PUN_ROOT.'lang/'.$pun_user['language'].'/admin.php';


// Install an extension
if (isset($_GET['install']) || isset($_GET['install_hotfix']))
{
	$id = preg_replace('/[^0-9a-z_]/', '', isset($_GET['install']) ? $_GET['install'] : $_GET['install_hotfix']);

	// Load manifest (either locally or from punbb.org updates service)
	if (isset($_GET['install']))
		$manifest = @file_get_contents(PUN_ROOT.'extensions/'.$id.'/manifest.xml');
	else
		$manifest = @end(get_remote_file('http://punbb.org/update/manifest/'.$id.'.xml', 16));

	// Parse manifest.xml into an array and validate it
	$ext_data = xml_to_array($manifest);
	$errors = validate_manifest($ext_data, $id);

	if (!empty($errors))
		message(isset($_GET['install']) ? $lang_common['Bad request'] : $lang_admin['Hotfix download failed']);

	// Setup breadcrumbs
	$pun_page['crumbs'] = array(
		array($pun_config['o_board_title'], pun_link($pun_url['index'])),
		array($lang_admin['Forum administration'], pun_link($pun_url['admin_index'])),
		array($lang_admin['Manage extensions'], pun_link($pun_url['admin_extensions'])),
		$lang_admin['Install extension']
	);

	if (isset($_POST['install_comply']))
	{
		if (version_compare(clean_version($pun_config['o_cur_version']), clean_version($ext_data['extension']['minversion']), '<'))
			message($lang_admin['PunBB minversion error']);
	
		$warnings = array();

		$new_uninstall_code = (isset($ext_data['extension']['uninstall']) && trim($ext_data['extension']['uninstall']) != '') ? '\''.$db->escape(trim($ext_data['extension']['uninstall'])).'\'' : 'NULL';

		ob_end_clean();
		// Look for notes to display later
		$install_notes['pre'] = array();
		$install_notes['post'] = array();
		$uninstall_notes['pre'] = array();
		$uninstall_notes['post'] = array();
		foreach ($ext_data['extension']['note'] as $note)
		{
			if (!isset($note['attributes']['type']))
				continue;

			if ($note['attributes']['type'] == 'install')
			{
				if (!isset($note['attributes']['timing']) || $note['attributes']['timing'] == 'pre')
					$install_notes['pre'][] = trim($note['content']);
				else if ($note['attributes']['timing'] == 'post')
					$install_notes['post'][] = trim($note['content']);
			}
			else if ($note['attributes']['type'] == 'uninstall')
			{
				if (!isset($note['attributes']['timing']) || $note['attributes']['timing'] == 'pre')
					$uninstall_notes['pre'][] = trim($note['content']);
				else if ($note['attributes']['timing'] == 'post')
					$uninstall_notes['post'][] = trim($note['content']);
			}
		}
		$uninstall_notes = serialize($uninstall_notes);

		if (!empty($install_notes['pre']) && !isset($_POST['install_notes_comply']))
		{
			define('PUN_PAGE_SECTION', 'extensions');
			define('PUN_PAGE', 'admin-extensions');
			require PUN_ROOT.'header.php';
?>

<div id="pun-main1" class="a-section a-main a-admin">

	<h1 class="main-title"><span><?php echo end($pun_page['crumbs']) ?></span></h1>

<?php echo generate_admin_menu(); ?>

	<div class="b-section b-dataform">
		<div class="c-section c-form">
			<form method="post" accept-charset="utf-8" action="<?php echo pun_link($pun_url['admin_extensions']) ?>?install=<?php echo $id ?>">
				<input type="hidden" name="csrf_token" value="<?php echo $pun_user['csrf_token'] ?>" />
				<input type="hidden" name="install_comply" value="1" />
			<div class="panel">
				<p><strong><?php echo $lang_admin['Install notes'] ?></strong></p>
				<p>
					<?php echo implode("\n\t\t\t\t</p>\n\t\t\t\t<p>\n\t\t\t\t\t", $install_notes['pre']); ?>

				</p>
				
				<p class="formsubmit"><input type="submit" name="install_notes_comply" value="<?php echo $lang_admin['Install'] ?>" /></p>
			</div>
			
			</form>
		</div>
	</div>

</div>

<?php
			require PUN_ROOT.'footer.php';
		}

		// Is this a fresh install or an upgrade?
		$result = $db->query('SELECT version FROM '.$db->prefix.'extensions WHERE id=\''.$db->escape($id).'\'') or error(__FILE__, __LINE__);
		if ($db->num_rows($result))
		{
			// $cur_version will be available to the extension install routine (to facilitate extension upgrades)
			$cur_version = $db->result($result);

			// Run the author supplied install code
			if (isset($ext_data['extension']['install']) && trim($ext_data['extension']['install']) != '')
				eval($ext_data['extension']['install']);

			$db->query('UPDATE '.$db->prefix.'extensions SET title=\''.$db->escape($ext_data['extension']['title']).'\', version=\''.$db->escape($ext_data['extension']['version']).'\', description=\''.$db->escape($ext_data['extension']['description']).'\', author=\''.$db->escape($ext_data['extension']['author']).'\', uninstall='.$new_uninstall_code.', uninstall_notes=\''.$db->escape($uninstall_notes).'\' WHERE id=\''.$db->escape($id).'\'') or error(__FILE__, __LINE__);
			$db->query('DELETE FROM '.$db->prefix.'extension_hooks WHERE extension_id=\''.$db->escape($id).'\'') or error(__FILE__, __LINE__);
		}
		else
		{
			// Run the author supplied install code
			if (isset($ext_data['extension']['install']) && trim($ext_data['extension']['install']) != '')
				eval($ext_data['extension']['install']);

			$db->query('INSERT INTO '.$db->prefix.'extensions (id, title, version, description, author, uninstall, uninstall_notes) VALUES(\''.$db->escape($ext_data['extension']['id']).'\', \''.$db->escape($ext_data['extension']['title']).'\', \''.$db->escape($ext_data['extension']['version']).'\', \''.$db->escape($ext_data['extension']['description']).'\', \''.$db->escape($ext_data['extension']['author']).'\', '.$new_uninstall_code.', \''.$db->escape($uninstall_notes).'\')') or error(__FILE__, __LINE__);
		}

		// Now insert the hooks
		$hooks = $ext_data['extension']['hooks']['hook'];
		if (!is_array(current($hooks)))
			$hooks = array($hooks);

		foreach ($hooks as $hook)
			$db->query('INSERT INTO '.$db->prefix.'extension_hooks (id, extension_id, code, installed) VALUES(\''.$db->escape(trim($hook['attributes']['id'])).'\', \''.$db->escape($id).'\', \''.$db->escape(trim($hook['content'])).'\', '.time().')') or error(__FILE__, __LINE__);

		// Empty the PHP cache
		$d = dir(PUN_CACHE_DIR);
		while (($entry = $d->read()) !== false)
		{
			if (substr($entry, strlen($entry)-4) == '.php')
				@unlink(PUN_CACHE_DIR.$entry);
		}
		$d->close();

		// Regenerate the hooks cache
		require_once PUN_ROOT.'include/cache.php';
		generate_hooks_cache();

		// Display warnings and notes if there are any
		if (!empty($warnings) && !empty($install_notes['post']))
		{
			define('PUN_PAGE_SECTION', 'extensions');
			define('PUN_PAGE', 'admin-extensions');
			require PUN_ROOT.'header.php';

?>
<div id="pun-main1" class="a-section a-main a-admin">

	<h1 class="main-title"><span><?php echo end($pun_page['crumbs']) ?></span></h1>

<?php echo generate_admin_menu(); ?>

	<div class="b-section b-dataform">
		<div class="c-section c-form">
<?php if (!empty($install_notes['post'])): ?>
			<div class="panel">
				<p><strong><?php echo $lang_admin['Install notes'] ?></strong></p>
				<p>
					<?php
					echo implode('</p><p>', $install_notes['post']);
					?>
				</p>
			</div>
<?php endif; ?>
			<div class="form-warn">
<?php if (!empty($warnings)): ?>
				<ul class="warnlist">
<?php

	while (list(, $cur_warning) = each($warnings))
		echo "\t\t\t\t\t".'<li><span>'.$cur_warning.'</span></li>'."\n";

?>
				</ul>
<?php endif; ?>
			</div>
		</div>
	</div>

</div>
<?php
			require PUN_ROOT.'footer.php';
		}
		else
			redirect(pun_link($pun_url['admin_extensions']), $lang_admin['Extension installed'].' '.$lang_admin['Redirect']);
	}

	define('PUN_PAGE_SECTION', 'extensions');
	define('PUN_PAGE', 'admin-extensions');
	require PUN_ROOT.'header.php';

?>
<div id="pun-main1" class="a-section a-main a-admin">

	<h1 class="main-title"><span><?php echo end($pun_page['crumbs']) ?></span></h1>

<?php echo generate_admin_menu(); ?>

	<div class="b-section b-dataform">
		<div class="c-section c-form">
			<form method="post" accept-charset="utf-8" action="<?php echo pun_link($pun_url['admin_extensions']).(isset($_GET['install']) ? '?install=' : '?install_hotfix=').$id ?>">
			<div class="panel">
				<input type="hidden" name="csrf_token" value="<?php echo $pun_user['csrf_token'] ?>" />
				<div class="form-intro">
					<h2><span><?php echo end($pun_page['crumbs']) ?></span></h2>
				</div>
				<div class="textset">
					<h3 class="field"><strong class="alabel"><?php echo $lang_admin['Extension title'] ?></strong> <span class="input"><?php echo htmlspecialchars($ext_data['extension']['title']) ?></span></h3>
					<ul class="field">
<?php if (strpos($id, 'hotfix_') !== 0): ?>						<li><strong class="alabel"><?php echo $lang_admin['Extension version'] ?></strong> <span class="input"><?php echo $ext_data['extension']['version'] ?></span></li>
<?php endif; ?>						<li><strong class="alabel"><?php echo $lang_admin['Extension author'] ?></strong> <span class="input"><?php echo $ext_data['extension']['author'] ?></span></li>
						<li><strong class="alabel"><?php echo $lang_admin['Extension requirements'] ?></strong> <span class="input">PunBB <?php echo $ext_data['extension']['minversion'] ?></span></li>
						<li><strong class="alabel"><?php echo $lang_admin['Extension description'] ?></strong> <span class="input"><?php echo htmlspecialchars($ext_data['extension']['description']) ?></span></li>
					</ul>
				</div>
<?php if (strpos($id, 'hotfix_') !== 0): ?>				<p><strong><?php echo $lang_admin['Extension overwrite warning'] ?></strong></p>
<?php endif; ?>				<?php
	if (version_compare(clean_version($pun_config['o_cur_version']), clean_version($ext_data['extension']['minversion']), '<'))
		echo '<p class="formsubmit">' . $lang_admin['PunBB minversion error'];
	else
	{
		echo '<p class="formsubmit"><input type="submit" name="install_comply" value="'.$lang_admin['Install'].'" />';
		if (version_compare(clean_version($pun_config['o_cur_version']), clean_version($ext_data['extension']['maxtestedon']), '>'))
			echo $lang_admin['PunBB maxtestedon warning'];
	}
	echo '</p>';
?>
			</div>
			</form>
		</div>
	</div>

</div>
<?php

	require PUN_ROOT.'footer.php';
}


// Uninstall an extension
else if (isset($_POST['uninstall']) || isset($_POST['uninstall_comply']))
{
	// User pressed the cancel button
	if (isset($_POST['uninstall_cancel']))
		redirect(pun_link($pun_url['admin_extensions']), $lang_admin['Cancel redirect']);

	$id = key($_POST['uninstall']);

	// Fetch info about the extension
	$result = $db->query('SELECT title, version, description, author, uninstall, uninstall_notes FROM '.$db->prefix.'extensions WHERE id=\''.$db->escape($id).'\'') or error(__FILE__, __LINE__);
	if (!$db->num_rows($result))
		message($lang_common['Bad request']);

	$ext_data = $db->fetch_assoc($result);

	// Setup breadcrumbs
	$pun_page['crumbs'] = array(
		array($pun_config['o_board_title'], pun_link($pun_url['index'])),
		array($lang_admin['Forum administration'], pun_link($pun_url['admin_index'])),
		array($lang_admin['Manage extensions'], pun_link($pun_url['admin_extensions'])),
		$lang_admin['Uninstall extension']
	);

	// If the user has confirmed the uninstall
	if (isset($_POST['uninstall_comply']))
	{
		define('SKIP_UNINSTALL', isset($_POST['skip_uninstall_code']));
		$warnings = array();
		$uninstall_notes = unserialize($ext_data['uninstall_notes']);


		if (!empty($uninstall_notes['pre']) && !isset($_POST['uninstall_notes_comply']))
		{
			define('PUN_PAGE_SECTION', 'extensions');
			define('PUN_PAGE', 'admin-extensions');
			require PUN_ROOT.'header.php';
?>

<div id="pun-main1" class="a-section a-main a-admin">

	<h1 class="main-title"><span><?php echo end($pun_page['crumbs']) ?></span></h1>

<?php echo generate_admin_menu(); ?>

	<div class="b-section b-dataform">
		<div class="c-section c-form">
			<form method="post" accept-charset="utf-8" action="<?php echo pun_link($pun_url['admin_extensions']) ?>?uninstall=<?php echo $id ?>">
				<input type="hidden" name="csrf_token" value="<?php echo $pun_user['csrf_token'] ?>" />
				<input type="hidden" name="uninstall[<?php echo $id ?>]" value="1" />
				<input type="hidden" name="uninstall_comply" value="1" />
			<div class="panel">
				<p><strong><?php echo $lang_admin['Uninstall notes'] ?></strong></p>
				<p>
					<?php echo implode("\n\t\t\t\t</p>\n\t\t\t\t<p>\n\t\t\t\t\t", $uninstall_notes['pre']); ?>

				</p>
				
				<p class="formsubmit"><input type="submit" name="uninstall_notes_comply" value="<?php echo $lang_admin['Uninstall'] ?>" /></p>
			</div>
			
			</form>
		</div>
	</div>

</div>

<?php
			require PUN_ROOT.'footer.php';
		}

		// Run uninstall code
		eval($ext_data['uninstall']);

		// Now delete the extension and its hooks from the db
		$db->query('DELETE FROM '.$db->prefix.'extension_hooks WHERE extension_id=\''.$db->escape($id).'\'') or error(__FILE__, __LINE__);
		$db->query('DELETE FROM '.$db->prefix.'extensions WHERE id=\''.$db->escape($id).'\'') or error(__FILE__, __LINE__);

		// Empty the PHP cache
		$d = dir(PUN_CACHE_DIR);
		while (($entry = $d->read()) !== false)
		{
			if (substr($entry, strlen($entry)-4) == '.php')
				@unlink(PUN_CACHE_DIR.$entry);
		}
		$d->close();

		// Regenerate the hooks cache
		require_once PUN_ROOT.'include/cache.php';
		generate_hooks_cache();

		// Display warnings and notes if there are any
		if (!empty($warnings) && !empty($uninstall_notes['post']))
		{
			define('PUN_PAGE_SECTION', 'extensions');
			define('PUN_PAGE', 'admin-extensions');
			require PUN_ROOT.'header.php';

?>
<div id="pun-main1" class="a-section a-main a-admin">

	<h1 class="main-title"><span><?php echo end($pun_page['crumbs']) ?></span></h1>

<?php echo generate_admin_menu(); ?>

	<div class="b-section b-dataform">
		<div class="c-section c-form">
<?php if (!empty($uninstall_notes['post'])): ?>
			<div class="panel">
				<p><strong><?php echo $lang_admin['Uninstall notes'] ?></strong></p>
				<p>
					<?php
					echo implode('</p><p>', $uninstall_notes['post']);
					?>
				</p>
			</div>
<?php endif; ?>
			<div class="form-warn">
<?php if (!empty($warnings)): ?>
				<ul class="warnlist">
<?php

	while (list(, $cur_warning) = each($warnings))
		echo "\t\t\t\t\t".'<li><span>'.$cur_warning.'</span></li>'."\n";

?>
				</ul>
<?php endif; ?>
			</div>
		</div>
	</div>

</div>
<?php
			require PUN_ROOT.'footer.php';
		}
		else
			redirect(pun_link($pun_url['admin_extensions']), $lang_admin['Extension uninstalled'].' '.$lang_admin['Redirect']);
	}
	else	// If the user hasn't comfirmed the uninstall
	{
		// Setup breadcrumbs
		$pun_page['crumbs'] = array(
			array($pun_config['o_board_title'], pun_link($pun_url['index'])),
			array($lang_admin['Forum administration'], pun_link($pun_url['admin_index'])),
			array($lang_admin['Manage extensions'], pun_link($pun_url['admin_extensions'])),
			$lang_admin['Uninstall extension']
		);

		define('PUN_PAGE_SECTION', 'extensions');
		define('PUN_PAGE', 'admin-extensions');
		require PUN_ROOT.'header.php';

?>
<div id="pun-main1" class="a-section a-main a-admin">

	<h1 class="main-title"><span><?php echo end($pun_page['crumbs']) ?></span></h1>

<?php echo generate_admin_menu(); ?>

	<div class="b-section b-dataform">
		<div class="c-section c-form">
			<form method="post" accept-charset="utf-8" action="<?php echo pun_link($pun_url['admin_extensions']) ?>">
			<div class="panel">
				<input type="hidden" name="csrf_token" value="<?php echo $pun_user['csrf_token'] ?>" />
				<input type="hidden" name="uninstall[<?php echo $id ?>]" value="1" />
				<div class="form-head">
					<h2><span><?php echo end($pun_page['crumbs']) ?> "<?php echo htmlspecialchars($ext_data['title']) ?>"</span></h2>
				</div>
<?php if (!empty($ext_data['uninstall'])): ?>				<fieldset class="checkset" id="fs1">
					<legend><span><?php echo $lang_admin['Uninstall options'] ?></span></legend>
					<p class="field">
						<span class="input"><input type="checkbox" id="fs1f1" name="skip_uninstall_code" value="1" /> <label for="fs1f1"><?php echo $lang_admin['Skip uninstall code'] ?></label><br /> <em class="field-info"><?php echo $lang_admin['Skip uninstall code info'] ?></em></span><br />
					</p>
				</fieldset>
<?php endif; ?>				<div class="form-warn">
					<p><?php printf($lang_admin['Uninstall extension confirm'], htmlspecialchars($ext_data['title'])) ?> </p>
				</div>
				<p class="formsubmit"><input type="submit" class="button" name="uninstall_comply" value="<?php echo $lang_admin['Uninstall'] ?>" /> <input type="submit" class="button" name="uninstall_cancel" value="<?php echo $lang_admin['Cancel'] ?>" /></p>
			</div>
			</form>
		</div>
	</div>

</div>
<?php

		require PUN_ROOT.'footer.php';
	}
}


// Generate an array of installed extensions
$inst_exts = array();
$result = $db->query('SELECT * FROM '.$db->prefix.'extensions ORDER BY title') or error(__FILE__, __LINE__);
while ($cur_ext = $db->fetch_assoc($result))
	$inst_exts[$cur_ext['id']] = $cur_ext;

// Setup breadcrumbs
$pun_page['crumbs'] = array(
	array($pun_config['o_board_title'], pun_link($pun_url['index'])),
	array($lang_admin['Forum administration'], pun_link($pun_url['admin_index'])),
	$lang_admin['Manage extensions']
);

define('PUN_PAGE_SECTION', 'extensions');
define('PUN_PAGE', 'admin-extensions');

require PUN_ROOT.'header.php';

?>
<div id="pun-main1" class="a-section a-main a-admin">

	<h1 class="main-title"><span><?php echo end($pun_page['crumbs']) ?></span></h1>

<?php echo generate_admin_menu(); ?>

	<div class="b-section b-dataform">
		<div class="c-section c-form">
			<div class="panel">
				<div class="form-intro">
					<h2><span><?php echo $lang_admin['Extensions available'] ?></span></h2>
				</div>
<?php

// Make sure we have XML support
if (function_exists('xml_parser_create'))
{
	echo "\t\t\t\t".'<div class="textset">'."\n";
	$num_exts = 0;
	$num_failed = 0;

	// Loop through any available hotfixes
	if (isset($pun_updates['hotfix']))
	{
		// If there's only one hotfix, add one layer of arrays so we can foreach over it
		if (!is_array(current($pun_updates['hotfix'])))
			$pun_updates['hotfix'] = array($pun_updates['hotfix']);

		foreach ($pun_updates['hotfix'] as $hotfix)
		{
			if (!array_key_exists($hotfix['attributes']['id'], $inst_exts))
			{
				echo "\t\t\t\t\t".'<p class="field hotfix"><strong class="alabel"><a href="'.pun_link($pun_url['admin_extensions']).'?install_hotfix='.urlencode($hotfix['attributes']['id']).'">'.$lang_admin['Install'].'</a></strong> <span class="input"><strong>'.htmlspecialchars($hotfix['content']).'</strong></span></p>'."\n";
				++$num_exts;
			}
		}
	}

	$d = dir(PUN_ROOT.'extensions');
	while (($entry = $d->read()) !== false)
	{
		if ($entry{0} != '.' && is_dir(PUN_ROOT.'extensions/'.$entry))
		{
			if (preg_match('/[^0-9a-z_]/', $entry))
			{
				echo "\t\t\t\t\t".'<p class="field"><strong class="alabel error">'.$lang_admin['Extension error'].'<span>: </span></strong> <span class="input">'.sprintf($lang_admin['Extension loading error'], htmlspecialchars($entry)).' '.$lang_admin['Illegal ID'].'</span></p>'."\n";
				++$num_failed;
				continue;
			}
			else if (!file_exists(PUN_ROOT.'extensions/'.$entry.'/manifest.xml'))
			{
				echo "\t\t\t\t\t".'<p class="field"><strong class="alabel error">'.$lang_admin['Extension error'].'<span>: </span></strong> <span class="input">'.sprintf($lang_admin['Extension loading error'], htmlspecialchars($entry)).' '.$lang_admin['Missing manifest'].'</span></p>'."\n";
				++$num_failed;
				continue;
			}

			// Parse manifest.xml into an array
			$ext_data = xml_to_array(@file_get_contents(PUN_ROOT.'extensions/'.$entry.'/manifest.xml'));
			if (empty($ext_data))
			{
				echo "\t\t\t\t\t".'<p class="field"><strong class="alabel error">'.$lang_admin['Extension error'].'<span>: </span></strong> <span class="input">'.sprintf($lang_admin['Extension loading error'], htmlspecialchars($entry)).' '.$lang_admin['Failed parse manifest'].'</span></p>'."\n";
				++$num_failed;
				continue;
			}

			// Validate manifest
			$errors = validate_manifest($ext_data, $entry);
			if (!empty($errors))
			{
				echo "\t\t\t\t\t".'<p class="field"><strong class="alabel error">'.$lang_admin['Extension error'].'<span>: </span></strong> <span class="input">'.sprintf($lang_admin['Extension loading error'], htmlspecialchars($entry)).' '.implode(' ', $errors).'</span></p>'."\n";
				++$num_failed;
			}
			else
			{
				if (!array_key_exists($entry, $inst_exts) || version_compare($inst_exts[$entry]['version'], $ext_data['extension']['version'], '!='))
				{
					echo "\t\t\t\t\t".'<p class="field"><strong class="alabel"><a href="'.pun_link($pun_url['admin_extensions']).'?install='.urlencode($entry).'">'.$lang_admin['Install'].'</a></strong> <span class="input"><strong>'.htmlspecialchars($ext_data['extension']['title']).(strpos($entry, 'hotfix_') !== 0 ? ' '.$ext_data['extension']['version'] : '').'</strong></span></p>'."\n";
					++$num_exts;
				}
			}
		}
	}
	$d->close();

	echo "\t\t\t\t".'</div>';

	if (!$num_exts)
	{
		if (!$num_failed)
			echo "\t\t\t\t".'<p>'.$lang_admin['No available extensions'].'</p>'."\n";
		else
			echo "\t\t\t\t".'<div class="form-warn"><p><strong>'.$lang_admin['Important'].'</strong> '.$lang_admin['No valid extensions'].'</p></div>';
	}
	else
	{
		if ($num_failed)
			echo "\t\t\t\t".'<div class="form-warn"><p><strong>'.$lang_admin['Important'].'</strong> '.$lang_admin['Some invalid extensions'].'</p></div>';
	}
}
else
	echo "\t\t\t\t".'<p>'.$lang_admin['No XML support'].'</p>'."\n";

?>
			</div>
<?php

if (!empty($inst_exts))
{

?>
			<div class="divider"><hr /></div>
			<form method="post" accept-charset="utf-8" action="<?php echo pun_link($pun_url['admin_extensions']) ?>">
			<div class="panel">
				<input type="hidden" name="csrf_token" value="<?php echo $pun_user['csrf_token'] ?>" />
				<div class="form-intro">
					<h2><span><?php echo $lang_admin['Installed extensions'] ?></span></h2>
				</div>
				<div class="form-warn"><p><?php echo '<strong>'.$lang_admin['Warning'].'</strong> '.$lang_admin['Installed extensions warn'] ?></p></div>
			</div>
<?php

	while (list($id, $ext) = @each($inst_exts))
	{

?>
			<div class="panel subpanel">
				<div class="textset">
					<h3 class="field"><strong class="alabel"><?php echo $lang_admin['Extension title'] ?><span>:</span></strong> <span class="input"><?php echo htmlspecialchars($ext['title']) ?></span></h3>
					<ul class="field">
<?php if (strpos($id, 'hotfix_') !== 0): ?>						<li><strong class="alabel"><?php echo $lang_admin['Extension version'] ?><span>:</span></strong> <span class="input"><?php echo $ext['version'] ?></span></li>
<?php endif; ?>						<li><strong class="alabel"><?php echo $lang_admin['Extension author'] ?><span>:</span></strong> <span class="input"><?php echo $ext['author'] ?></span></li>
<?php if ($ext['description'] != ''): ?>						<li><strong class="alabel"><?php echo $lang_admin['Extension description'] ?><span>:</span></strong> <span class="input"><?php echo htmlspecialchars($ext['description']) ?></span></li>
<?php endif; ?>					</ul>
				</div>
				<p class="formsubmit"><input type="submit" name="uninstall[<?php echo $id ?>]" value="<?php echo $lang_admin['Uninstall'] ?>" /></p>
			</div>
<?php

	}
	echo "\t\t\t".'</form>'."\n";
}
else
	echo "\t\t\t".'<div class="panel">'."\n\t\t\t\t".'<div class="form-intro">'."\n\t\t\t\t\t".'<h2><span>'.$lang_admin['Installed extensions'].'</span></h2>'."\n\t\t\t\t".'</div>'."\n\t\t\t\t".'<p>'.$lang_admin['No installed extensions'].'</p>'."\n\t\t\t".'</div>'."\n";

?>
		</div>
	</div>

</div>
<?php

require PUN_ROOT.'footer.php';
