<?php
/***********************************************************************

  Copyright (C) 2002-2007  Rickard Andersson (rickard@punbb.org)

  This file is part of PunBB.

  PunBB is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 2 of the License,
  or (at your option) any later version.

  PunBB is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
  MA  02111-1307  USA

************************************************************************/


define('PUN_ROOT', '../');
require PUN_ROOT.'include/common.php';
require PUN_ROOT.'include/common_admin.php';


if ($pun_user['g_id'] != PUN_ADMIN)
	message($lang_common['No permission']);

// Load the admin.php language file
require PUN_ROOT.'lang/'.$pun_user['language'].'/admin.php';


// Add a "default" forum
if (isset($_POST['add_forum']))
{
	$add_to_cat = intval($_POST['add_to_cat']);
	if ($add_to_cat < 1)
		message($lang_common['Bad request']);

	$forum_name = trim($_POST['forum_name']);
	$position = intval($_POST['position']);

	if ($forum_name == '')
		message($lang_admin['Must enter forum message']);

	$db->query('INSERT INTO '.$db->prefix.'forums (forum_name, disp_position, cat_id) VALUES(\''.$db->escape($forum_name).'\', '.$position.', '.$add_to_cat.')') or error(__FILE__, __LINE__);

	// Regenerate the quickjump cache
	require_once PUN_ROOT.'include/cache.php';
	generate_quickjump_cache();

	redirect(pun_link($pun_url['admin_forums']), $lang_admin['Forum added'].' '.$lang_admin['Redirect']);
}


// Delete a forum
else if (isset($_GET['del_forum']))
{
	$forum_id = intval($_GET['del_forum']);
	if ($forum_id < 1)
		message($lang_common['Bad request']);

	// User pressed the cancel button
	if (isset($_POST['del_forum_cancel']))
		redirect(pun_link($pun_url['admin_forums']), $lang_admin['Cancel redirect']);


	if (isset($_POST['del_forum_comply']))	// Delete a forum with all posts
	{
		@set_time_limit(0);

		// Prune all posts and topics
		prune($forum_id, 1, -1);

		// Locate any "orphaned redirect topics" and delete them
		$result = $db->query('SELECT t1.id FROM '.$db->prefix.'topics AS t1 LEFT JOIN '.$db->prefix.'topics AS t2 ON t1.moved_to=t2.id WHERE t2.id IS NULL AND t1.moved_to IS NOT NULL') or error(__FILE__, __LINE__);
		$num_orphans = $db->num_rows($result);

		if ($num_orphans)
		{
			for ($i = 0; $i < $num_orphans; ++$i)
				$orphans[] = $db->result($result, $i);

			$db->query('DELETE FROM '.$db->prefix.'topics WHERE id IN('.implode(',', $orphans).')') or error(__FILE__, __LINE__);
		}

		// Delete the forum and any forum specific group permissions
		$db->query('DELETE FROM '.$db->prefix.'forums WHERE id='.$forum_id) or error(__FILE__, __LINE__);
		$db->query('DELETE FROM '.$db->prefix.'forum_perms WHERE forum_id='.$forum_id) or error(__FILE__, __LINE__);

		// Regenerate the quickjump cache
		require_once PUN_ROOT.'include/cache.php';
		generate_quickjump_cache();

		redirect(pun_link($pun_url['admin_forums']), $lang_admin['Forum deleted'].' '.$lang_admin['Redirect']);
	}
	else	// If the user hasn't confirmed the delete
	{
		$result = $db->query('SELECT forum_name FROM '.$db->prefix.'forums WHERE id='.$forum_id) or error(__FILE__, __LINE__);
		$forum_name = htmlspecialchars($db->result($result));


		// Setup breadcrumbs
		$pun_page['crumbs'] = array(
			array($pun_config['o_board_title'], pun_link($pun_url['index'])),
			array($lang_admin['Forum administration'], pun_link($pun_url['admin_index'])),
			array($lang_admin['Forums'], pun_link($pun_url['admin_forums'])),
			$lang_admin['Delete forum']
		);

		define('PUN_PAGE_SECTION', 'setup');
		define('PUN_PAGE', 'admin-forums');
		require PUN_ROOT.'header.php';

?>
<div id="pun-main1" class="a-section a-main a-admin">

	<h1 class="main-title"><span><?php echo end($pun_page['crumbs']) ?></span></h1>

<?php echo generate_admin_menu(); ?>

	<div class="b-section b-dataform">
		<div class="c-section c-form">
			<form method="post" accept-charset="utf-8" action="<?php echo pun_link($pun_url['admin_forums']) ?>?del_forum=<?php echo $forum_id ?>">
			<div class="panel">
				<input type="hidden" name="csrf_token" value="<?php echo $pun_user['csrf_token'] ?>" />
				<div class="form-intro">
					<h2><span><?php echo end($pun_page['crumbs']).' "'.$forum_name.'"' ?></span></h2>
				</div>
				<div class="form-warn">
					<p><?php printf($lang_admin['Delete forum info'], $forum_name) ?></p>
					<p><strong class="warntext"><?php echo $lang_admin['Delete forum warning'] ?></strong></p>
				</div>
				<p class="formsubmit"><input type="submit" class="button" name="del_forum_comply" value="<?php echo $lang_admin['Delete'] ?>" /> <input type="submit" class="button" name="del_forum_cancel" value="<?php echo $lang_admin['Cancel'] ?>" /></p>
			</div>
			</form>
		</div>
	</div>

</div>
<?php

		require PUN_ROOT.'footer.php';
	}
}


// Update forum positions
else if (isset($_POST['update_positions']))
{
	$positions = array_map('intval', $_POST['position']);

	$result = $db->query('SELECT f.id AS fid, f.disp_position FROM '.$db->prefix.'categories AS c INNER JOIN '.$db->prefix.'forums AS f ON c.id=f.cat_id ORDER BY c.disp_position, c.id, f.disp_position') or error(__FILE__, __LINE__);

	while ($cur_forum = $db->fetch_assoc($result))
	{
		// If these aren't set, we're looking at a forum that was added after
		// the admin started to editing: we don't want to mess with it
		if (isset($positions[$cur_forum['fid']]))
		{
			$new_disp_position = $positions[$cur_forum['fid']];

			if ($new_disp_position < 0)
				message($lang_admin['Must be integer']);

			// We only want to update if we changed the position
			if ($cur_forum['disp_position'] != $new_disp_position)
				$db->query('UPDATE '.$db->prefix.'forums SET disp_position='.$new_disp_position.' WHERE id='.$cur_forum['fid']) or error(__FILE__, __LINE__);
		}
	}

	// Regenerate the quickjump cache
	require_once PUN_ROOT.'include/cache.php';
	generate_quickjump_cache();

	redirect(pun_link($pun_url['admin_forums']), $lang_admin['Forums updated'].' '.$lang_admin['Redirect']);
}


else if (isset($_GET['edit_forum']))
{
	$forum_id = intval($_GET['edit_forum']);
	if ($forum_id < 1)
		message($lang_common['Bad request']);

	// Update group permissions for $forum_id
	if (isset($_POST['save']))
	{
		// Start with the forum details
		$forum_name = trim($_POST['forum_name']);
		$forum_desc = pun_linebreaks(trim($_POST['forum_desc']));
		$cat_id = intval($_POST['cat_id']);
		$sort_by = intval($_POST['sort_by']);
		$redirect_url = isset($_POST['redirect_url']) ? trim($_POST['redirect_url']) : null;
		$approval = ($_POST['approval'] == 1) ? 1 : 0;

		if ($forum_name == '')
			message($lang_admin['Must enter forum message']);

		if ($cat_id < 1)
			message($lang_common['Bad request']);

		$forum_desc = ($forum_desc != '') ? '\''.$db->escape($forum_desc).'\'' : 'NULL';
		$redirect_url = ($redirect_url != '') ? '\''.$db->escape($redirect_url).'\'' : 'NULL';

		$db->query('UPDATE '.$db->prefix.'forums SET forum_name=\''.$db->escape($forum_name).'\', forum_desc='.$forum_desc.', redirect_url='.$redirect_url.', approval='.$approval.', sort_by='.$sort_by.', cat_id='.$cat_id.' WHERE id='.$forum_id) or error(__FILE__, __LINE__);

		// Now let's deal with the permissions
		if (isset($_POST['read_forum_old']))
		{
			$result = $db->query('SELECT g_id, g_read_board, g_post_replies, g_post_topics FROM '.$db->prefix.'groups WHERE g_id!='.PUN_ADMIN) or error(__FILE__, __LINE__);
			while ($cur_group = $db->fetch_assoc($result))
			{
				$read_forum_new = ($cur_group['g_read_board'] == '1') ? isset($_POST['read_forum_new'][$cur_group['g_id']]) ? '1' : '0' : intval($_POST['read_forum_old'][$cur_group['g_id']]);
				$post_replies_new = isset($_POST['post_replies_new'][$cur_group['g_id']]) ? '1' : '0';
				$post_topics_new = isset($_POST['post_topics_new'][$cur_group['g_id']]) ? '1' : '0';

				// Check if the new settings differ from the old
				if ($read_forum_new != $_POST['read_forum_old'][$cur_group['g_id']] || $post_replies_new != $_POST['post_replies_old'][$cur_group['g_id']] || $post_topics_new != $_POST['post_topics_old'][$cur_group['g_id']])
				{
					// If the new settings are identical to the default settings for this group, delete it's row in forum_perms
					if ($read_forum_new == '1' && $post_replies_new == $cur_group['g_post_replies'] && $post_topics_new == $cur_group['g_post_topics'])
						$db->query('DELETE FROM '.$db->prefix.'forum_perms WHERE group_id='.$cur_group['g_id'].' AND forum_id='.$forum_id) or error(__FILE__, __LINE__);
					else
					{
						// Run an UPDATE and see if it affected a row, if not, INSERT
						$db->query('UPDATE '.$db->prefix.'forum_perms SET read_forum='.$read_forum_new.', post_replies='.$post_replies_new.', post_topics='.$post_topics_new.' WHERE group_id='.$cur_group['g_id'].' AND forum_id='.$forum_id) or error(__FILE__, __LINE__);
						if (!$db->affected_rows())
							$db->query('INSERT INTO '.$db->prefix.'forum_perms (group_id, forum_id, read_forum, post_replies, post_topics) VALUES('.$cur_group['g_id'].', '.$forum_id.', '.$read_forum_new.', '.$post_replies_new.', '.$post_topics_new.')') or error(__FILE__, __LINE__);
					}
				}
			}
		}

		// If approval is disabled make sure all posts in the forum have been set as approved
		$db->query('UPDATE '.$db->prefix.'posts as p, '.$db->prefix.'topics as t, '.$db->prefix.'forums as f SET p.approved=1 WHERE f.id='.$forum_id.' AND t.forum_id=f.id AND p.topic_id=t.id') or error(__FILE__, __LINE__);

		// Regenerate the quickjump cache
		require_once PUN_ROOT.'include/cache.php';
		generate_quickjump_cache();

		redirect(pun_link($pun_url['admin_forums']), $lang_admin['Forum updated'].' '.$lang_admin['Redirect']);
	}
	else if (isset($_POST['revert_perms']))
	{
		$db->query('DELETE FROM '.$db->prefix.'forum_perms WHERE forum_id='.$forum_id) or error(__FILE__, __LINE__);

		// Regenerate the quickjump cache
		require_once PUN_ROOT.'include/cache.php';
		generate_quickjump_cache();

		redirect(pun_link($pun_url['admin_forums']).'?edit_forum='.$forum_id, $lang_admin['Permissions reverted'].' '.$lang_admin['Redirect']);
	}

	// Fetch forum info
	$result = $db->query('SELECT id, forum_name, forum_desc, redirect_url, approval, num_topics, sort_by, cat_id FROM '.$db->prefix.'forums WHERE id='.$forum_id) or error(__FILE__, __LINE__);
	if (!$db->num_rows($result))
		message($lang_common['Bad request']);

	$cur_forum = $db->fetch_assoc($result);


	// Setup breadcrumbs
	$pun_page['crumbs'] = array(
		array($pun_config['o_board_title'], pun_link($pun_url['index'])),
		array($lang_admin['Forum administration'], pun_link($pun_url['admin_index'])),
		array($lang_admin['Forums'], pun_link($pun_url['admin_forums'])),
		$lang_admin['Edit forum']
	);

	define('PUN_PAGE_SECTION', 'setup');
	define('PUN_PAGE', 'admin-forums');
	require PUN_ROOT.'header.php';

?>
<div id="pun-main1" class="a-section a-main a-admin">

	<h1 class="main-title"><span><?php echo end($pun_page['crumbs']) ?></span></h1>

<?php echo generate_admin_menu(); ?>

	<div class="b-section b-dataform">
		<div class="c-section c-form">
			<form method="post" accept-charset="utf-8" action="<?php echo pun_link($pun_url['admin_forums']) ?>?edit_forum=<?php echo $forum_id ?>">
			<div class="panel">
				<input type="hidden" name="csrf_token" value="<?php echo $pun_user['csrf_token'] ?>" />
				<div class="form-intro">
					<h2 class="prefix"><span><?php echo $lang_admin['Edit forum details head'] ?>: </span></h2>
					<p><?php echo $lang_admin['Edit forum details intro'] ?></p>
				</div>
				<fieldset class="textset" id="fs1">
					<legend><span><?php echo $lang_admin['Edit forum details legend'] ?></span></legend>
					<p class="field">
						<label for="fs1f1"><?php echo $lang_admin['Forum name'] ?></label><br />
						<span class="input"><input type="text" id="fs1f1" name="forum_name" size="35" maxlength="80" value="<?php echo htmlspecialchars($cur_forum['forum_name']) ?>" /></span>
					</p>
					<p class="field areafield">
						<label for="fs1f2"><?php echo $lang_admin['Forum description'] ?></label><br />
						<span class="input"><textarea id="fs1f2" name="forum_desc" rows="3" cols="50"><?php echo htmlspecialchars($cur_forum['forum_desc']) ?></textarea></span>
					</p>
					<p class="field">
						<label for="fs1f3"><?php echo $lang_admin['Category'] ?></label><br />
						<span class="input">
							<select id="fs1f3" name="cat_id">
<?php

	$result = $db->query('SELECT id, cat_name FROM '.$db->prefix.'categories ORDER BY disp_position') or error(__FILE__, __LINE__);
	while ($cur_cat = $db->fetch_assoc($result))
	{
		$selected = ($cur_cat['id'] == $cur_forum['cat_id']) ? ' selected="selected"' : '';
		echo "\t\t\t\t\t\t\t\t".'<option value="'.$cur_cat['id'].'"'.$selected.'>'.htmlspecialchars($cur_cat['cat_name']).'</option>'."\n";
	}

?>
							</select>
						</span>
					</p>
					<p class="field">
						<label for="fs1f4"><?php echo $lang_admin['Sort topics by'] ?></label><br />
						<span class="input">
							<select id="fs1f4" name="sort_by">
								<option value="0"<?php if ($cur_forum['sort_by'] == '0') echo ' selected="selected"' ?>><?php echo $lang_admin['Sort last post'] ?></option>
								<option value="1"<?php if ($cur_forum['sort_by'] == '1') echo ' selected="selected"' ?>><?php echo $lang_admin['Sort topic start'] ?></option>
							</select>
						</span>
					</p>
					<p class="field">
						<label for="fs1f5"><?php echo $lang_admin['Redirect URL'] ?></label><br />
						<span class="input"><?php echo ($cur_forum['num_topics']) ? '<input type="text" id="fld5" name="redirect_url" size="45" maxlength="100" value="Only available in empty forums" disabled="disabled" />' : '<input type="text" id="fld5" name="redirect_url" size="45" maxlength="100" value="'.htmlspecialchars($cur_forum['redirect_url']).'" />'; ?></span>
					</p>
					<p class="field">
						<label for="fld6"><?php echo $lang_admin['Post moderation queue'] ?></label><br />
						<span class="input">
							<select id="fld6" name="approval">
								<option value="0"<?php if ($cur_forum['approval'] == '0') echo ' selected="selected"' ?>><?php echo $lang_admin['Disabled'] ?></option>
								<option value="1"<?php if ($cur_forum['approval'] == '1') echo ' selected="selected"' ?>><?php echo $lang_admin['Enabled'] ?></option>
							</select>
						</span>
					</p>
				</fieldset>
				<div class="form-intro">
					<h2 class="prefix"><span><?php echo $lang_admin['Edit forum perms head'] ?>:</span></h2>
					<p><?php echo $lang_admin['Edit forum perms intro'] ?> <a href="<?php echo pun_link($pun_url['admin_groups']) ?>"><?php echo $lang_admin['User groups'] ?></a>.</p>
				</div>
					<div class="textset">
						<p class="field">
							<strong class="alabel"><?php echo $lang_admin['Restore defaults'] ?></strong><br />
							<span class="input"><input type="submit" class="button" name="revert_perms" value="<?php echo $lang_admin['Restore defaults'] ?>" /></span>
						</p>
					</div>
<?php

	$i = 2;
	$result = $db->query('SELECT g.g_id, g.g_title, g.g_read_board, g.g_post_replies, g.g_post_topics, fp.read_forum, fp.post_replies, fp.post_topics FROM '.$db->prefix.'groups AS g LEFT JOIN '.$db->prefix.'forum_perms AS fp ON (g.g_id=fp.group_id AND fp.forum_id='.$forum_id.') WHERE g.g_id!='.PUN_ADMIN.' ORDER BY g.g_id') or error(__FILE__, __LINE__);
	while ($cur_perm = $db->fetch_assoc($result))
	{
		$read_forum = ($cur_perm['read_forum'] != '0') ? true : false;
		$post_replies = (($cur_perm['g_post_replies'] == '0' && $cur_perm['post_replies'] == '1') || ($cur_perm['g_post_replies'] == '1' && $cur_perm['post_replies'] != '0')) ? true : false;
		$post_topics = (($cur_perm['g_post_topics'] == '0' && $cur_perm['post_topics'] == '1') || ($cur_perm['g_post_topics'] == '1' && $cur_perm['post_topics'] != '0')) ? true : false;

		// Determine if the current sittings differ from the default or not
		$read_forum_def = ($cur_perm['read_forum'] == '0') ? false : true;
		$post_replies_def = (($post_replies && $cur_perm['g_post_replies'] == '0') || (!$post_replies && ($cur_perm['g_post_replies'] == '' || $cur_perm['g_post_replies'] == '1'))) ? false : true;
		$post_topics_def = (($post_topics && $cur_perm['g_post_topics'] == '0') || (!$post_topics && ($cur_perm['g_post_topics'] == '' || $cur_perm['g_post_topics'] == '1'))) ? false : true;

?>
					<fieldset class="checkset" id="fs<?php echo $i ?>">
						<legend><span><?php echo '<span class="acchide">'.$lang_admin['User group'].': </span>'.htmlspecialchars($cur_perm['g_title']) ?></span></legend>
						<p class="field">
							<span class="input"><input type="hidden" name="read_forum_old[<?php echo $cur_perm['g_id'] ?>]" value="<?php echo ($read_forum) ? '1' : '0'; ?>" /><input type="checkbox" id="fs<?php echo $i ?>f1" name="read_forum_new[<?php echo $cur_perm['g_id'] ?>]" value="1"<?php if ($read_forum) echo ' checked="checked"'; echo ($cur_perm['g_read_board'] == '0') ? ' disabled="disabled"' : ''; ?> /> <label for="fs<?php echo $i ?>f1"<?php if (!$read_forum_def) echo ' class="nodefault"'; ?>><?php echo $lang_admin['Read forum']; if (!$read_forum_def) echo '<span class="acchide">&#160;'.$lang_admin['Not default'].'</span>'; ?></label> <input type="hidden" name="post_replies_old[<?php echo $cur_perm['g_id'] ?>]" value="<?php echo ($post_replies) ? '1' : '0'; ?>" /><input type="checkbox" id="fs<?php echo $i ?>f2" name="post_replies_new[<?php echo $cur_perm['g_id'] ?>]" value="1"<?php if ($post_replies) echo ' checked="checked"'; echo ($cur_forum['redirect_url'] != '') ? ' disabled="disabled"' : ''; ?> /> <label for="fs<?php echo $i ?>f2"<?php if (!$post_replies_def) echo ' class="nodefault"'; ?>><?php echo $lang_admin['Post replies']; if (!$post_replies_def) echo '<span class="acchide">&#160;'.$lang_admin['Not default'].'</span>'; ?></label> <input type="hidden" name="post_topics_old[<?php echo $cur_perm['g_id'] ?>]" value="<?php echo ($post_topics) ? '1' : '0'; ?>" /><input type="checkbox" id="fs<?php echo $i ?>f3" name="post_topics_new[<?php echo $cur_perm['g_id'] ?>]" value="1"<?php if ($post_topics) echo ' checked="checked"'; echo ($cur_forum['redirect_url'] != '') ? ' disabled="disabled"' : ''; ?> /> <label for="fs<?php echo $i ?>f3"<?php if (!$post_topics_def) echo ' class="nodefault"'; ?>><?php echo $lang_admin['Post topics']; if (!$post_topics_def) echo '<span class="acchide">&#160;'.$lang_admin['Not default'].'</span>'; ?></label></span>
						</p>
					</fieldset>
<?php

		++$i;
	}

?>
				<p class="formsubmit"><input type="submit" class="button" name="save" value="<?php echo $lang_admin['Save changes'] ?>" /></p>
			</div>
			</form>
		</div>
	</div>

</div>

<?php

	require PUN_ROOT.'footer.php';
}


// Setup breadcrumbs
$pun_page['crumbs'] = array(
	array($pun_config['o_board_title'], pun_link($pun_url['index'])),
	array($lang_admin['Forum administration'], pun_link($pun_url['admin_index'])),
	$lang_admin['Forums']
);

define('PUN_PAGE_SECTION', 'setup');
define('PUN_PAGE', 'admin-forums');
require PUN_ROOT.'header.php';

?>
<div id="pun-main1" class="a-section a-main a-admin">

	<h1 class="main-title"><span><?php echo end($pun_page['crumbs']) ?></span></h1>

<?php echo generate_admin_menu(); ?>

	<div class="b-section b-dataform">
		<div class="c-section c-form">
			<form method="post" accept-charset="utf-8" action="<?php echo pun_link($pun_url['admin_forums']) ?>?action=adddel">
			<div class="panel">
				<input type="hidden" name="csrf_token" value="<?php echo $pun_user['csrf_token'] ?>" />
				<div class="form-intro">
					<h2 class="prefix"><span><?php echo $lang_admin['Add forum'] ?>: </span></h2>
					<p><?php echo $lang_admin['Add forum intro'] ?></p>
				</div>
				<div class="textset" id="fs1">
					<p class="field">
						<label for="fs1f1"><?php echo $lang_admin['Forum name'] ?></label><br />
						<span class="input"><input type="text" id="fs1f1" name="forum_name" size="35" maxlength="80" /></span>
					</p>
					<p class="field">
						<label for="fs1f2"><?php echo $lang_admin['Position'] ?></label><br />
						<span class="input"><input type="text" id="fs1f2" name="position" size="3" maxlength="3" /></span>
					</p>
					<p class="field">
						<label for="fs1f3"><?php echo $lang_admin['Add to category'] ?></label><br />
						<span class="input"><select id="fs1f3" name="add_to_cat">
<?php

	$result = $db->query('SELECT id, cat_name FROM '.$db->prefix.'categories ORDER BY disp_position') or error(__FILE__, __LINE__);
	while ($cur_cat = $db->fetch_assoc($result))
		echo "\t\t\t\t\t\t\t".'<option value="'.$cur_cat['id'].'">'.htmlspecialchars($cur_cat['cat_name']).'</option>'."\n";

?>
						</select></span>
					</p>
				</div>
				<p class="formsubmit"><input type="submit" class="button" name="add_forum" value=" <?php echo $lang_admin['Add forum'] ?> " /></p>
			</div>
			</form>

<?php

// Display all the categories and forums
$result = $db->query('SELECT c.id AS cid, c.cat_name, f.id AS fid, f.forum_name, f.disp_position FROM '.$db->prefix.'categories AS c INNER JOIN '.$db->prefix.'forums AS f ON c.id=f.cat_id ORDER BY c.disp_position, c.id, f.disp_position') or error(__FILE__, __LINE__);

if ($db->num_rows($result))
{

?>
			<div class="divider"><hr /></div>
			<form method="post" accept-charset="utf-8" action="<?php echo pun_link($pun_url['admin_forums']) ?>?action=edit">
			<div class="panel">
				<input type="hidden" name="csrf_token" value="<?php echo $pun_user['csrf_token'] ?>" />
				<h2 class="prefix"><span><?php echo $lang_admin['Edit forums'] ?>: </span></h2>
				<p><?php echo $lang_admin['Edit forums intro'] ?></p>
			</div>

<?php

	$cur_category = 0;
	$i = 2;

	while ($cur_forum = $db->fetch_assoc($result))
	{
		if ($cur_forum['cid'] != $cur_category)	// A new category since last iteration?
		{
			if ($i > 2) echo "\t\t\t".'</div>'."\n";

?>
			<div class="panel subpanel">
				<div class="form-intro">
					<h3><span><?php echo $lang_admin['Category'].': '.htmlspecialchars($cur_forum['cat_name']) ?></span></h3>
				</div>
<?php

			$cur_category = $cur_forum['cid'];
		}

?>
				<div class="textset" id="fs<?php echo $i ?>">
					<h4 class="field">
						<span class="alabel"><?php echo $lang_admin['Forum'] ?><span>: </span></span><span class="input"><?php echo htmlspecialchars($cur_forum['forum_name']) ?></span>
					</h4>
					<p class="field">
						<span class="alabel"><a href="<?php echo pun_link($pun_url['admin_forums']) ?>?edit_forum=<?php echo $cur_forum['fid'] ?>"><?php echo $lang_admin['Edit'].'<span class="acchide"> '.htmlspecialchars($cur_forum['forum_name']).'</span>' ?></a><br /> <a href="<?php echo pun_link($pun_url['admin_forums']) ?>?del_forum=<?php echo $cur_forum['fid'] ?>"><?php echo $lang_admin['Delete'].'<span class="acchide"> '.htmlspecialchars($cur_forum['forum_name']).'</span>' ?></a><br /> <label for="fs<?php echo $i ?>f1"> <?php echo $lang_admin['Position'] ?></label></span><br />
						<span class="input"><input type="text" id="fs<?php echo $i ?>f1" name="position[<?php echo $cur_forum['fid'] ?>]" size="3" maxlength="3" value="<?php echo $cur_forum['disp_position'] ?>" /></span>
					</p>
				</div>
<?php

		++$i;
	}

?>
			</div>
			<p class="formsubmit"><input type="submit" class="button" name="update_positions" value="<?php echo $lang_admin['Update positions'] ?>" /> <span><?php echo $lang_admin['Update all forums'] ?></span></p>
			</form>
<?php

}

?>
		</div>
	</div>

</div>
<?php

require PUN_ROOT.'footer.php';
