<?php
/***********************************************************************

  Copyright (C) 2002-2007  Rickard Andersson (rickard@punbb.org)

  This file is part of PunBB.

  PunBB is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 2 of the License,
  or (at your option) any later version.

  PunBB is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
  MA  02111-1307  USA

************************************************************************/


define('PUN_ROOT', '../');
require PUN_ROOT.'include/common.php';
require PUN_ROOT.'include/common_admin.php';


if ($pun_user['g_id'] != PUN_ADMIN)
	message($lang_common['No permission']);

// Load the admin.php language file
require PUN_ROOT.'lang/'.$pun_user['language'].'/admin.php';


// Add a rank
if (isset($_POST['add_rank']))
{
	$rank = trim($_POST['new_rank']);
	$min_posts = intval($_POST['new_min_posts']);

	if ($rank == '')
		message($lang_admin['Title message']);

	if ($min_posts < 0)
		message($lang_admin['Min posts message']);

	// Make sure there isn't already a rank with the same min_posts value
	$result = $db->query('SELECT 1 FROM '.$db->prefix.'ranks WHERE min_posts='.$min_posts) or error(__FILE__, __LINE__);
	if ($db->num_rows($result))
		message(sprintf($lang_admin['Min posts occupied message'], $min_posts));

	$db->query('INSERT INTO '.$db->prefix.'ranks (rank, min_posts) VALUES(\''.$db->escape($rank).'\', '.$min_posts.')') or error(__FILE__, __LINE__);

	// Regenerate the ranks cache
	require_once PUN_ROOT.'include/cache.php';
	generate_ranks_cache();

	redirect(pun_link($pun_url['admin_ranks']), $lang_admin['Rank added'].' '.$lang_admin['Redirect']);
}


// Update a rank
else if (isset($_POST['update']))
{
	$id = intval(key($_POST['update']));

	$rank = trim($_POST['rank'][$id]);
	$min_posts = intval($_POST['min_posts'][$id]);

	if ($rank == '')
		message($lang_admin['Title message']);

	if ($min_posts < 0)
		message($lang_admin['Min posts message']);

	// Make sure there isn't already a rank with the same min_posts value
	$result = $db->query('SELECT 1 FROM '.$db->prefix.'ranks WHERE id!='.$id.' AND min_posts='.$min_posts) or error(__FILE__, __LINE__);
	if ($db->num_rows($result))
		message(sprintf($lang_admin['Same min posts message'], $min_posts));

	$db->query('UPDATE '.$db->prefix.'ranks SET rank=\''.$db->escape($rank).'\', min_posts='.$min_posts.' WHERE id='.$id) or error(__FILE__, __LINE__);

	// Regenerate the ranks cache
	require_once PUN_ROOT.'include/cache.php';
	generate_ranks_cache();

	redirect(pun_link($pun_url['admin_ranks']), $lang_admin['Rank updated'].' '.$lang_admin['Redirect']);
}


// Remove a rank
else if (isset($_POST['remove']))
{
	$id = intval(key($_POST['remove']));

	$db->query('DELETE FROM '.$db->prefix.'ranks WHERE id='.$id) or error(__FILE__, __LINE__);

	// Regenerate the ranks cache
	require_once PUN_ROOT.'include/cache.php';
	generate_ranks_cache();

	redirect(pun_link($pun_url['admin_ranks']), $lang_admin['Rank removed'].' '.$lang_admin['Redirect']);
}


// Load the cached ranks
if (file_exists(PUN_CACHE_DIR.'cache_ranks.php'))
	include PUN_CACHE_DIR.'cache_ranks.php';

if (!defined('PUN_RANKS_LOADED'))
{
	require_once PUN_ROOT.'include/cache.php';
	generate_ranks_cache();
	require PUN_CACHE_DIR.'cache_ranks.php';
}

	
// Setup breadcrumbs
$pun_page['crumbs'] = array(
	array($pun_config['o_board_title'], pun_link($pun_url['index'])),
	array($lang_admin['Forum administration'], pun_link($pun_url['admin_index'])),
	$lang_admin['Ranks']
);

define('PUN_PAGE_SECTION', 'users');
define('PUN_PAGE', 'admin-ranks');
require PUN_ROOT.'header.php';

?>
<div id="pun-main1" class="a-section a-main a-admin">

	<h1 class="main-title"><span><?php echo end($pun_page['crumbs']) ?></span></h1>

<?php echo generate_admin_menu(); ?>

	<div class="b-section b-dataform">
		<div class="c-section c-form">
		<form method="post" accept-charset="utf-8" action="<?php echo pun_link($pun_url['admin_ranks']) ?>?action=foo">
			<div class="panel">
				<input type="hidden" name="csrf_token" value="<?php echo $pun_user['csrf_token'] ?>" />
				<div class="form-intro">
					<h2 class="prefix"><span><?php echo $lang_admin['Add new rank'] ?>: </span></h2>
					<p><?php printf($lang_admin['Add rank intro'], '<strong><a href="'.pun_link($pun_url['admin_options_features']).'#fs4">'.$lang_admin['Options'].' - '.$lang_admin['Features'].'</a></strong>') ?></p>
				</div>
				<fieldset class="textset">
					<legend><span><?php echo $lang_admin['Add rank legend'] ?></span></legend>
						<p class="field">
							<label for="fld1"><?php echo $lang_admin['Rank title'] ?></label><br />
							<span class="input"><input type="text" id="fld1" name="new_rank" size="24" maxlength="50" /></span>
						</p>
						<p class="field">
							<label for="fld2"><?php echo $lang_admin['Min posts'] ?></label><br />
							<span class="input"><input type="text" id="fld2" name="new_min_posts" size="7" maxlength="7" /></span>
						</p>
				</fieldset>
				<p class="formsubmit"><input type="submit" class="button" name="add_rank" value="<?php echo $lang_admin['Add rank'] ?>" /></p>
			</div>
			<div class="divider"><hr /></div>
<?php

if (!empty($pun_ranks))
{

?>
			<div class="panel">
				<h2 class="prefix"><span><?php echo $lang_admin['Existing ranks'] ?>: </span></h2>
				<p><?php echo $lang_admin['Existing ranks intro'] ?></p>
			</div>
<?php

	foreach ($pun_ranks as $rank_key => $cur_rank)
	{

	?>
			<div class="panel subpanel">
				<fieldset class="textset" id="fs<?php echo ($rank_key + 2) ?>">
					<legend><span><?php echo $lang_admin['Rank'].' '.($rank_key + 1) ?></span></legend>
					<p class="field">
						<label for="fs<?php echo ($rank_key + 2) ?>f1"><?php echo $lang_admin['Rank title'] ?></label><br />
						<span class="input"><input type="text" id="fs<?php echo ($rank_key + 2) ?>f1" name="rank[<?php echo $cur_rank['id'] ?>]" value="<?php echo htmlspecialchars($cur_rank['rank']) ?>" size="24" maxlength="50" /></span>
					</p>
					<p class="field">
						<label for="fs<?php echo ($rank_key + 2) ?>f2"><?php echo $lang_admin['Min posts'] ?></label><br />
						<span class="input"><input type="text" id="fs<?php echo ($rank_key + 2) ?>f2" name="min_posts[<?php echo $cur_rank['id'] ?>]" value="<?php echo $cur_rank['min_posts'] ?>" size="7" maxlength="7" /></span>
					</p>
				</fieldset>
				<p class="formsubmit"><input type="submit" class="button" name="update[<?php echo $cur_rank['id'] ?>]" value="<?php echo $lang_admin['Update'] ?>" /> <input type="submit" class="button" name="remove[<?php echo $cur_rank['id'] ?>]" value="<?php echo $lang_admin['Remove'] ?>" /></p>
			</div>
	<?php

	}
}
else
{

?>
			<div class="panel">
				<div class="form-intro">
					<h2 class="prefix"><span><?php echo $lang_admin['Existing ranks'] ?>: </span></h2>
					<p><?php echo $lang_admin['Existing ranks intro'] ?></p>
				</div>
				<p><?php echo $lang_admin['No ranks'] ?></p>
			</div>
<?php

}

?>
		</form>
		</div>
	</div>

</div>
<?php

require PUN_ROOT.'footer.php';
