<?php
/***********************************************************************

  Copyright (C) 2002-2007  Rickard Andersson (rickard@punbb.org)

  This file is part of PunBB.

  PunBB is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 2 of the License,
  or (at your option) any later version.

  PunBB is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
  MA  02111-1307  USA

************************************************************************/


define('PUN_ROOT', './');
require PUN_ROOT.'include/common.php';


if ($pun_user['g_read_board'] == '0')
	message($lang_common['No view']);

// Load the post.php language file
require PUN_ROOT.'lang/'.$pun_user['language'].'/post.php';


$tid = isset($_GET['tid']) ? intval($_GET['tid']) : 0;
$fid = isset($_GET['fid']) ? intval($_GET['fid']) : 0;
if ($tid < 1 && $fid < 1 || $tid > 0 && $fid > 0)
	message($lang_common['Bad request']);


// Fetch some info about the topic and/or the forum
if ($tid)
{
	$query = array(
		'SELECT'	=> 'f.id, f.forum_name, f.moderators, f.approval, f.redirect_url, fp.post_replies, fp.post_topics, t.subject, t.closed, s.user_id AS is_subscribed',
		'FROM'		=> 'topics AS t',
		'JOINS'		=> array(
			array(
				'INNER JOIN'	=> 'forums AS f',
				'ON'			=> 'f.id=t.forum_id'
			),
			array(
				'LEFT JOIN'		=> 'forum_perms AS fp',
				'ON'			=> '(fp.forum_id=f.id AND fp.group_id='.$pun_user['g_id'].')'
			),
			array(
				'LEFT JOIN'		=> 'subscriptions AS s',
				'ON'			=> '(t.id=s.topic_id AND s.user_id='.$pun_user['id'].')'
			)
		),
		'WHERE'		=> '(fp.read_forum IS NULL OR fp.read_forum=1) AND t.id='.$tid
	);
}
else
{
	$query = array(
		'SELECT'	=> 'f.id, f.forum_name, f.moderators, f.approval, f.redirect_url, fp.post_replies, fp.post_topics',
		'FROM'		=> 'forums AS f',
		'JOINS'		=> array(
			array(
				'LEFT JOIN'		=> 'forum_perms AS fp',
				'ON'			=> '(fp.forum_id=f.id AND fp.group_id='.$pun_user['g_id'].')'
			)
		),
		'WHERE'		=> '(fp.read_forum IS NULL OR fp.read_forum=1) AND f.id='.$fid
	);
}

($hook = get_hook('po_qr_get_forum_info')) ? eval($hook) : null;
$result = $db->query_build($query) or error(__FILE__, __LINE__);

if (!$db->num_rows($result))
	message($lang_common['Bad request']);

$cur_posting = $db->fetch_assoc($result);
$is_subscribed = $tid && $cur_posting['is_subscribed'];

// Is someone trying to post into a redirect forum?
if ($cur_posting['redirect_url'] != '')
	message($lang_common['Bad request']);

// Sort out who the moderators are and if we are currently a moderator (or an admin)
$mods_array = ($cur_posting['moderators'] != '') ? unserialize($cur_posting['moderators']) : array();
$is_admmod = ($pun_user['g_id'] == PUN_ADMIN || ($pun_user['g_moderator'] == '1' && array_key_exists($pun_user['username'], $mods_array))) ? true : false;

// Do we have permission to post?
if ((($tid && (($cur_posting['post_replies'] == '' && $pun_user['g_post_replies'] == '0') || $cur_posting['post_replies'] == '0')) ||
	($fid && (($cur_posting['post_topics'] == '' && $pun_user['g_post_topics'] == '0') || $cur_posting['post_topics'] == '0')) ||
	(isset($cur_posting['closed']) && $cur_posting['closed'] == '1')) &&
	!$is_admmod)
	message($lang_common['No permission']);

// Does the post need to be approved?
$approved = ($cur_posting['approval'] == 1 && $pun_user['g_posts_approved'] == 0 && !$is_admmod) ? 0 : 1;


// Start with a clean slate
$errors = array();

// Did someone just hit "Submit" or "Preview"?
if (isset($_POST['form_sent']))
{
	// Make sure form_user is correct
	if (($pun_user['is_guest'] && $_POST['form_user'] != 'Guest') || (!$pun_user['is_guest'] && $_POST['form_user'] != $pun_user['username']))
		message($lang_common['Bad request']);

	// Flood protection
	if (!$pun_user['is_guest'] && !isset($_POST['preview']) && $pun_user['last_post'] != '' && (time() - $pun_user['last_post']) < $pun_user['g_post_flood'] && (time() - $pun_user['last_post']) >= 0)
		$errors[] = sprintf($lang_post['Flood'], $pun_user['g_post_flood']);

	// If it's a new topic
	if ($fid)
	{
		$subject = trim($_POST['req_subject']);

		if ($subject == '')
			$errors[] = $lang_post['No subject'];
		else if (pun_strlen($subject) > 70)
			$errors[] = $lang_post['Too long subject'];
		else if ($pun_config['p_subject_all_caps'] == '0' && strtoupper($subject) == $subject && !$pun_user['is_admmod'])
			$subject = ucwords(strtolower($subject));
	}

	// If the user is logged in we get the username and e-mail from $pun_user
	if (!$pun_user['is_guest'])
	{
		$username = $pun_user['username'];
		$email = $pun_user['email'];
	}
	// Otherwise it should be in $_POST
	else
	{
		$username = trim($_POST['req_username']);
		$email = strtolower(trim(($pun_config['p_force_guest_email'] == '1') ? $_POST['req_email'] : $_POST['email']));

		// Load the prof_reg.php/register.php language files
		require PUN_ROOT.'lang/'.$pun_user['language'].'/prof_reg.php';
		require PUN_ROOT.'lang/'.$pun_user['language'].'/register.php';

		// It's a guest, so we have to validate the username
		$errors = array_merge($errors, validate_username($username));

		if ($pun_config['p_force_guest_email'] == '1' || $email != '')
		{
			require PUN_ROOT.'include/email.php';
			if (!is_valid_email($email))
				$errors[] = $lang_common['Invalid e-mail'];
		}
	}

	// If we're an administrator or moderator, make sure the CSRF token in $_POST is valid
	if (($pun_user['g_id'] == PUN_ADMIN || $pun_user['g_moderator'] == '1') && $_POST['csrf_token'] !== $pun_user['csrf_token'])
		$errors[] = $lang_post['CSRF token mismatch'];

	// Clean up message from POST
	$message = pun_linebreaks(trim($_POST['req_message']));

	if ($message == '')
		$errors[] = $lang_post['No message'];
	else if (strlen($message) > 65535)
		$errors[] = $lang_post['Too long message'];
	else if ($pun_config['p_message_all_caps'] == '0' && strtoupper($message) == $message && !$pun_user['is_admmod'])
		$message = ucwords(strtolower($message));

	// Validate BBCode syntax
	if ($pun_config['p_message_bbcode'] == '1' && strpos($message, '[') !== false && strpos($message, ']') !== false)
	{
		require PUN_ROOT.'include/parser.php';
		$message = preparse_bbcode($message, $errors);
	}

	if ($db_type != 'mysql' && $db_type != 'mysqli')
		require PUN_ROOT.'include/search_idx.php';

	$hide_smilies = isset($_POST['hide_smilies']) ? 1 : 0;
	$subscribe = isset($_POST['subscribe']) ? 1 : 0;

	$now = time();

	// Did everything go according to plan?
	if (empty($errors) && !isset($_POST['preview']))
	{
		// If it's a reply
		if ($tid)
		{
			add_post(array(
					'is_guest'		=>	$pun_user['is_guest'],
					'poster'		=>	$username,
					'poster_id'		=>	$pun_user['id'],	// Always 1 for guest posts
					'poster_email'	=>	($pun_user['is_guest'] && $email != '') ? $email : null,	// Always null for non-guest posts
					'subject'		=>	$cur_posting['subject'],
					'message'		=>	$message,
					'hide_smilies'	=>	$hide_smilies,
					'approved'		=>	$approved,
					'posted'		=>	$now,
					'subscr_action'	=>	($pun_config['o_subscriptions'] == '1' && $subscribe && !$is_subscribed) ? 1 : (($pun_config['o_subscriptions'] == '1' && !$subscribe && $is_subscribed) ? 2 : 0),
					'topic_id'		=>	$tid,
					'forum_id'		=>	$cur_posting['id']
				),
				$new_pid	// By ref
			);
		}
		// If it's a new topic
		else if ($fid)
		{
			add_topic(array(
				'is_guest'		=>	$pun_user['is_guest'],
				'poster'		=>	$username,
				'poster_id'		=>	$pun_user['id'],	// Always 1 for guest posts
				'poster_email'	=>	($pun_user['is_guest'] && $email != '') ? $email : null,	// Always null for non-guest posts
				'subject'		=>	$subject,
				'message'		=>	$message,
				'hide_smilies'	=>	$hide_smilies,
				'approved'		=>	$approved,
				'posted'		=>	$now,
				'subscribe'		=>	($pun_config['o_subscriptions'] == '1' && (isset($_POST['subscribe']) && $_POST['subscribe'] == '1')),
				'forum_id'		=>	$fid
				),
				$new_tid,	// By ref
				$new_pid	// By ref
			);
		}

		if (!$pun_user['is_guest'])
		{
			// If the posting user is logged in, increment his/her post count
			$query = array(
				'UPDATE'	=> 'users',
				'SET'		=> 'num_posts=num_posts+1, last_post='.$now,
				'WHERE'		=> 'id='.$pun_user['id'],
				'PARAMS'	=> array(
					'LOW_PRIORITY'	=> 1	// MySQL only
				)
			);

			($hook = get_hook('po_qr_increment_num_posts')) ? eval($hook) : null;
			$db->query_build($query) or error(__FILE__, __LINE__);

			// Add/update the topic in our list of tracked topics
			$tracked_topics = get_tracked_topics();
			$tracked_topics[$tid ? $tid : $new_tid] = time();
			set_tracked_topics($tracked_topics);
		}

		// Redirect to forum, topic or post depending on whether we're a guest or not and whether the post/topic is approved or not
		if ($pun_user['is_guest'] && $approved == 0)
		{
			if ($fid)
				redirect(pun_link($pun_url['forum'], $fid), $lang_post['Post redirect']);
			else
				redirect(pun_link($pun_url['topic'], $tid), $lang_post['Post redirect']);
		}
		else
			redirect(pun_link($pun_url['post'], $new_pid), $lang_post['Post redirect']);
	}
}


// Are we quoting someone?
if ($tid && isset($_GET['qid']))
{
	$qid = intval($_GET['qid']);
	if ($qid < 1)
		message($lang_common['Bad request']);

	// Get the quote and quote poster
	$query = array(
		'SELECT'	=> 'p.poster, p.message',
		'FROM'		=> 'posts AS p',
		'WHERE'		=> 'approved=1 AND id='.$qid.' AND topic_id='.$tid
	);

	($hook = get_hook('po_qr_get_quote')) ? eval($hook) : null;
	$result = $db->query_build($query) or error(__FILE__, __LINE__);
	if (!$db->num_rows($result))
		message($lang_common['Bad request']);

	list($q_poster, $q_message) = $db->fetch_row($result);

	$q_message = str_replace('[img]', '[url]', $q_message);
	$q_message = str_replace('[/img]', '[/url]', $q_message);
	$q_message = htmlspecialchars($q_message);

	if ($pun_config['p_message_bbcode'] == '1')
	{
		// If username contains a square bracket, we add "" or '' around it (so we know when it starts and ends)
		if (strpos($q_poster, '[') !== false || strpos($q_poster, ']') !== false)
		{
			if (strpos($q_poster, '\'') !== false)
				$q_poster = '"'.$q_poster.'"';
			else
				$q_poster = '\''.$q_poster.'\'';
		}
		else
		{
			// Get the characters at the start and end of $q_poster
			$ends = substr($q_poster, 0, 1).substr($q_poster, -1, 1);

			// Deal with quoting "Username" or 'Username' (becomes '"Username"' or "'Username'")
			if ($ends == '\'\'')
				$q_poster = '"'.$q_poster.'"';
			else if ($ends == '""')
				$q_poster = '\''.$q_poster.'\'';
		}

		$quote = '[quote='.$q_poster.']'.$q_message.'[/quote]'."\n";
	}
	else
		$quote = '> '.sprintf($lang_common['User wrote'], $q_poster).':'."\n\n".'> '.$q_message."\n";
}


// Setup breadcrumbs
$pun_page['crumbs'][] = array($pun_config['o_board_title'], pun_link($pun_url['index']));
$pun_page['crumbs'][] = array($cur_posting['forum_name'], pun_link($pun_url['forum'], $cur_posting['id']));
if ($tid) $pun_page['crumbs'][] = array($cur_posting['subject'], pun_link($pun_url['topic'], $tid));
$pun_page['crumbs'][] = $tid ? $lang_post['Post reply'] : $lang_post['Post new topic'];

define('PUN_PAGE', 'post');
require PUN_ROOT.'header.php';

?>
<div id="pun-main1" class="a-section a-main">

	<h1 class="main-title"><span><?php echo end($pun_page['crumbs']) ?></span></h1>

	<div class="b-postform">
		<div class="c-section c-form">
<?php

// If preview selected and there are no errors
if (isset($_POST['preview']) && empty($errors))
{
	require_once PUN_ROOT.'include/parser.php';
	$preview_message = parse_message(trim($_POST['req_message']), $hide_smilies);

?>
			<div class="panel">
				<div class="form-intro">
					<h2><span><?php echo $lang_post['Post preview'] ?></span></h2>
				</div>
				<div class="post-entry">
					<div class="entry-content">
					<?php echo $preview_message."\n" ?>
					</div>
				</div>
			</div>
			<div class="divider"><hr /></div>
<?php

}

?>
			<form id="afocus" method="post" accept-charset="utf-8" action="<?php echo $tid ? pun_link($pun_url['new_reply'], $tid) : pun_link($pun_url['new_topic'], $fid) ?>">
			<div class="panel">

<?php

// Setup post help/navigation links
$post_help = array();

if ($tid && $pun_config['o_topic_review'] != '0') $post_help[] = '<li><a href="#pun-main3">'.$lang_post['Skip to review'].'</a></li>';
if ($pun_config['p_message_bbcode'] == '1') $post_help[] = '<li><a class="help" href="'.pun_link($pun_url['help'], 'bbcode').'" onclick="window.open(this.href); return false;" title="'.$lang_common['New window warn'].'">'.$lang_common['BBCode help'].'</a></li>';
if ($pun_config['p_message_img_tag'] == '1') $post_help[] = '<li><a class="help" href="'.pun_link($pun_url['help'], 'img').'" onclick="window.open(this.href); return false;" title="'.$lang_common['New window warn'].'">'.$lang_common['img tag help'].'</a></li>';
if ($pun_config['o_smilies'] == '1') $post_help[] = '<li><a class="help" href="'.pun_link($pun_url['help'], 'smilies').'" onclick="window.open(this.href); return false;" title="'.$lang_common['New window warn'].'">'.$lang_common['Smilies help'].'</a></li>';

if (!empty($post_help))
{

?>
				<div class="form-intro">
					<ul class="optionlist">
						<?php echo implode("\n\t\t\t\t\t\t", $post_help)."\n" ?>
					</ul>
				</div>
<?php

}

if ($pun_user['is_admmod'])
	echo "\t\t\t\t".'<input type="hidden" name="csrf_token" value="'.$pun_user['csrf_token'].'" />'."\n";

// Display any errors
if (!empty($errors))
{

?>
				<div class="form-warn">
					<p><strong><?php echo $lang_common['Warning'] ?></strong> <?php echo $lang_post['Post errors'] ?></p>
					<ul class="warnlist">
<?php

	while (list(, $cur_error) = each($errors))
		echo "\t\t\t\t\t\t".'<li><span>'.$cur_error.'</span></li>'."\n";

?>
					</ul>
				</div>
<?php

}

$fld_count = 0;

?>
				<p id="req-msg" class="req-warn"><?php printf($lang_common['Required warn'], '<em class="req-text">'.$lang_common['Required mark'].'<span> '.$lang_common['Required'].'</span></em>') ?></p>
				<fieldset>
					<legend><span><?php echo $lang_common['Write message legend'] ?></span></legend>
					<input type="hidden" name="form_sent" value="1" />
					<input type="hidden" name="form_user" value="<?php echo (!$pun_user['is_guest']) ? htmlspecialchars($pun_user['username']) : 'Guest'; ?>" />
<?php

if ($pun_user['is_guest'])
{
	$email_form_name = ($pun_config['p_force_guest_email'] == '1') ? 'req_email' : 'email';

?>
					<p class="field required">
						<label for="fld<?php echo ++$fld_count ?>"><?php echo $lang_post['Guest name'] ?> <em class="req-text"><?php echo $lang_common['Required mark'].' <span>'.$lang_common['Required'].'</span>' ?></em></label><br />
						<span class="input"><input type="text" id="fld<?php echo $fld_count ?>" name="req_username" value="<?php if (isset($_POST['req_username'])) echo htmlspecialchars($username); ?>" size="25" maxlength="25" /></span>
					</p>
					<p class="field<?php if ($pun_config['p_force_guest_email'] == '1') echo ' required' ?>">
						<label for="fld<?php echo ++$fld_count ?>"><?php echo $lang_common['E-mail']; if ($pun_config['p_force_guest_email'] == '1') echo ' <em class="req-text">'.$lang_common['Required mark'].' <span>'.$lang_common['Required'].'</span></em>' ?></label><br />
						<span class="input"><input type="text" id="fld<?php echo $fld_count ?>" name="<?php echo $email_form_name ?>" value="<?php if (isset($_POST[$email_form_name])) echo htmlspecialchars($email); ?>" size="50" maxlength="80" /></span>
					</p>
<?php

}

if ($fid)
{

?>
					<p class="longfield required">
						<label for="fld<?php echo ++$fld_count ?>"><?php echo $lang_common['Subject'] ?> <em class="req-text"><?php echo $lang_common['Required mark'].' <span>'.$lang_common['Required'].'</span>' ?></em></label><br />
						<span class="input"><input id="fld<?php echo $fld_count ?>" type="text" name="req_subject" value="<?php if (isset($_POST['req_subject'])) echo htmlspecialchars($subject); ?>" size="80" maxlength="70" /></span>
					</p>
<?php

}

?>
					<p id="compose" class="areafield required">
						<label for="fld<?php echo ++$fld_count ?>"><?php echo $lang_common['Message'] ?> <em class="req-text"><?php echo $lang_common['Required mark'].' <span>'.$lang_common['Required'].'</span>' ?></em></label><br />
						<span class="input"><textarea id="fld<?php echo $fld_count ?>" name="req_message" rows="20" cols="95"><?php echo isset($_POST['req_message']) ? htmlspecialchars($message) : (isset($quote) ? $quote : ''); ?></textarea></span>
					</p>
<?php

$checkboxes = array();
if ($pun_config['o_smilies'] == '1')
	$checkboxes[] = '<span class="input"><input type="checkbox" id="fld'.(++$fld_count).'" name="hide_smilies" value="1"'.(isset($_POST['hide_smilies']) ? ' checked="checked"' : '').' /> <label for="fld'.$fld_count.'">'.$lang_post['Hide smilies'].'</label></span><br />';

// Check/uncheck the checkbox for subscriptions depending on scenario
if (!$pun_user['is_guest'] && $pun_config['o_subscriptions'] == '1')
{
	$checked = false;

	// If it's a preview
	if (isset($_POST['preview']))
		$checked = isset($_POST['subscribe']) ? true : false;
	// If auto subscribed
	else if ($pun_user['auto_notify'])
		$checked = true;
	// If already subscribed to the topic
	else if ($is_subscribed)
		$checked = true;

	$checkboxes[] = '<span class="input"><input type="checkbox" id="fld'.(++$fld_count).'" name="subscribe" value="1"'.($checked ? ' checked="checked"' : '').' /> <label for="flf'.$fld_count.'">'.($is_subscribed ? $lang_post['Stay subscribed'] : $lang_post['Subscribe']).'</label></span><br />';
}

if (!empty($checkboxes))
{

?>
					<fieldset class="checkset">
						<legend><span><?php echo $lang_post['Post options'] ?></span></legend>
						<?php echo implode("\n\t\t\t\t\t\t", $checkboxes)."\n"; ?>
					</fieldset>
				</fieldset>
<?php

}

if ($approved == 0)
{

?>
				<div class="form-warn">
					<p><strong><?php echo $lang_common['Warning'] ?></strong> <?php echo $pun_user['is_guest'] ? $lang_common['Approval guest warning'] : $lang_common['Approval warning'] ?></p>
				</div>
<?php

}

?>
				<p class="formsubmit"><input type="submit" class="button" name="submit" value="<?php echo $lang_common['Submit'] ?>" accesskey="s" title="<?php echo $lang_common['Submit title'] ?>" /> <input type="submit" class="button" name="preview" value="<?php echo $lang_post['Preview'] ?>" accesskey="p" title="<?php echo $lang_post['Preview title'] ?>" /></p>
			</div>
			</form>
		</div>
	</div>

</div>
<?php


// Check if the topic review is to be displayed
if ($tid && $pun_config['o_topic_review'] != '0')
{
	require_once PUN_ROOT.'include/parser.php';

	// Get posts to display in topic review
	$query = array(
		'SELECT'	=> 'p.id, p.poster, p.message, p.hide_smilies, p.posted',
		'FROM'		=> 'posts AS p',
		'WHERE'		=> 'topic_id='.$tid,
		'ORDER BY'	=>	'id DESC',
		'LIMIT'		=>	$pun_config['o_topic_review']
	);

	($hook = get_hook('po_qr_get_topic_review_posts')) ? eval($hook) : null;
	$result = $db->query_build($query) or error(__FILE__, __LINE__);

?>
<div id="pun-main2" class="a-section a-main">

	<h2 class="main-title"><span><?php echo $lang_post['Topic review'] ?></span></h2>

	<div class="b-section b-topic">
<?php

	$post_count = 0;

	while ($cur_post = $db->fetch_assoc($result))
	{
		++$post_count;
		$post_class = 'post';

		$cur_post['message'] = parse_message($cur_post['message'], $cur_post['hide_smilies']);

?>
		<div class="post">
			<div class="posthead">
				<h3><strong><?php echo $post_count ?></strong> <cite class="author"><span><?php echo $lang_common['Posted by'] ?></span> <em><?php echo $cur_post['poster'] ?></em></cite> <abbr class="published" title="<?php echo date("Y-m-d\Th:i:s\Z", $cur_post['posted']) ?>"><a class="permalink" rel="bookmark" title="<?php echo $lang_post['Permalink post'] ?>" href="<?php echo pun_link($pun_url['post'], $cur_post['id']) ?>"><?php echo format_time($cur_post['posted']) ?></a></abbr></h3>
			</div>
			<div class="postmain">
				<div class="postbody">
					<div class="post-entry">
						<div class="entry-content">
						<?php echo $cur_post['message']."\n" ?>
						</div>
					</div>
				</div>
			</div>
		</div>
<?php

	}

?>
	</div>

</div>
<?php

}

require PUN_ROOT.'footer.php';
