<?php
/***********************************************************************

  Copyright (C) 2002-2007  Rickard Andersson (rickard@punbb.org)

  This file is part of PunBB.

  PunBB is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 2 of the License,
  or (at your option) any later version.

  PunBB is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
  MA  02111-1307  USA

************************************************************************/


define('PUN_ROOT', './');
require PUN_ROOT.'include/common.php';


// If we are logged in, we shouldn't be here
if (!$pun_user['is_guest'])
{
	header('Location: '.pun_link($pun_url['index']));
	exit;
}

// Load the register.php language file
require PUN_ROOT.'lang/'.$pun_user['language'].'/register.php';


if ($pun_config['o_regs_allow'] == '0')
	message($lang_register['No new regs']);


// User pressed the cancel button
if (isset($_GET['cancel']))
	redirect(pun_link($pun_url['index']), $lang_register['Reg cancel redirect']);


// Show the rules
else if ($pun_config['o_rules'] == '1' && !isset($_GET['agree']) && !isset($_POST['form_sent']))
{
	// Setup breadcrumbs
	$pun_page['crumbs'] = array(
		array($pun_config['o_board_title'], pun_link($pun_url['index'])),
		array($lang_common['Register'], pun_link($pun_url['register'])),
		$lang_common['Rules']
	);

	define('PUN_PAGE', 'rules');
	require PUN_ROOT.'header.php';

?>
<div class="a-section a-main">

	<h1 class="main-title"><span><?php echo end($pun_page['crumbs']) ?></span></h1>

	<div class="b-section b-dataform">
		<div class="c-section c-form">
			<div class="panel">
				<div class="userbox">
					<?php echo $pun_config['o_rules_message'] ?>
				</div>
			</div>
			<form method="get" accept-charset="utf-8" action="<?php echo $base_url ?>/register.php">
			<div class="panel">
				<p><strong>In order to register you must first agree to the rules set out above</strong></p>
				<p class="formsubmit"><input type="submit" class="button" name="agree" value="<?php echo $lang_register['Agree'] ?>" /> <input type="submit" class="button" name="cancel" value="<?php echo $lang_register['Cancel'] ?>" /></p>
			</div>
			</form>
		</div>
	</div>

</div>
<?php

	require PUN_ROOT.'footer.php';
}


else if (isset($_POST['form_sent']))
{
	// Check that someone from this IP didn't register a user within the last hour (DoS prevention)
	$query = array(
		'SELECT'	=> '1',
		'FROM'		=> 'users AS u',
		'WHERE'		=> 'u.registration_ip=\''.get_remote_address().'\' AND u.registered>'.(time() - 3600)
	);

	($hook = get_hook('rg_qr_check_register_flood')) ? eval($hook) : null;
	$result = $db->query_build($query) or error(__FILE__, __LINE__);
	if ($db->num_rows($result))
		message($lang_register['Registration flood']);

	$username = trim($_POST['req_username']);
	$email1 = strtolower(trim($_POST['req_email1']));

	if ($pun_config['o_regs_verify'] == '1')
	{
		$email2 = strtolower(trim($_POST['req_email2']));

		$password1 = random_key(8, true);
		$password2 = $password1;
	}
	else
	{
		$password1 = trim($_POST['req_password1']);
		$password2 = trim($_POST['req_password2']);
	}

	// Validate the username
	$errors = validate_username($username);
	if (!empty($errors))
		message(current($errors));

	// ... and the password
	if (pun_strlen($password1) < 4)
		message($lang_register['Pass too short']);
	else if ($password1 != $password2)
		message($lang_register['Pass not match']);

	// ... and the e-mail address
	require PUN_ROOT.'include/email.php';

	if (!is_valid_email($email1))
		message($lang_common['Invalid e-mail']);
	else if ($pun_config['o_regs_verify'] == '1' && $email1 != $email2)
		message($lang_register['E-mail not match']);

	// Check if it's a banned e-mail address
	$banned_email = is_banned_email($email1);
	if ($banned_email && $pun_config['p_allow_banned_email'] == '0')
		message($lang_register['Banned e-mail']);

	// Check if someone else already has registered with that e-mail address
	$dupe_list = array();

	$query = array(
		'SELECT'	=> 'u.username',
		'FROM'		=> 'users AS u',
		'WHERE'		=> 'u.email=\''.$email1.'\''
	);

	($hook = get_hook('rg_qr_check_email_dupe')) ? eval($hook) : null;
	$result = $db->query_build($query) or error(__FILE__, __LINE__);
	if ($db->num_rows($result))
	{
		if ($pun_config['p_allow_dupe_email'] == '0')
			message($lang_register['Dupe e-mail']);

		while ($cur_dupe = $db->fetch_assoc($result))
			$dupe_list[] = $cur_dupe['username'];
	}

	// Make sure we got a valid language string
	if (isset($_POST['language']))
	{
		$language = preg_replace('#[\.\\\/]#', '', $_POST['language']);
		if (!file_exists(PUN_ROOT.'lang/'.$language.'/common.php'))
			message($lang_common['Bad request']);
	}
	else
		$language = $pun_config['o_default_lang'];

	$save_pass = (!isset($_POST['save_pass']) || $_POST['save_pass'] != '1') ? 0 : 1;
	$email_setting = intval($_POST['email_setting']);
	if ($email_setting < 0 || $email_setting > 2) $email_setting = 1;

	$initial_group_id = ($pun_config['o_regs_verify'] == '0') ? $pun_config['o_default_user_group'] : PUN_UNVERIFIED;
	$salt = random_key(12);
	$password_hash = sha1($salt.sha1($password1));

	// Insert the new user into the database. We do this now to get the last inserted id for later use.
	$errors = add_user(array(
			'username'				=>	$username,
			'group_id'				=>	$initial_group_id,
			'salt'					=>	$salt,
			'password'				=>	$password1,
			'password_hash'			=>	$password_hash,
			'email'					=>	$email1,
			'email_setting'			=>	$email_setting,
			'save_pass'				=>	$save_pass,
			'timezone'				=>	$_POST['timezone'],
			'language'				=>	$language,
			'style'					=>	$pun_config['o_default_style'],
			'registered'			=>	time(),
			'registration_ip'		=>	get_remote_address(),
			'activate_key'			=>	($pun_config['o_regs_verify'] == '1') ? '\''.random_key(8, true).'\'' : 'NULL',
			'require_verification'	=>	($pun_config['o_regs_verify'] == '1'),
			'notify_admins'			=>	($pun_config['o_regs_report'] == '1')
		),
		$new_uid	// By ref
	);

	// If we previously found out that the e-mail was banned
	if ($banned_email && $pun_config['o_mailing_list'] != '')
	{
		$mail_subject = 'Alert - Banned e-mail detected';
		$mail_message = 'User \''.$username.'\' registered with banned e-mail address: '.$email1."\n\n".'User profile: '.pun_link($pun_url['user'], $new_uid)."\n\n".'-- '."\n".'Forum Mailer'."\n".'(Do not reply to this message)';

		pun_mail($pun_config['o_mailing_list'], $mail_subject, $mail_message);
	}

	// If we previously found out that the e-mail was a dupe
	if (!empty($dupe_list) && $pun_config['o_mailing_list'] != '')
	{
		$mail_subject = 'Alert - Duplicate e-mail detected';
		$mail_message = 'User \''.$username.'\' registered with an e-mail address that also belongs to: '.implode(', ', $dupe_list)."\n\n".'User profile: '.pun_link($pun_url['user'], $new_uid)."\n\n".'-- '."\n".'Forum Mailer'."\n".'(Do not reply to this message)';

		pun_mail($pun_config['o_mailing_list'], $mail_subject, $mail_message);
	}

	// Must the user verify the registration or do we log him/her in right now?
	if ($pun_config['o_regs_verify'] == '1')
		message(sprintf($lang_register['Reg e-mail'], '<a href="mailto:'.$pun_config['o_admin_email'].'">'.$pun_config['o_admin_email'].'</a>'));

	pun_setcookie($new_uid, $password_hash, ($save_pass != '0') ? time() + 31536000 : 0);

	redirect('index.php', $lang_register['Reg complete']);
}


// Setup breadcrumbs
$pun_page['crumbs'] = array(
	array($pun_config['o_board_title'], pun_link($pun_url['index'])),
	$lang_common['Register']
);

define('PUN_PAGE', 'register');
require PUN_ROOT.'header.php';

$set_count = 0;
$fld_count = 0;
$req_field = '<em class="req-text">&#160;<span>'.$lang_common['Required'].'</span></em>';

// Setup form help/instructions
$register_info[] = '<li id="req-msg" class="warn req-warn">'.sprintf($lang_common['Required warn'], $req_field).'</li>';

if ($pun_config['o_regs_verify'] != '0')
	$register_info[] = '<li class="warn">'.$lang_register['E-mail info'].'</li>';


?>
<div id="pun-main" class="main">

	<h1><span><?php echo end($pun_page['crumbs']) ?></span></h1>

	<div class="main-form">
		<h2><span><?php printf($lang_register['Register at'], htmlspecialchars($pun_config['o_board_title'])) ?></span></h2>
		<form class="fm-form" id="afocus" method="post" accept-charset="utf-8" action="<?php echo $base_url ?>/register.php?action=register">
			<div class="hidden">
				<input type="hidden" name="form_sent" value="1" />
			</div>
			<div class="form-info" style="overflow:hidden;background:#F2F8FD;">
				<p><?php echo $lang_register['Register intro'] ?></p>
			</div>
			<div class="form-info">
				<ul>
					<?php echo implode("\n\t\t\t\t\t", $register_info)."\n" ?>
				</ul>
			</div>
			<fieldset class="fm-set set<?php echo ++$set_count ?>">
				<legend class="fm-legend"><strong><?php echo $lang_register['Reg required legend'] ?></strong></legend>
				<div class="text required">
					<label for="fld<?php echo ++$fld_count ?>">
						<span class="fm-label"><?php echo $lang_register['Username'] ?></span><?php echo $req_field ?><br />
						<span class="fm-input"><input type="text" id="fld<?php echo $fld_count ?>" name="req_username" size="35" maxlength="25" /></span><br />
				 		<small><?php echo $lang_register['Username help'] ?></small>
				 	</label>
				</div>
<?php if ($pun_config['o_regs_verify'] == '0'): ?>				<div class="text required">
					<label for="fld<?php echo ++$fld_count ?>">
						<span class="fm-label"><?php echo $lang_register['Password'] ?></span><?php echo $req_field ?><br />
						<span class="fm-input"><input type="password" id="fld<?php echo $fld_count ?>" name="req_password1" size="35" /></span><br />
						<small><?php echo $lang_register['Password help'] ?></small>
					</label>
				</div>
				<div class="text required">
					<label for="fld<?php echo ++$fld_count ?>">
						<span class="fm-label"><?php echo $lang_register['Confirm password'] ?></span><?php echo $req_field ?><br />
						<span class="fm-input"><input type="password" id="fld<?php echo $fld_count ?>" name="req_password2" size="35" /></span><br />
						<small><?php echo $lang_register['Confirm password help'] ?></small>
					</label>
				</div>
<?php endif; ?>				<div class="text required">
					<label for="fld<?php echo ++$fld_count ?>">
						<span class="fm-label"><?php echo $lang_register['E-mail'] ?></span><?php echo $req_field ?><br />
						<span class="fm-input"><input type="text" id="fld<?php echo $fld_count ?>" name="req_email1" size="35" maxlength="80" /></span><br />
						<small><?php echo $lang_register['E-mail help'] ?></small>
					</label>
				</div>
<?php if ($pun_config['o_regs_verify'] == '1'): ?>				<div class="text required">
					<label for="fld<?php echo ++$fld_count ?>">
						<span class="fm-label"><?php echo $lang_register['Confirm e-mail'] ?></span><?php echo $req_field ?><br />
						<span class="fm-input"><input type="text" id="fld<?php echo $fld_count ?>" name="req_email2" size="35" maxlength="80" /></span>
						<small><?php echo $lang_register['Confirm e-mail help'] ?></small>
					</label>
				</div>
<?php endif; ?>			</fieldset>
			<fieldset class="fm-set set<?php echo ++$set_count ?>">
				<legend class="fm-legend"><strong><?php echo $lang_register['Reg local legend'] ?></strong></legend>
<?php

		$languages = array();
		$d = dir(PUN_ROOT.'lang');
		while (($entry = $d->read()) !== false)
		{
			if ($entry != '.' && $entry != '..' && is_dir(PUN_ROOT.'lang/'.$entry) && file_exists(PUN_ROOT.'lang/'.$entry.'/common.php'))
				$languages[] = $entry;
		}
		$d->close();

		// Only display the language selection box if there's more than one language available
		if (count($languages) > 1)
		{
			natcasesort($languages);

?>
				<div class="select">
					<label for="fld<?php echo ++$fld_count ?>">
						<span class="fm-label"><?php echo $lang_register['Language'] ?></span><br />
						<span class="fm-input"><select id="fld<?php echo $fld_count ?>" name="language">
<?php

			while (list(, $temp) = @each($languages))
			{
				if ($pun_config['o_default_lang'] == $temp)
					echo "\t\t\t\t\t\t".'<option value="'.$temp.'" selected="selected">'.$temp.'</option>'."\n";
				else
					echo "\t\t\t\t\t\t".'<option value="'.$temp.'">'.$temp.'</option>'."\n";
			}

?>
						</select></span>
					</label>
				</div>
<?php

		}

?>
				<div class="select">
					<label for="fld<?php echo ++$fld_count ?>">
						<span class="fm-label"><?php echo $lang_register['Your timezone'] ?></span><br />
						<span class="fm-input"><select id="fld<?php echo $fld_count ?>" name="timezone">
						<option value="-12"<?php if ($pun_config['o_default_timezone'] == -12 ) echo ' selected="selected"' ?>>-12</option>
						<option value="-11"<?php if ($pun_config['o_default_timezone'] == -11) echo ' selected="selected"' ?>>-11</option>
						<option value="-10"<?php if ($pun_config['o_default_timezone'] == -10) echo ' selected="selected"' ?>>-10</option>
						<option value="-9.5"<?php if ($pun_config['o_default_timezone'] == -9.5) echo ' selected="selected"' ?>>-9.5</option>
						<option value="-9"<?php if ($pun_config['o_default_timezone'] == -9 ) echo ' selected="selected"' ?>>-09</option>
						<option value="-8.5"<?php if ($pun_config['o_default_timezone'] == -8.5) echo ' selected="selected"' ?>>-8.5</option>
						<option value="-8"<?php if ($pun_config['o_default_timezone'] == -8 ) echo ' selected="selected"' ?>>-08 PST</option>
						<option value="-7"<?php if ($pun_config['o_default_timezone'] == -7 ) echo ' selected="selected"' ?>>-07 MST</option>
						<option value="-6"<?php if ($pun_config['o_default_timezone'] == -6 ) echo ' selected="selected"' ?>>-06 CST</option>
						<option value="-5"<?php if ($pun_config['o_default_timezone'] == -5 ) echo ' selected="selected"' ?>>-05 EST</option>
						<option value="-4"<?php if ($pun_config['o_default_timezone'] == -4 ) echo ' selected="selected"' ?>>-04 AST</option>
						<option value="-3.5"<?php if ($pun_config['o_default_timezone'] == -3.5) echo ' selected="selected"' ?>>-3.5</option>
						<option value="-3"<?php if ($pun_config['o_default_timezone'] == -3 ) echo ' selected="selected"' ?>>-03 ADT</option>
						<option value="-2"<?php if ($pun_config['o_default_timezone'] == -2 ) echo ' selected="selected"' ?>>-02</option>
						<option value="-1"<?php if ($pun_config['o_default_timezone'] == -1) echo ' selected="selected"' ?>>-01</option>
						<option value="0"<?php if ($pun_config['o_default_timezone'] == 0) echo ' selected="selected"' ?>>00 GMT</option>
						<option value="1"<?php if ($pun_config['o_default_timezone'] == 1) echo ' selected="selected"' ?>>+01 CET</option>
						<option value="2"<?php if ($pun_config['o_default_timezone'] == 2 ) echo ' selected="selected"' ?>>+02</option>
						<option value="3"<?php if ($pun_config['o_default_timezone'] == 3 ) echo ' selected="selected"' ?>>+03</option>
						<option value="3.5"<?php if ($pun_config['o_default_timezone'] == 3.5 ) echo ' selected="selected"' ?>>+03.5</option>
						<option value="4"<?php if ($pun_config['o_default_timezone'] == 4 ) echo ' selected="selected"' ?>>+04</option>
						<option value="4.5"<?php if ($pun_config['o_default_timezone'] == 4.5 ) echo ' selected="selected"' ?>>+04.5</option>
						<option value="5"<?php if ($pun_config['o_default_timezone'] == 5 ) echo ' selected="selected"' ?>>+05</option>
						<option value="5.5"<?php if ($pun_config['o_default_timezone'] == 5.5 ) echo ' selected="selected"' ?>>+05.5</option>
						<option value="6"<?php if ($pun_config['o_default_timezone'] == 6 ) echo ' selected="selected"' ?>>+06</option>
						<option value="6.5"<?php if ($pun_config['o_default_timezone'] == 6.5 ) echo ' selected="selected"' ?>>+06.5</option>
						<option value="7"<?php if ($pun_config['o_default_timezone'] == 7 ) echo ' selected="selected"' ?>>+07</option>
						<option value="8"<?php if ($pun_config['o_default_timezone'] == 8 ) echo ' selected="selected"' ?>>+08</option>
						<option value="9"<?php if ($pun_config['o_default_timezone'] == 9 ) echo ' selected="selected"' ?>>+09</option>
						<option value="9.5"<?php if ($pun_config['o_default_timezone'] == 9.5 ) echo ' selected="selected"' ?>>+09.5</option>
						<option value="10"<?php if ($pun_config['o_default_timezone'] == 10) echo ' selected="selected"' ?>>+10</option>
						<option value="10.5"<?php if ($pun_config['o_default_timezone'] == 10.5 ) echo ' selected="selected"' ?>>+10.5</option>
						<option value="11"<?php if ($pun_config['o_default_timezone'] == 11) echo ' selected="selected"' ?>>+11</option>
						<option value="11.5"<?php if ($pun_config['o_default_timezone'] == 11.5 ) echo ' selected="selected"' ?>>+11.5</option>
						<option value="12"<?php if ($pun_config['o_default_timezone'] == 12 ) echo ' selected="selected"' ?>>+12</option>
						<option value="13"<?php if ($pun_config['o_default_timezone'] == 13 ) echo ' selected="selected"' ?>>+13</option>
						<option value="14"<?php if ($pun_config['o_default_timezone'] == 14 ) echo ' selected="selected"' ?>>+14</option>
						</select></span>
					</label>
				</div>
				<fieldset class="group">
					<legend><span class="fm-label"><?php echo $lang_register['Adjust for DST'] ?></span></legend>
					<div class="fm-item"><label for="fld<?php echo ++$fld_count ?>"><input type="checkbox" id="fld<?php echo $fld_count ?>" /> <?php echo $lang_register['DST label'] ?></label></div>
				</fieldset>
			</fieldset>
			<fieldset class="fm-set set<?php echo ++$set_count ?>">
				<legend class="fm-legend"><strong><?php echo $lang_register['Reg other legend'] ?></strong></legend>
				<fieldset class="group">
					<legend><span><?php echo $lang_register['E-mail settings legend'] ?></span></legend>
					<div class="fm-item"><label for="fld<?php echo ++$fld_count ?>"><input type="radio" id="fld<?php echo $fld_count ?>" name="email_setting" value="0" /> <?php echo $lang_register['E-mail setting 1'] ?></label></div>
					<div class="fm-item"><label for="fld<?php echo ++$fld_count ?>"><input type="radio" id="fld<?php echo $fld_count ?>" name="email_setting" value="1" checked="checked" /> <?php echo $lang_register['E-mail setting 2'] ?></label></div>
					<div class="fm-item"><label for="fld<?php echo ++$fld_count ?>"><input type="radio" id="fld<?php echo $fld_count ?>" name="email_setting" value="2" /> <?php echo $lang_register['E-mail setting 3'] ?></label></div>
				</fieldset>
				<fieldset class="group">
					<legend><span class="fm-label"><?php echo $lang_register['Persistent login legend'] ?>:</span></legend>
						<div class="fm-item"><label for="fld<?php echo ++$fld_count ?>"><input type="checkbox" id="fld<?php echo $fld_count ?>" name="save_pass" value="1" checked="checked" /> <?php echo $lang_register['Save user/pass'] ?></label></div>
				</fieldset>
			</fieldset>
			<div class="form-buttons">
				<span class="submit"><input type="submit" class="button" name="register" value="<?php echo $lang_register['Register'] ?>" /></span>
			</div>
		</form>
	</div>

</div>
<?php

require PUN_ROOT.'footer.php';