<?php
/***********************************************************************

  Copyright (C) 2002-2007  Rickard Andersson (rickard@punbb.org)

  This file is part of PunBB.

  PunBB is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 2 of the License,
  or (at your option) any later version.

  PunBB is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
  MA  02111-1307  USA

************************************************************************/


define('PUN_ROOT', './');
require PUN_ROOT.'include/common.php';


// Load the search.php language file
require PUN_ROOT.'lang/'.$pun_user['language'].'/search.php';

if ($pun_user['g_read_board'] == '0')
	message($lang_common['No view']);
else if ($pun_user['g_search'] == '0')
	message($lang_search['No search permission']);


// Detect two byte character sets
$multibyte = (isset($lang_common['lang_multibyte']) && $lang_common['lang_multibyte']) ? true : false;


// Figure out what to do :-)
if (isset($_GET['action']) || isset($_GET['search_id']))
{
	// Grab and validate all the submitted data
	$action = (isset($_GET['action'])) ? $_GET['action'] : null;
	$value = (isset($_GET['value'])) ? intval($_GET['value']) : 86400;
	$forum = (isset($_GET['forum'])) ? intval($_GET['forum']) : -1;
	$sort_dir = (isset($_GET['sort_dir'])) ? (($_GET['sort_dir'] == 'DESC') ? 'DESC' : 'ASC') : 'DESC';
	if (isset($search_id)) unset($search_id);

	// If a search_id was supplied
	if (isset($_GET['search_id']))
	{
		$search_id = intval($_GET['search_id']);
		if ($db_type == 'mysql' || $db_type == 'mysqli')
			message($lang_common['Bad request']);
		if ($search_id < 1)
			message($lang_common['Bad request']);
	}
	// If it's a regular search (keywords and/or author)
	else if ($action == 'search')
	{
		$keywords = (isset($_GET['keywords'])) ? strtolower(trim($_GET['keywords'])) : null;
		$author = (isset($_GET['author'])) ? strtolower(trim($_GET['author'])) : null;

		if (preg_match('#^[\*%]+$#', $keywords) || pun_strlen(str_replace(array('*', '%'), '', $keywords)) < 3)
			$keywords = '';

		if (preg_match('#^[\*%]+$#', $author) || pun_strlen(str_replace(array('*', '%'), '', $author)) < 2)
			$author = '';

		if (!$keywords && !$author)
			message($lang_search['No terms']);

		if ($author)
			$author = str_replace('*', '%', $author);

		$show_as = (isset($_GET['show_as'])) ? $_GET['show_as'] : 'posts';
		$sort_by = (isset($_GET['sort_by'])) ? intval($_GET['sort_by']) : null;
		$search_in = (!isset($_GET['search_in']) || $_GET['search_in'] == 'all') ? 0 : (($_GET['search_in'] == 'message') ? 1 : -1);
	}
	// If it's a user search (by id)
	else if ($action == 'show_user_posts' || $action == 'show_user_topics')
	{
		$user_id = isset($_GET['user_id']) ? intval($_GET['user_id']) : 0;
		if ($user_id < 2)
			message($lang_common['Bad request']);
	}
	else
	{
		if ($action != 'show_new' && $action != 'show_recent' && $action != 'show_unanswered' && $action != 'show_subscriptions')
			message($lang_common['Bad request']);
	}

	// First of all lets find out if we need to cache the results, we only need to do this for detailed queries, not quicksearches or for mysql(i)
	if ($db_type != 'mysql' && $db_type != 'mysqli' && (isset($keywords) || isset($author)))
	{
		// We need to grab results, insert them into the cache and reload with a search id before showing them
		$keyword_results = $author_results = array();

		// Search a specific forum?
		$forum_sql = ($forum != -1 || ($forum == -1 && $pun_config['o_search_all_forums'] == '0')) ? ' AND t.forum_id = '.$forum : '';

		// If it's a search for keywords
		if ($keywords)
		{
			$stopwords = (array)@file(PUN_ROOT.'lang/'.$pun_user['language'].'/stopwords.txt');
			$stopwords = array_map('trim', $stopwords);

			// Are we searching for multibyte charset text?
			if ($multibyte)
			{
				// Strip out excessive whitespace
				$keywords = trim(preg_replace('#\s+#', ' ', $keywords));

				$keywords_array = explode(' ', $keywords);
			}
			else
			{
				// Filter out non-alphabetical chars
				$noise_match = array('^', '$', '&', '(', ')', '<', '>', '`', '\'', '"', '|', ',', '@', '_', '?', '%', '~', '[', ']', '{', '}', ':', '\\', '/', '=', '#', '\'', ';', '!', '¤');
				$noise_replace = array(' ', ' ', ' ', ' ', ' ', ' ', ' ', '',  '',   ' ', ' ', ' ', ' ', '',  ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', '' ,  ' ', ' ', ' ', ' ',  ' ', ' ', ' ');
				$keywords = str_replace($noise_match, $noise_replace, $keywords);

				// Strip out excessive whitespace
				$keywords = trim(preg_replace('#\s+#', ' ', $keywords));

				// Fill an array with all the words
				$keywords_array = explode(' ', $keywords);

				if (empty($keywords_array))
					message($lang_search['No hits']);

				while (list($i, $word) = @each($keywords_array))
				{
					$num_chars = pun_strlen($word);

					if ($num_chars < 3 || $num_chars > 20 || in_array($word, $stopwords))
						unset($keywords_array[$i]);
				}

				// Should we search in message body or topic subject specifically?
				$search_in_cond = ($search_in) ? (($search_in > 0) ? ' AND m.subject_match = 0' : ' AND m.subject_match = 1') : '';
			}

			$word_count = 0;
			$match_type = 'and';
			$result_list = array();
			@reset($keywords_array);
			while (list(, $cur_word) = @each($keywords_array))
			{
				switch ($cur_word)
				{
					case 'and':
					case 'or':
					case 'not':
						$match_type = $cur_word;
						break;

					default:
					{
						// Are we searching for multibyte charset text?
						if ($multibyte)
						{
							$cur_word = $db->escape('%'.str_replace('*', '', $cur_word).'%');
							$cur_word_like = ($db_type == 'pgsql') ? 'ILIKE \''.$cur_word.'\'' : 'LIKE \''.$cur_word.'\'';

							if ($search_in > 0)
								$sql = 'SELECT id FROM '.$db->prefix.'posts WHERE message '.$cur_word_like;
							else if ($search_in < 0)
								$sql = 'SELECT p.id FROM '.$db->prefix.'posts AS p INNER JOIN '.$db->prefix.'topics AS t ON t.id=p.topic_id WHERE t.subject '.$cur_word_like.' GROUP BY p.id, t.id';
							else
								$sql = 'SELECT p.id FROM '.$db->prefix.'posts AS p INNER JOIN '.$db->prefix.'topics AS t ON t.id=p.topic_id WHERE p.message '.$cur_word_like.' OR t.subject '.$cur_word_like.' GROUP BY p.id, t.id';
						}
						else
						{
							$cur_word = str_replace('*', '%', $cur_word);
							$sql = 'SELECT m.post_id FROM '.$db->prefix.'search_words AS w INNER JOIN '.$db->prefix.'search_matches AS m ON m.word_id = w.id WHERE w.word LIKE \''.$cur_word.'\''.$search_in_cond;
						}

						$result = $db->query($sql, true) or error(__FILE__, __LINE__);

						$row = array();
						while ($temp = $db->fetch_row($result))
						{
							$row[$temp[0]] = 1;

							if (!$word_count)
								$result_list[$temp[0]] = 1;
							else if ($match_type == 'or')
								$result_list[$temp[0]] = 1;
							else if ($match_type == 'not')
								$result_list[$temp[0]] = 0;
						}

						if ($match_type == 'and' && $word_count)
						{
							@reset($result_list);
							while (list($post_id,) = @each($result_list))
							{
								if (!isset($row[$post_id]))
									$result_list[$post_id] = 0;
							}
						}

						++$word_count;
						$db->free_result($result);

						break;
					}
				}
			}

			@reset($result_list);
			while (list($post_id, $matches) = @each($result_list))
			{
				if ($matches)
					$keyword_results[] = $post_id;
			}

			unset($result_list);
		}

		// If it's a search for author name (and that author name isn't Guest)
		if ($author && strtolower($author) != 'guest' && strtolower($author) != strtolower($lang_common['Guest']))
		{
			switch ($db_type)
			{
				case 'pgsql':
					$result = $db->query('SELECT id FROM '.$db->prefix.'users WHERE username ILIKE \''.$db->escape($author).'\'') or error(__FILE__, __LINE__);
					break;

				default:
					$result = $db->query('SELECT id FROM '.$db->prefix.'users WHERE username LIKE \''.$db->escape($author).'\'') or error(__FILE__, __LINE__);
					break;
			}

			if ($db->num_rows($result))
			{
				$user_ids = '';
				while ($row = $db->fetch_row($result))
					$user_ids .= (($user_ids != '') ? ',' : '').$row[0];

				$result = $db->query('SELECT id FROM '.$db->prefix.'posts WHERE poster_id IN('.$user_ids.')') or error(__FILE__, __LINE__);

				$search_ids = array();
				while ($row = $db->fetch_row($result))
					$author_results[] = $row[0];

				$db->free_result($result);
			}
		}


		if ($author && $keywords)
		{
			// If we searched for both keywords and author name we want the intersection between the results
			$search_ids = array_intersect($keyword_results, $author_results);
			unset($keyword_results, $author_results);
		}
		else if ($keywords)
			$search_ids = $keyword_results;
		else
			$search_ids = $author_results;

		$num_hits = count($search_ids);
		if (!$num_hits)
			message($lang_search['No hits']);


		if ($show_as == 'topics')
		{
			$result = $db->query('SELECT t.id FROM '.$db->prefix.'posts AS p INNER JOIN '.$db->prefix.'topics AS t ON t.id=p.topic_id INNER JOIN '.$db->prefix.'forums AS f ON f.id=t.forum_id LEFT JOIN '.$db->prefix.'forum_perms AS fp ON (fp.forum_id=f.id AND fp.group_id='.$pun_user['g_id'].') WHERE (fp.read_forum IS NULL OR fp.read_forum=1) AND p.id IN('.implode(',', $search_ids).')'.$forum_sql.' GROUP BY t.id', true) or error(__FILE__, __LINE__);

			$search_ids = array();
			while ($row = $db->fetch_row($result))
				$search_ids[] = $row[0];

			$db->free_result($result);

			$num_hits = count($search_ids);
		}
		else
		{
			$result = $db->query('SELECT p.id FROM '.$db->prefix.'posts AS p INNER JOIN '.$db->prefix.'topics AS t ON t.id=p.topic_id INNER JOIN '.$db->prefix.'forums AS f ON f.id=t.forum_id LEFT JOIN '.$db->prefix.'forum_perms AS fp ON (fp.forum_id=f.id AND fp.group_id='.$pun_user['g_id'].') WHERE (fp.read_forum IS NULL OR fp.read_forum=1) AND p.id IN('.implode(',', $search_ids).')'.$forum_sql, true) or error(__FILE__, __LINE__);

			$search_ids = array();
			while ($row = $db->fetch_row($result))
				$search_ids[] = $row[0];

			$db->free_result($result);

			$num_hits = count($search_ids);
		}

		// Prune "old" search results
		$old_searches = array();
		$result = $db->query('SELECT ident FROM '.$db->prefix.'online') or error(__FILE__, __LINE__);

		if ($db->num_rows($result))
		{
			while ($row = $db->fetch_row($result))
				$old_searches[] = '\''.$db->escape($row[0]).'\'';

			$db->query('DELETE FROM '.$db->prefix.'search_cache WHERE ident NOT IN('.implode(',', $old_searches).')') or error(__FILE__, __LINE__);
		}

		// Final search results
		$search_results = implode(',', $search_ids);

		// Fill an array with our results and search properties
		$temp['search_results'] = $search_results;
		$temp['sort_by'] = $sort_by;
		$temp['sort_dir'] = $sort_dir;
		$temp['show_as'] = $show_as;
		$temp = serialize($temp);
		$search_id = mt_rand(1, 2147483647);

		$ident = ($pun_user['is_guest']) ? get_remote_address() : $pun_user['username'];

		$db->query('INSERT INTO '.$db->prefix.'search_cache (id, ident, search_data) VALUES('.$search_id.', \''.$db->escape($ident).'\', \''.$db->escape($temp).'\')') or error(__FILE__, __LINE__);

		$db->end_transaction();
		$db->close();

		// Redirect the user to the cached result page
		header('Location: '.str_replace('&amp;', '&', pun_link($pun_url['search_results'], $search_id)));
		exit;
	}

	// If we're still running we don't need to cache results but we still need to get them, either from the cache or from their respective sources.

	// We're doing a fulltext search!
	if (($db_type == 'mysql' || $db_type == 'mysqli') && (isset($keywords) || isset($author)))
	{
		// Are we searching topic subjects, or posts, or both?
		if ($show_as == 'posts')
			$search_in_cond = 'MATCH(p.message) AGAINST(\''.$db->escape($keywords).'\')';
		else
			$search_in_cond = '(MATCH(p.message) AGAINST(\''.$db->escape($keywords).'\') OR MATCH(t.subject) AGAINST(\''.$db->escape($keywords).'\'))';

		// Are we limiting the results to a specific forum?
		if ($forum != -1 || ($forum == -1 && $pun_config['o_search_all_forums'] == '0'))
			$forum_where = ' AND f.id = '.$forum;
		else
			$forum_where = '';

		// Sort out WHERE clauses for author and keywords
		if ($keywords && $author)
		{
			$post_sql = 'WHERE (fp.read_forum IS NULL OR fp.read_forum=1) AND p.poster LIKE \''.$db->escape($author).'\' AND '.$search_in_cond.$forum_where;
			$topic_sql = 'FROM '.$db->prefix.'posts AS p LEFT JOIN '.$db->prefix.'topics AS t ON p.topic_id=t.id LEFT JOIN '.$db->prefix.'forums AS f ON f.id=t.forum_id LEFT JOIN '.$db->prefix.'forum_perms AS fp ON (fp.forum_id=f.id AND fp.group_id='.$pun_user['g_id'].') WHERE (fp.read_forum IS NULL OR fp.read_forum=1) AND p.poster LIKE \''.$db->escape($author).'\' AND '.$search_in_cond.$forum_where.' AND p.approved=1 GROUP BY t.id';
		}
		else if ($keywords)
		{
			$post_sql = 'WHERE (fp.read_forum IS NULL OR fp.read_forum=1) AND '.$search_in_cond.$forum_where;
			$topic_sql = 'FROM '.$db->prefix.'topics AS t LEFT JOIN '.$db->prefix.'forums AS f ON f.id=t.forum_id LEFT JOIN '.$db->prefix.'forum_perms AS fp ON (fp.forum_id=f.id AND fp.group_id='.$pun_user['g_id'].') '.(($search_in >= 0) ? 'LEFT JOIN '.$db->prefix.'posts AS p ON p.topic_id=t.id ' : '') . 'WHERE (fp.read_forum IS NULL OR fp.read_forum=1) AND '.$search_in_cond.$forum_where.' AND p.approved=1 GROUP BY t.id';
		}
		else if ($author)
		{
			$post_sql = 'WHERE (fp.read_forum IS NULL OR fp.read_forum=1) AND p.poster LIKE \''.$db->escape($author).'\''.$forum_where;
			$topic_sql = 'FROM '.$db->prefix.'topics AS t LEFT JOIN '.$db->prefix.'forums AS f ON f.id=t.forum_id LEFT JOIN '.$db->prefix.'forum_perms AS fp ON (fp.forum_id=f.id AND fp.group_id='.$pun_user['g_id'].') LEFT JOIN '.$db->prefix.'posts AS p ON p.topic_id=t.id WHERE (fp.read_forum IS NULL OR fp.read_forum=1) AND p.poster LIKE \''.$db->escape($author).'\''.$forum_where.' AND p.approved=1 GROUP BY t.id';
		}

		// Sort out how to order the results
		switch ($sort_by)
		{
			case 1:
				$sort_by_sql = ($show_as == 'topics') ? 't.poster' : 'p.poster';
				break;

			case 2:
				$sort_by_sql = 't.subject';
				break;

			case 3:
				$sort_by_sql = 't.forum_id';
				break;

			case 4:
				$sort_by_sql = 't.last_post';
				break;

			default:
				$sort_by_sql = ($show_as == 'topics') ? 't.posted' : 'p.posted';
				break;
		}

		// Generate the query to give us our results
		if ($show_as == 'posts')
			$sql = 'SELECT p.id AS pid, p.poster AS pposter, p.posted AS pposted, p.poster_id, SUBSTRING(p.message, 1, 1000) AS message, t.id AS tid, t.poster, t.subject, t.first_post_id, t.posted, t.last_post, t.last_post_id, t.last_poster, t.num_replies, t.forum_id, f.forum_name FROM '.$db->prefix.'posts AS p LEFT JOIN '.$db->prefix.'topics AS t ON t.id=p.topic_id LEFT JOIN '.$db->prefix.'forums AS f ON f.id=t.forum_id LEFT JOIN '.$db->prefix.'forum_perms AS fp ON (fp.forum_id=f.id AND fp.group_id='.$pun_user['g_id'].') '.$post_sql.' AND p.approved=1 ORDER BY '.$sort_by_sql.' '.$sort_dir;
		else
			$sql = 'SELECT t.id AS tid, t.poster, t.subject, t.first_post_id, t.last_post, t.last_post_id, t.last_poster, t.num_replies, t.closed, t.forum_id, f.forum_name '.$topic_sql.' ORDER BY '.$sort_by_sql.' '.$sort_dir;

		$url_type = $pun_url['search_resultft'];

		$search_id = array($keywords, $forum, $author, ($search_in == 0 ) ? 'all' : (($search_in == 1) ? 'message' : 'subject'), $sort_by, $sort_dir, $show_as);
	}
	// We aren't doing a fulltext but we are getting results, if a valid search_id was supplied we attempt to fetch the search results from the cache
	else if (isset($search_id))
	{
		$ident = ($pun_user['is_guest']) ? get_remote_address() : $pun_user['username'];

		$result = $db->query('SELECT search_data FROM '.$db->prefix.'search_cache WHERE id='.$search_id.' AND ident=\''.$db->escape($ident).'\'') or error(__FILE__, __LINE__);
		if ($row = $db->fetch_assoc($result))
		{
			$temp = unserialize($row['search_data']);

			$search_results = $temp['search_results'];
			$sort_by = $temp['sort_by'];
			$sort_dir = $temp['sort_dir'];
			$show_as = $temp['show_as'];

			unset($temp);
		}
		else
			message($lang_search['No hits']);

		switch ($sort_by)
		{
			case 1:
				$sort_by_sql = ($show_as == 'topics') ? 't.poster' : 'p.poster';
				break;

			case 2:
				$sort_by_sql = 't.subject';
				break;

			case 3:
				$sort_by_sql = 't.forum_id';
				break;

			case 4:
				$sort_by_sql = 't.last_post';
				break;

			default:
				$sort_by_sql = ($show_as == 'topics') ? 't.posted' : 'p.posted';
				break;
		}

		if ($show_as == 'posts')
		{
			$substr_sql = ($db_type != 'sqlite') ? 'SUBSTRING' : 'SUBSTR';
			$sql = 'SELECT p.id AS pid, p.poster AS pposter, p.posted AS pposted, p.poster_id, '.$substr_sql.'(p.message, 1, 1000) AS message, t.id AS tid, t.poster, t.subject, t.first_post_id, t.posted, t.last_post, t.last_post_id, t.last_poster, t.num_replies, t.forum_id, f.forum_name, p.approved FROM '.$db->prefix.'posts AS p INNER JOIN '.$db->prefix.'topics AS t ON t.id=p.topic_id INNER JOIN '.$db->prefix.'forums AS f ON f.id=t.forum_id WHERE p.id IN('.$search_results.') AND p.approved=1 ORDER BY '.$sort_by_sql;
		}
		else
			$sql = 'SELECT t.id AS tid, t.poster, t.subject, t.first_post_id, t.posted, t.last_post, t.last_post_id, t.last_poster, t.num_replies, t.closed, t.forum_id, f.forum_name FROM '.$db->prefix.'topics AS t LEFT JOIN '.$db->prefix.'posts as p ON p.topic_id=t.id INNER JOIN '.$db->prefix.'forums AS f ON f.id=t.forum_id WHERE t.id IN('.$search_results.') GROUP BY t.id HAVING p.approved=1 ORDER BY p.id, '.$sort_by_sql;

		$url_type = $pun_url['search_results'];
	}
	else if ($action == 'show_new' || $action == 'show_recent' || $action == 'show_user_posts' || $action == 'show_user_topics' || $action == 'show_subscriptions' || $action == 'show_unanswered')
	{
		$search_id = '';
		$show_as = 'topics';
		switch ($action)
		{
			case 'show_new':
				if ($pun_user['is_guest'])
					message($lang_common['No permission']);
				$sql = 'SELECT t.id AS tid, t.poster, t.subject, t.last_post, t.last_post_id, t.last_poster, t.num_replies, t.closed, t.forum_id, f.forum_name, p.approved FROM '.$db->prefix.'topics AS t LEFT JOIN '.$db->prefix.'posts as p ON p.topic_id=t.id INNER JOIN '.$db->prefix.'forums AS f ON f.id=t.forum_id LEFT JOIN '.$db->prefix.'forum_perms AS fp ON (fp.forum_id=f.id AND fp.group_id='.$pun_user['g_id'].') WHERE (fp.read_forum IS NULL OR fp.read_forum=1) AND t.last_post>'.$pun_user['last_visit'].' AND t.moved_to IS NULL GROUP BY t.id HAVING p.approved=1 ORDER BY t.last_post DESC, p.id ASC';
				$url_type = $pun_url['search_new'];
				break;

			case 'show_recent':
				$sql = 'SELECT t.id AS tid, t.poster, t.subject, t.last_post, t.last_post_id, t.last_poster, t.num_replies, t.closed, t.forum_id, f.forum_name FROM '.$db->prefix.'topics AS t LEFT JOIN '.$db->prefix.'posts as p ON p.topic_id=t.id INNER JOIN '.$db->prefix.'forums AS f ON f.id=t.forum_id LEFT JOIN '.$db->prefix.'forum_perms AS fp ON (fp.forum_id=f.id AND fp.group_id='.$pun_user['g_id'].') WHERE (fp.read_forum IS NULL OR fp.read_forum=1) AND p.posted>'.(time() - $value).' AND t.moved_to IS NULL AND p.approved=1 GROUP BY t.id ORDER BY t.last_post DESC';
				$url_type = $pun_url['search_24h'];
				break;

			case 'show_user_posts':
				$sql = 'SELECT p.id AS pid, p.poster AS pposter, p.posted AS pposted, p.poster_id, SUBSTRING(p.message, 1, 1000) AS message, t.id AS tid, t.poster, t.subject, t.first_post_id, t.posted, t.last_post, t.last_post_id, t.last_poster, t.num_replies, t.forum_id, f.forum_name FROM '.$db->prefix.'posts AS p LEFT JOIN '.$db->prefix.'topics AS t ON t.id=p.topic_id LEFT JOIN '.$db->prefix.'forums AS f ON f.id=t.forum_id LEFT JOIN '.$db->prefix.'forum_perms AS fp ON (fp.forum_id=f.id AND fp.group_id='.$pun_user['g_id'].') WHERE (fp.read_forum IS NULL OR fp.read_forum=1) AND p.poster_id='.$user_id.' AND p.approved=1 ORDER BY pposted DESC';
				$url_type = $pun_url['search_user_posts'];
				$search_id = $user_id;
				$show_as = 'posts';
				break;

			case 'show_user_topics':
				$sql = 'SELECT t.id AS tid, t.poster, t.subject, t.last_post, t.last_post_id, t.last_poster, t.num_replies, t.closed, t.forum_id, f.forum_name FROM '.$db->prefix.'topics AS t LEFT JOIN '.$db->prefix.'posts AS p ON (t.first_post_id=p.id) INNER JOIN '.$db->prefix.'forums AS f ON f.id=t.forum_id LEFT JOIN '.$db->prefix.'forum_perms AS fp ON (fp.forum_id=f.id AND fp.group_id='.$pun_user['g_id'].') WHERE (fp.read_forum IS NULL OR fp.read_forum=1) AND p.poster_id='.$user_id.' AND p.approved=1 ORDER BY t.last_post DESC';
				$url_type = $pun_url['search_user_topics'];
				$search_id = $user_id;
				break;

			case 'show_subscriptions':
				if ($pun_user['is_guest'])
					message($lang_common['Bad request']);
				$sql = 'SELECT t.id AS tid, t.poster, t.subject, t.last_post, t.last_post_id, t.last_poster, t.num_replies, t.closed, t.forum_id, f.forum_name FROM '.$db->prefix.'topics AS t INNER JOIN '.$db->prefix.'subscriptions AS s ON (t.id=s.topic_id AND s.user_id='.$pun_user['id'].') INNER JOIN '.$db->prefix.'forums AS f ON f.id=t.forum_id LEFT JOIN '.$db->prefix.'forum_perms AS fp ON (fp.forum_id=f.id AND fp.group_id='.$pun_user['g_id'].') WHERE (fp.read_forum IS NULL OR fp.read_forum=1) ORDER BY t.last_post DESC';
				$url_type = $pun_url['search_subscriptions'];
				break;

			case 'show_unanswered':
			default:
				$sql = 'SELECT t.id AS tid, t.poster, t.subject, t.last_post, t.last_post_id, t.last_poster, t.num_replies, t.closed, t.forum_id, f.forum_name FROM '.$db->prefix.'topics AS t LEFT JOIN '.$db->prefix.'posts AS p ON t.id=p.topic_id INNER JOIN '.$db->prefix.'forums AS f ON f.id=t.forum_id LEFT JOIN '.$db->prefix.'forum_perms AS fp ON (fp.forum_id=f.id AND fp.group_id='.$pun_user['g_id'].') WHERE (fp.read_forum IS NULL OR fp.read_forum=1) AND t.num_replies=0 AND t.moved_to IS NULL AND p.approved=1 GROUP BY t.id ORDER BY t.last_post DESC';
				$url_type = $pun_url['search_unanswered'];
		}
	}
	else
		message($lang_common['Bad request']);


	// We now have a query that will give us our results in $sql, lets get the data!
	$result = $db->query($sql) or error(__FILE__, __LINE__);

	// Make sure we actually have some results
	$num_hits = $db->num_rows($result);
	if ($num_hits == 0)
	{
		switch ($action)
		{
			case 'show_new':
				message($lang_search['No new posts']);

			case 'show_recent':
				message($lang_search['No recent posts']);

			case 'show_user_posts':
				message($lang_search['No user posts']);

			case 'show_user_topics':
				message($lang_search['No user topics']);

			case 'show_subscriptions':
				message($lang_search['No subscriptions']);

			case 'show_unanswered':
				message($lang_search['No unanswered']);

			default:
				message($lang_search['No hits']);
		}
	}

	//$db->free_result($result);

	// Get topic/forum tracking data
	if (!$pun_user['is_guest'])
		$tracked_topics = get_tracked_topics();

	// Determine the topic or post offset (based on $_GET['p'])
	$per_page = ($show_as == 'posts') ? $pun_user['disp_posts'] : $pun_user['disp_topics'];
	$num_pages = ceil($num_hits / $per_page);

	$p = (!isset($_GET['p']) || $_GET['p'] <= 1 || $_GET['p'] > $num_pages) ? 1 : $_GET['p'];
	$start_from = $per_page * ($p - 1);
	$finish_at = min(($start_from + $per_page), $num_hits);

	// Generate paging links
	$page_links = "\t\t".'<p class="pagelinks"><strong>'.$lang_common['Pages'].'</strong> '.paginate($num_pages, $p, $url_type, $search_id).'</p>'."\n";


	$sql .= ' '.$sort_dir.' LIMIT '.$start_from.', '.$per_page;

	//$result = $db->query($sql) or error(__FILE__, __LINE__);

	// Ok now we've collected the data we need, lets display it
	$search_set = array();
	$row_num = 0;
	while ($row = $db->fetch_assoc($result))
	{
		if ($start_from <= $row_num && $finish_at > $row_num)
			$search_set[] = $row;
		$row_num++;
	}

	$db->free_result($result);


	// Navigation links for header and page numbering for title/meta description
	if ($p < $num_pages)
	{
		$pun_page['nav'][] = '<link rel="last" href="'.pun_sublink($url_type, $pun_url['page'], $num_pages, $search_id).'" title="'.$lang_common['Page'].' '.$num_pages.'" />';
		$pun_page['nav'][] = '<link rel="next" href="'.pun_sublink($url_type, $pun_url['page'], ($p + 1), $search_id).'" title="'.$lang_common['Page'].' '.($p + 1).'" />';
	}
	if ($p > 1)
	{
		$pun_page['nav'][] = '<link rel="prev" href="'.pun_sublink($url_type, $pun_url['page'], ($p - 1), $search_id).'" title="'.$lang_common['Page'].' '.($p - 1).'" />';
		$pun_page['nav'][] = '<link rel="first" href="'.pun_link($url_type, $search_id).'" title="'.$lang_common['Page'].' 1" />';
		$pun_page['page'] = $p;
	}

	// Setup breadcrumbs and results header and footer
	$paged_options[] = '<a href="'.pun_link($pun_url['search']).'">'.$lang_search['Perform new search'].'</a>';

	$pun_page['crumbs'][] = array($pun_config['o_board_title'], pun_link($pun_url['index']));
	if ($action == 'show_new')
	{
		$pun_page['crumbs'][] = $lang_common['New posts'];
		$results_selection = $lang_search['New selection'].' '.sprintf($lang_search['Results selection'], $start_from + 1, $finish_at, $num_hits);
		$paged_options[] = '<a href="'.pun_link($pun_url['mark_read']).'">'.$lang_common['Mark all as read'].'</a>';
	}
	else if ($action == 'show_recent')
	{
		$pun_page['crumbs'][] = $lang_common['Show recent posts'];
		$results_selection = $lang_search['Recent selection'].' '.sprintf($lang_search['Results selection'], $start_from + 1, $finish_at, $num_hits);
	}
	else if ($action == 'show_unanswered')
	{
		$pun_page['crumbs'][] = $lang_common['Show unanswered posts'];
		$results_selection = $lang_common['Show unanswered posts'].' '.sprintf($lang_search['Results selection'], $start_from + 1, $finish_at, $num_hits);
	}
	else if ($action == 'show_user_posts')
	{
		$pun_page['crumbs'][] = sprintf($lang_search['Posts by'], $search_set[0]['pposter']);
		$results_selection = sprintf($lang_search['Posts by'], $search_set[0]['pposter']).' '.sprintf($lang_search['Results selection'], $start_from + 1, $finish_at, $num_hits);
		$paged_options[] = '<a href="'.pun_link($pun_url['search_user_topics'], $search_id).'">'.sprintf($lang_search['Topics by'], $search_set[0]['pposter']).'</a>';
	}
	else if ($action == 'show_user_topics')
	{
		$pun_page['crumbs'][] = sprintf($lang_search['Topics by'], $search_set[0]['poster']);
		$results_selection = sprintf($lang_search['Topics by'], $search_set[0]['poster']).' '.sprintf($lang_search['Results selection'], $start_from + 1, $finish_at, $num_hits);
		$paged_options[] = '<a href="'.pun_link($pun_url['search_user_posts'], $search_id).'">'.sprintf($lang_search['Posts by'], $search_set[0]['poster']).'</a>';

	}
	else if ($action == 'show_subscriptions')
	{
		$pun_page['crumbs'][] = $lang_common['Show subscriptions'];
		$results_selection = $lang_search['Subscribed selection'].' '.sprintf($lang_search['Results selection'], $start_from + 1, $finish_at, $num_hits);
	}
	else
	{
		$pun_page['crumbs'][] = $lang_search['Search results'];
		$results_selection = (($show_as == 'topics') ? $lang_search['Topics selection'] : $lang_search['Posts selection']).' '.sprintf($lang_search['Results selection'], $start_from + 1, $finish_at, $num_hits);
	}

	$paged_options = '<p>'.implode(' ', $paged_options).'</p>';
	$page_selection = '<strong class="page-selection">'.sprintf($lang_search['Page selection'], $p, $num_pages).'</strong>';

	define('PUN_PAGE', $show_as == 'topics' ? 'searchtopics' : 'searchposts');
	require PUN_ROOT.'header.php';

	if ($show_as == 'topics')
	{

?>
<div id="pun-main" class="main paged">

	<h1><span><?php echo end($pun_page['crumbs']) ?></span></h1>

	<div class="paged-head">
<?php echo $page_links ?>
	</div>

	<div class="main-forum">
		<h2><span><?php if(isset($page_selection)) echo $page_selection ?>	<?php echo $results_selection ?></span></h2>
		<table cellspacing="0" summary="<?php echo $lang_search['Table summary'] ?>">
			<thead>
				<tr>
					<th class="tcl" scope="col"><?php echo $lang_common['Topic']; ?></th>
					<th class="tc2" scope="col"><?php echo $lang_common['Forum'] ?></th>
					<th class="tc3" scope="col"><?php echo $lang_common['Replies'] ?></th>
					<th class="tcr" scope="col"><?php echo $lang_common['Last post'] ?></th>
				</tr>
			</thead>
			<tbody class="hasicon">
<?php

	}

	if ($show_as == 'posts')
	{

?>
<div id="pun-main" class="main paged">

	<h1><span><?php echo end($pun_page['crumbs']) ?></span></h1>

	<div class="paged-head">
<?php echo $page_links ?>
	</div>

	<div class="main-topic">
		<h2><span><?php if(isset($page_selection)) echo $page_selection ?>	<?php echo $results_selection ?></span></h2>
<?php

	}

	$row_count = $start_from;

	// Finally, lets loop through the results and output them
	for ($i = 0; $i < count($search_set); ++$i)
	{
		$forum = '<a href="'.pun_link($pun_url['forum'], $search_set[$i]['forum_id']).'">'.htmlspecialchars($search_set[$i]['forum_name']).'</a>';

		if ($pun_config['o_censoring'] == '1')
			$search_set[$i]['subject'] = censor_words($search_set[$i]['subject']);

		if ($show_as == 'posts')
		{
			$post_head = array('<strong><a class="permalink" rel="bookmark" title="'.$lang_search['Permalink post'].'" href="'.pun_link($pun_url['post'], $search_set[$i]['pid']).'">'.$lang_search['Go to post'].' '.($start_from +($i + 1)).'</a></strong>');

			$subject = '<a class="permalink" rel="bookmark" title="'.$lang_search['Permalink topic'].'" href="'.pun_link($pun_url['topic'], $search_set[$i]['tid']).'">'.htmlspecialchars($search_set[$i]['subject']).'</a>';

			if ($pun_config['o_censoring'] == '1')
				$search_set[$i]['message'] = censor_words($search_set[$i]['message']);

			$message = str_replace("\n", '<br />', htmlspecialchars($search_set[$i]['message']));

			if ($search_set[$i]['poster_id'] > 1)
				$post_head[] ='<cite><span>'.(($search_set[$i]['posted'] == $search_set[$i]['pposted']) ? $lang_search['Topic by'] : $lang_search['Reply by']).'</span> <a title="'.sprintf($lang_search['Go to profile'], htmlspecialchars($search_set[$i]['pposter'])).'" href="'.pun_link($pun_url['user'], $search_set[$i]['poster_id']).'">'.htmlspecialchars($search_set[$i]['pposter']).'</a></cite>';

			$post_head[] = '<a class="permalink" rel="bookmark" title="'.$lang_search['Permalink post'].'" href="'.pun_link($pun_url['post'], $search_set[$i]['pid']).'">'.format_time($search_set[$i]['pposted']).'</a>';

			if (pun_strlen($message) >= 1000)
				$message .= ' …';

?>
		<div id="<?php echo (($i + 1) == 1) ? 'postfirst' : 'post'.($i + 1) ?>" class="post<?php if (($i + 1) % 2 == 0) echo ' altstyle'; ?>">
			<div class="posthead">
				<h3><?php echo implode(' ', $post_head) ?></h3>
			</div>
			<div class="postmain">
				<div class="postbody">
					<div class="post-entry">
						<ul class="post-searchlinks">
							<li><strong><?php echo $lang_common['Forum'] ?></strong>: <span><?php echo $forum ?></span></li>
							<li><strong><?php echo $lang_common['Topic'] ?></strong> (<?php echo $search_set[$i]['num_replies'].' '.strtolower($lang_common['Replies']) ?>): <span class="entry-title"><?php echo $subject ?></span></li>
						</ul>
						<div class="entry-content">
						<p><?php echo $message ?></p>
						</div>
					</div>
				</div>
			</div>
		</div>
<?php

		}
		else
		{
			++$row_count;
			$row_style = ($row_count % 2 != 0) ? 'altstyle' : '';
			$icon_type = 'icon';

			$subject = '<a href="'.pun_link($pun_url['topic'], $search_set[$i]['tid']).'">'.htmlspecialchars($search_set[$i]['subject']).'</a> <span class="byuser">'.$lang_common['by'].'&#160;'.htmlspecialchars($search_set[$i]['poster']).'</span>';

			if ($search_set[$i]['closed'] != '0')
			{
				$subject = '<span class="closedtopic">'.$lang_common['Closed'].'</span> <a href="'.pun_link($pun_url['topic'], $search_set[$i]['tid']).'"><span class="acchide">'.$row_count.'&#160;</span>'.htmlspecialchars($search_set[$i]['subject']).'</a> <span class="byuser">'.$lang_common['by'].'&#160;'.htmlspecialchars($search_set[$i]['poster']).'</span>';
				$row_style .= ' iclosed ';
			}

			if (!$pun_user['is_guest'] && $search_set[$i]['last_post'] > $pun_user['last_visit'] && (!isset($tracked_topics[$search_set[$i]['tid']]) || $tracked_topics[$search_set[$i]['tid']] < $search_set[$i]['last_post']))
			{
				$row_style .= ' inew';
				$icon_type = 'icon inew';
				$subject_new_posts = '<span class="newposts">[&#160;<a href="'.pun_link($pun_url['topic_new_posts'], $search_set[$i]['tid']).'" title="'.$lang_common['New posts info'].'">'.$lang_common['New posts'].'</a>&#160;]</span>';
			}
			else
				$subject_new_posts = null;

			$num_pages_topic = ceil(($search_set[$i]['num_replies'] + 1) / $pun_user['disp_posts']);

			if ($num_pages_topic > 1)
				$subject_multipage = '[&#160;<span class="acchide">'.$lang_common['Pages'].':&#160;</span>'.paginate($num_pages_topic, -1, $pun_url['topic'], $search_set[$i]['tid']).'&#160;]';
			else
				$subject_multipage = null;

			// Should we show the "New posts" and/or the multipage links?
			if (!empty($subject_new_posts) || !empty($subject_multipage))
			{
				$subject .= '&#160; '.(!empty($subject_new_posts) ? $subject_new_posts : '');
				$subject .= !empty($subject_multipage) ? ' '.$subject_multipage : '';
			}

?>
				<tr<?php if ($row_style != '') echo ' class="'.trim($row_style).'"' ?>>
					<td class="tcl"><div class="<?php echo $icon_type ?>"><!-- --></div><?php echo '<span class="acchide">'.$row_count.'</span> '.$subject ?></td>
					<td class="tc2"><?php echo $forum ?></td>
					<td class="tc3"><?php echo $search_set[$i]['num_replies'] ?></td>
					<td class="tcr"><?php echo '<a href="'.pun_link($pun_url['post'], $search_set[$i]['last_post_id']).'">'.format_time($search_set[$i]['last_post']).'</a> '.$lang_common['by'].'&#160;'.htmlspecialchars($search_set[$i]['last_poster']) ?></td>
				</tr>
<?php

		}
	}

	if ($show_as == 'topics')
	{

?>
			</tbody>
		</table>
<?php

	}

?>
	</div>

	<div class="paged-options">
<?php echo $paged_options ?>
	</div>

	<div class="paged-foot">
<?php echo $page_links ?>
	</div>

</div>

<div class="pun-crumbs">
	<p class="crumbs"><?php echo generate_crumbs(false) ?></p>
</div>
<?php

	require PUN_ROOT.'footer.php';
}

// Setup quick search list
if (!$pun_user['is_guest'])
	$qs_links[] = '<a href="'.pun_link($pun_url['search_new']).'" title="'.$lang_common['Show new posts info'].'">'.$lang_common['New posts'].'</a>';

$qs_links[] = '<a href="'.pun_link($pun_url['search_24h']).'">'.$lang_common['Show recent posts'].'</a>';
$qs_links[] = '<a href="'.pun_link($pun_url['search_unanswered']).'">'.$lang_common['Show unanswered posts'].'</a>';

if (!$pun_user['is_guest'])
{
	$qs_links[] = '<a href="'.pun_link($pun_url['search_user_posts'], $pun_user['id']).'">'.$lang_common['Show your posts'].'</a>';
	$qs_links[] = '<a href="'.pun_link($pun_url['search_user_topics'], $pun_user['id']).'">'.$lang_common['Show your topics'].'</a>';

	if ($pun_config['o_subscriptions'] == '1')
		$qs_links[] = '<a href="'.pun_link($pun_url['search_subscriptions']).'">'.$lang_common['Show subscriptions'].'</a>';
}

// Setup breadcrumbs
$pun_page['crumbs'] = array(
	array($pun_config['o_board_title'], pun_link($pun_url['index'])),
	$lang_common['Search']
);

define('PUN_PAGE', 'search');
require PUN_ROOT.'header.php';

$set_count = 0;
$fld_count = 0;

// Setup form help/instructions
$search_info[] = '<li class="actions" id="qs-links"><div><strong>'.$lang_search['Predefined searches'].' </strong>'.implode(' ',$qs_links).'</div></li>';
$search_info[] = '<li>'.$lang_search['Search info'].'</li>';
$search_info[] = '<li>'.$lang_search['Refine info'].'</li>';
$search_info[] = '<li>'.$lang_search['Wildcard info'].'</li>';

?>
<div id="pun-main" class="main">

	<h1><span><?php echo $lang_common['Search'] ?></span></h1>

	<div class="main-form">
		<h2><span><?php echo $lang_search['Search heading'] ?></span></h2>
		<form id="afocus" class="fm-form" method="get" accept-charset="utf-8" action="<?php echo pun_link($pun_url['search']) ?>">
			<div class="hidden">
				<input type="hidden" name="action" value="search" />
			</div>
			<div class="form-info">
				<ul>
					<?php echo implode("\n\t\t\t\t\t", $search_info)."\n" ?>
				</ul>
			</div>
			<fieldset class="fm-set set<?php echo ++$set_count ?>">
				<legend class="fm-legend"><strong><?php echo $lang_search['Search legend'] ?></strong></legend>
				<div class="text">
					<label for="fld<?php echo ++$fld_count ?>">
						<span class="fm-label"><?php echo $lang_search['Keyword search'] ?></span><br />
						<span class="fm-input"><input type="text" id="fld<?php echo $fld_count ?>" name="keywords" size="40" maxlength="100" /></span><br />
						<small><?php echo $lang_search['Keyword info'] ?></small>
					</label>
				</div>
				<div class="text">
					<label for="fld<?php echo ++$fld_count ?>">
						<span class="fm-label"><?php echo $lang_search['Author search'] ?></span><br />
						<span class="fm-input"><input id="fld<?php echo $fld_count ?>" type="text" name="author" size="25" maxlength="25" /></span><br />
						<small><?php echo $lang_search['Author info'] ?></small>
					</label>
				</div>
				<div class="select">
					<label for="fld<?php echo ++$fld_count ?>">
						<span class="fm-label"><?php echo $lang_search['Forum search'] ?></span><br />
						<span class="fm-input"><select id="fld<?php echo $fld_count ?>" name="forum">
<?php

if ($pun_config['o_search_all_forums'] == '1' || $pun_user['is_admmod'])
	echo "	\t\t\t\t\t".'<option value="-1">'.$lang_search['All forums'].'</option>'."\n";

// Get the list of categories and forums
$query = array(
	'SELECT'	=> 'c.id AS cid, c.cat_name, f.id AS fid, f.forum_name, f.redirect_url',
	'FROM'		=> 'categories AS c',
	'JOINS'		=> array(
		array(
			'INNER JOIN'	=> 'forums AS f',
			'ON'			=> 'c.id=f.cat_id'
		),
		array(
			'LEFT JOIN'		=> 'forum_perms AS fp',
			'ON'			=> '(fp.forum_id=f.id AND fp.group_id='.$pun_user['g_id'].')'
		)
	),
	'WHERE'		=> '(fp.read_forum IS NULL OR fp.read_forum=1) AND f.redirect_url IS NULL',
	'ORDER BY'	=> 'c.disp_position, c.id, f.disp_position'
);

($hook = get_hook('se_qr_get_cats_and_forums')) ? eval($hook) : null;
$result = $db->query_build($query, true) or error(__FILE__, __LINE__);

$cur_category = 0;
while ($cur_forum = $db->fetch_assoc($result))
{
	if ($cur_forum['cid'] != $cur_category)	// A new category since last iteration?
	{
		if ($cur_category)
			echo "\t\t\t\t\t\t".'</optgroup>'."\n";

		echo "\t\t\t\t\t\t".'<optgroup label="'.htmlspecialchars($cur_forum['cat_name']).'">'."\n";
		$cur_category = $cur_forum['cid'];
	}

	echo "\t\t\t\t\t\t".'<option value="'.$cur_forum['fid'].'">'.htmlspecialchars($cur_forum['forum_name']).'</option>'."\n";
}

?>
						</optgroup>
						</select></span><br />
					</label>
				</div>
			</fieldset>
			<fieldset class="fm-set set<?php echo ++$set_count ?>">
				<legend class="fm-legend"><strong><?php echo $lang_search['Results legend'] ?></strong></legend>
				<div class="select">
					<label for="fld<?php echo ++$fld_count ?>">
						<span class="fm-label"><?php echo $lang_search['Sort by'] ?></span><br />
						<span class="fm-input"><select id="fld<?php echo $fld_count ?>" name="sort_by">
						<option value="0"><?php echo $lang_search['Sort by post time'] ?></option>
						<option value="1"><?php echo $lang_search['Sort by author'] ?></option>
						<option value="2"><?php echo $lang_search['Sort by subject'] ?></option>
						<option value="3"><?php echo $lang_search['Sort by forum'] ?></option>
						<option value="4"><?php echo $lang_search['Sort by relevance'] ?></option>
						</select></span><br />
					</label>
				</div>
				<fieldset class="group">
					<legend><span><?php echo $lang_search['Sort order'] ?></span></legend>
					<div class="fm-item"><label for="fld<?php echo ++$fld_count ?>"><input type="radio" id="fld<?php echo $fld_count ?>" name="sort_dir" value="DESC" /> <?php echo $lang_search['Descending'] ?></label></div>
					<div class="fm-item"><label for="fld<?php echo ++$fld_count ?>"><input type="radio" id="fld<?php echo $fld_count ?>" name="sort_dir" value="ASC" checked="checked" /> <?php echo $lang_search['Ascending'] ?></label></div>
				</fieldset>
				<fieldset class="group">
					<legend><span><?php echo $lang_search['Show as'] ?></span></legend>
					<div class="fm-item"><label for="fld<?php echo ++$fld_count ?>"><input type="radio" id="fld<?php echo $fld_count ?>" name="show_as" value="topics" checked="checked" /> <?php echo $lang_search['Show as topics'] ?></label></div>
					<div class="fm-item"><label for="fld<?php echo ++$fld_count ?>"><input type="radio" id="fld<?php echo $fld_count ?>" name="show_as" value="posts" /> <?php echo $lang_search['Show as posts'] ?></label></div>
				</fieldset>
			</fieldset>
			<div class="form-buttons">
				<span class="submit"><input type="submit" name="search" value="<?php echo $lang_search['Submit search'] ?>" accesskey="s" title="<?php echo $lang_common['Submit title'] ?>" /></span>
			</div>
		</form>
	</div>

</div>
<?php

require PUN_ROOT.'footer.php';