<?php
/***********************************************************************

  Copyright (C) 2002-2007  Rickard Andersson (rickard@punbb.org)

  This file is part of PunBB.

  PunBB is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 2 of the License,
  or (at your option) any later version.

  PunBB is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
  MA  02111-1307  USA

************************************************************************/


define('PUN_ROOT', './');
require PUN_ROOT.'include/common.php';


if ($pun_user['g_read_board'] == '0')
	message($lang_common['No view']);
else if ($pun_user['g_view_users'] == '0')
	message($lang_common['No permission']);

// Load the userlist.php language file
require PUN_ROOT.'lang/'.$pun_user['language'].'/userlist.php';

// Load the search.php language file
require PUN_ROOT.'lang/'.$pun_user['language'].'/search.php';


// Determine if we are allowed to view post counts
$show_post_count = ($pun_config['o_show_post_count'] == '1' || $pun_user['is_admmod']) ? true : false;

$username = (isset($_GET['username']) && $_GET['username'] != '-' && $pun_user['g_search_users'] == '1') ? $_GET['username'] : '';
$show_group = (!isset($_GET['show_group']) || intval($_GET['show_group']) < -1 && intval($_GET['show_group']) > 2) ? -1 : intval($_GET['show_group']);
$sort_by = (!isset($_GET['sort_by']) || $_GET['sort_by'] != 'username' && $_GET['sort_by'] != 'registered' && ($_GET['sort_by'] != 'num_posts' || !$show_post_count)) ? 'username' : $_GET['sort_by'];
$sort_dir = (!isset($_GET['sort_dir']) || $_GET['sort_dir'] != 'ASC' && $_GET['sort_dir'] != 'DESC') ? 'ASC' : strtoupper($_GET['sort_dir']);


// Create any SQL for the WHERE clause
$where_sql = array();
$like_command = ($db_type == 'pgsql') ? 'ILIKE' : 'LIKE';

if ($pun_user['g_search_users'] == '1' && $username != '')
	$where_sql[] = 'u.username '.$like_command.' \''.$db->escape(str_replace('*', '%', $username)).'\'';
if ($show_group > -1)
	$where_sql[] = 'u.group_id='.$show_group;


// Fetch user count
$query = array(
	'SELECT'	=> 'COUNT(u.id)',
	'FROM'		=> 'users AS u',
	'WHERE'		=> 'u.id>1'
);

if (!empty($where_sql))
	$query['WHERE'] .= ' AND '.implode(' AND ', $where_sql);

($hook = get_hook('ul_qr_get_user_count')) ? eval($hook) : null;
$result = $db->query_build($query) or error(__FILE__, __LINE__);
$num_users = $db->result($result);

// Determine the user offset (based on $_GET['p'])
$num_pages = ceil($num_users / 50);

$p = (!isset($_GET['p']) || $_GET['p'] <= 1 || $_GET['p'] > $num_pages) ? 1 : $_GET['p'];
$start_from = 50 * ($p - 1);

// Generate paging links
$main_links = '<div class="pagelinks">'.$lang_common['Pages'].': '.paginate($num_pages, $p, $pun_url['users_browse'], array($show_group, $sort_by, strtoupper($sort_dir), ($username != '') ? urlencode($username) : '-')).'</div>';

// Navigation links for header and page numbering for title/meta description
if ($p < $num_pages)
{
	$pun_page['nav'][] = '<link rel="last" href="'.pun_sublink($pun_url['users_browse'], $pun_url['page'], $num_pages, array($show_group, $sort_by, strtoupper($sort_dir), ($username != '') ? urlencode($username) : '-')).'" title="'.$lang_common['Page'].' '.$num_pages.'" />';
	$pun_page['nav'][] = '<link rel="next" href="'.pun_sublink($pun_url['users_browse'], $pun_url['page'], ($p + 1), array($show_group, $sort_by, strtoupper($sort_dir), ($username != '') ? urlencode($username) : '-')).'" title="'.$lang_common['Page'].' '.($p + 1).'" />';
}
if ($p > 1)
{
	$pun_page['nav'][] = '<link rel="prev" href="'.pun_sublink($pun_url['users_browse'], $pun_url['page'], ($p - 1), array($show_group, $sort_by, strtoupper($sort_dir), ($username != '') ? urlencode($username) : '-')).'" title="'.$lang_common['Page'].' '.($p - 1).'" />';
	$pun_page['nav'][] = '<link rel="first" href="'.pun_link($pun_url['users_browse'], array($show_group, $sort_by, strtoupper($sort_dir), ($username != '') ? urlencode($username) : '-')).'" title="'.$lang_common['Page'].' 1" />';
	$pun_page['page'] = $p;
}

// Setup breadcrumbs
$pun_page['crumbs'] = array(
	array($pun_config['o_board_title'], pun_link($pun_url['index'])), $lang_common['User list']
);

define('PUN_ALLOW_INDEX', 1);
define('PUN_PAGE', 'userlist');
require PUN_ROOT.'header.php';

$fld_count = 0;

?>
<div id="pun-main" class="a-section a-main main paged">

	<h1><span><?php echo end($pun_page['crumbs']) ?></span></h1>

	<div class="paged-head">
		<?php echo $main_links."\n" ?>
	</div>

	<div class="b-section b-dataform">
		<div class="c-section c-form">
			<form id="afocus" method="get" accept-charset="utf-8" action="<?php echo $base_url ?>/userlist.php">
			<div class="panel">
				<div class="form-intro">
					<p><?php echo $lang_ul['User search info'] ?></p>
				</div>
				<fieldset class="textset">
					<legend><span><?php echo $lang_ul['User find legend'] ?></span></legend>
<?php if ($pun_user['g_search_users'] == '1'): ?>					<p class="field">
						<label for="fld<?php echo ++$fld_count ?>"><?php echo $lang_ul['Search for username'] ?></label><br />
						<span class="input"><input type="text" id="fld<?php echo $fld_count ?>" name="username" value="<?php echo htmlspecialchars($username) ?>" size="25" maxlength="25" /></span>
					</p>
<?php endif; ?>					<p class="field">
						<label for="fld<?php echo ++$fld_count ?>"><?php echo $lang_ul['User group'] ?></label><br />
						<span class="input"><select id="fld<?php echo $fld_count ?>" name="show_group">
						<option value="-1"<?php if ($show_group == -1) echo ' selected="selected"' ?>><?php echo $lang_ul['All users'] ?></option>
<?php

// Get the list of user groups (excluding the guest group)
$query = array(
	'SELECT'	=> 'g.g_id, g.g_title',
	'FROM'		=> 'groups AS g',
	'WHERE'		=> 'g.g_id!='.PUN_GUEST,
	'ORDER BY'	=> 'g.g_id'
);

($hook = get_hook('ul_qr_get_groups')) ? eval($hook) : null;
$result = $db->query_build($query) or error(__FILE__, __LINE__);

while ($cur_group = $db->fetch_assoc($result))
{
	if ($cur_group['g_id'] == $show_group)
		echo "\t\t\t\t\t\t\t".'<option value="'.$cur_group['g_id'].'" selected="selected">'.htmlspecialchars($cur_group['g_title']).'</option>'."\n";
	else
		echo "\t\t\t\t\t\t\t".'<option value="'.$cur_group['g_id'].'">'.htmlspecialchars($cur_group['g_title']).'</option>'."\n";
}

?>
						</select></span>
					</p>
					<p class="field">
						<label for="fld<?php echo ++$fld_count ?>"><?php echo $lang_search['Sort by'] ?></label><br />
						<span class="input"><select id="fld<?php echo $fld_count ?>" name="sort_by">
						<option value="username"<?php if ($sort_by == 'username') echo ' selected="selected"' ?>><?php echo $lang_common['Username'] ?></option>
						<option value="registered"<?php if ($sort_by == 'registered') echo ' selected="selected"' ?>><?php echo $lang_common['Registered'] ?></option>
<?php if ($show_post_count): ?>						<option value="num_posts"<?php if ($sort_by == 'num_posts') echo ' selected="selected"' ?>><?php echo $lang_ul['No of posts'] ?></option>
<?php endif; ?>						</select></span>
					</p>
					<p class="field">
						<label for="fld<?php echo ++$fld_count ?>"><?php echo $lang_search['Sort order'] ?></label><br />
						<span class="input"><select id="fld<?php echo $fld_count ?>" name="sort_dir">
						<option value="ASC"<?php if ($sort_dir == 'ASC') echo ' selected="selected"' ?>><?php echo $lang_search['Ascending'] ?></option>
						<option value="DESC"<?php if ($sort_dir == 'DESC') echo ' selected="selected"' ?>><?php echo $lang_search['Descending'] ?></option>
						</select></span>
					</p>
				</fieldset>
				<p class="formsubmit"><input type="submit" class="button" name="search" value="<?php echo $lang_common['Submit'] ?>" accesskey="s" title="<?php echo $lang_common['Submit title'] ?>" /></p>
			</div>
			<div class="panel subpanel">
<?php

// Grab the users
$query = array(
	'SELECT'	=> 'u.id, u.username, u.title, u.num_posts, u.registered, g.g_id, g.g_user_title',
	'FROM'		=> 'users AS u',
	'JOINS'		=> array(
		array(
			'LEFT JOIN'		=> 'groups AS g',
			'ON'			=> 'g.g_id=u.group_id'
		)
	),
	'WHERE'		=> 'u.id>1',
	'ORDER BY'	=> $sort_by.' '.$sort_dir,
	'LIMIT'		=> $start_from.', 50'
);

if (!empty($where_sql))
	$query['WHERE'] .= ' AND '.implode(' AND ', $where_sql);

($hook = get_hook('ul_qr_get_users')) ? eval($hook) : null;
$result = $db->query_build($query) or error(__FILE__, __LINE__);
$row_count = 0;

if ($db->num_rows($result))
{

?>
				<table cellspacing="0" summary="<?php echo $lang_ul['Table summary'] ?>">
				<caption class="field"><strong class="alabel"><?php echo ($pun_user['g_search_users'] == '1' && $username != '') || ($show_group > -1) ? $lang_ul['Number of users found'] : $lang_ul['Number of users'] ?> <?php echo ' ('.$num_users.')' ?></strong></caption>
				<thead>
					<tr>
						<th class="tcl" scope="col"><?php echo $lang_common['Username'] ?></th>
						<th class="tc2" scope="col"><?php echo $lang_common['Title'] ?></th>
<?php if ($show_post_count): ?>						<th class="tc3" scope="col"><?php echo $lang_common['Posts'] ?></th>
<?php endif; ?>						<th class="tcr" scope="col"><?php echo $lang_common['Registered'] ?></th>
					</tr>
				</thead>
				<tbody>
<?php

	while ($user_data = $db->fetch_assoc($result))
	{
		++$row_count;

?>
					<tr<?php if ($row_count % 2 != 0) echo ' class="altstyle"' ?>>
						<td class="tcl"><a href="<?php echo pun_link($pun_url['user'], $user_data['id']) ?>"><?php echo htmlspecialchars($user_data['username']) ?></a></td>
						<td class="tc2"><?php echo get_title($user_data) ?></td>
<?php if ($show_post_count): ?>						<td class="tc3"><?php echo $user_data['num_posts'] ?></td>
<?php endif; ?>						<td class="tcr"><?php echo format_time($user_data['registered'], true) ?></td>
					</tr>
<?php

	}

?>
				</tbody>
				</table>
<?php

}
else
	echo "\t\t\t\t".'<p><strong>'.$lang_ul['No users found'].'</strong></p>'."\n";

?>
			</div>
			</form>
		</div>
	</div>

	<div class="paged-foot">
		<?php echo $main_links."\n" ?>
	</div>

</div>
<?php

require PUN_ROOT.'footer.php';
